#include "Genetic.h"

void Genetic::run(int maxIter , int maxIterNonProd)
{
	params->indexProgressReport = 0;
	int nbIterNonProd = 0 ;
	int nbIter = 0 ;
	double bestSolutionCost = 1.e30;
	if (population->getBestFeasible() != NULL) 
		bestSolutionCost = population->getBestFeasible()->myCostSol.penalizedCost;

	while (nbIter < maxIter && nbIterNonProd < maxIterNonProd && (clock() - params->startTime <= ticks))
	{	
		// CROSSOVER
		crossoverOX(offspring, population->getBinaryTournament(),population->getBinaryTournament()) ;

		// LOCAL SEARCH
		localSearch->run(offspring);
		population->addIndividual(offspring,true);

		// 50% CHANCES REPAIR IN CASE OF INFEASIBILITY
		if (!offspring->isFeasible && std::rand() % 2 == 0)
		{
			localSearch->repair(offspring);
			if (offspring->isFeasible) // Add in the population in case of success
				population->addIndividual(offspring,false);			
		}

		// MINING SEQUENCE INFORMATION ON A PERCENTAGE OF THE LOCAL MINIMA
		if (std::rand() % params->samplingRatioMining == 0)
		{
			mining->collect(offspring);
			#if ACTIVE_SIDE_EXPERIMENTS == 2
				if (offspring->isFeasible) sideExperiments->updatePatternCostExperiment2(offspring);
			#endif
		}

		if (offspring->isFeasible && offspring->myCostSol.penalizedCost < bestSolutionCost - MY_EPSILON)
		{ 
			nbIterNonProd = 1; 
			bestSolutionCost = offspring->myCostSol.penalizedCost; 
			//std::cout << "NEW BEST SOLUTION: " << bestSolutionCost << std::endl;
		}
		else nbIterNonProd ++ ;

		// DIVERSIFICATION
		if (nbIterNonProd % 8001 == 8000)
		{
			std::cout << "Diversification" << std::endl;
			population->diversify();
		}

		// PENALTY MANAGEMENT
		if (nbIter % 100 == 0) 
			population->managePenalties() ;

		// MORE TRACES
		if (nbIter % 500 == 0)
			population->printState(nbIter);

		// CHECKPOINT REPORTS
		if (clock() - params->startTime > (ticks*params->progressReports[params->indexProgressReport]) / 100)
		{
			if (params->indexProgressReport == params->progressReports.size() - 1) break; // To avoid duplicate prints at 100% of the CPU time
			population->exportBestSolution(params->pathToSolution);

			// Additional traces for statistical analysis of pattern frequency and quality (Figures 3 and 4 in the paper)
			#if ACTIVE_SIDE_EXPERIMENTS == 1
				if (params->progressReports[params->indexProgressReport] == 20) sideExperiments->exportExperiment1();
			#elif ACTIVE_SIDE_EXPERIMENTS == 2
				if (params->progressReports[params->indexProgressReport] == 20) sideExperiments->exportExperiment2();
			#endif

			params->indexProgressReport++;
		}

		nbIter ++ ;
	}

	// END OF THE ALGORITHM
	std::cout << "Time elapsed: " << clock() << std::endl ;
	std::cout << "Number of iterations: " << nbIter << std::endl ;
}

void Genetic::crossoverOX(Individual * result, Individual * parent1, Individual * parent2)
{
	// We initialize a frequency table to track the customers which have been already inserted
	std::vector <bool> freqClient = std::vector <bool> (params->nbClients + 1, false);

	// We pick the beginning and end of the crossover zone
	int start = std::rand() % params->nbClients;
	int end = std::rand() % params->nbClients;
	while (end == start && params->nbClients > 1)
		end = std::rand() % params->nbClients;

	int j = start;
	// we keep the elements from "start" to "end"
	while ((j % params->nbClients) != ((end + 1) % params->nbClients))
	{
		result->chromT[j % params->nbClients] = parent1->chromT[j % params->nbClients];
		freqClient[result->chromT[j % params->nbClients]] = true;
		j++;
	}

	// We fill the rest of the elements in the order of the second parent
	for (int i = 1; i <= params->nbClients; i++)
	{
		int temp = parent2->chromT[(end + i) % params->nbClients];
		if (freqClient[temp] == false)
		{
			result->chromT[j % params->nbClients] = temp;
			j++;
		}
	}

	// Completing the individual with the Split algorithm
	split->generalSplit(result, std::max<int>(std::max<int>(parent1->myCostSol.nbRoutes, parent2->myCostSol.nbRoutes),params->lbVehicles));
}

Genetic::Genetic(Params * params, Split * split, Population * population, LocalSearch * localSearch, Mining * mining, clock_t ticks) : params(params), split(split), population(population), localSearch(localSearch), mining(mining), ticks(ticks)
{
	offspring = new Individual(params, true);
	#if ACTIVE_SIDE_EXPERIMENTS > 0
		sideExperiments = new SideExperiments(this->params,this->mining);
	#endif
}

Genetic::~Genetic(void)
{ 
	delete offspring ;
}
