#include "LocalSearch.h" 

void LocalSearch::run(Individual * indiv)
{
	penalityLength = params->penalityLength;
	penalityCapa = params->penalityCapa;
	loadIndividual(indiv);

	// Shuffling O(n) elements at each call of the local search: sufficient to randomize while not generating a bottleneck
	std::random_shuffle(orderNodes.begin(), orderNodes.end());
	for (int i = 1; i <= params->nbClients; i++)
		if (std::rand()%params->proxCst==0) 
			std::random_shuffle(params->correlatedVertices[i].begin(), params->correlatedVertices[i].end());
	
	if (params->settingPILS == 1 || params->settingPILS == 3) localSearchPILS();
	localSearchRI(); 
	if (params->settingPILS == 2) localSearchPILS();
	
	exportIndividual(indiv); 
}

void LocalSearch::repair(Individual * indiv)
{
	double tempCapa = params->penalityCapa;
	double tempLength = params->penalityLength;

	params->penalityCapa *= 10;
	params->penalityLength *= 10;
	run(indiv); // First try
	if (!indiv->isFeasible)
	{
		params->penalityCapa *= 500; // Using a factor 500 instead of 10, which guarantees to achieve feasibility in most cases.
		params->penalityLength *= 500;
		run(indiv); // Second try
	}
	params->penalityCapa = tempCapa;
	params->penalityLength = tempLength;

	// Do not forget to reevaluate the penalized cost of the individual after setting back the parameter values to their level
	indiv->myCostSol.penalizedCost = indiv->myCostSol.distance + indiv->myCostSol.capacityViol*params->penalityCapa + indiv->myCostSol.lengthViol*params->penalityLength;
}

void LocalSearch::localSearchPILS()
{
	params->totalPILSPhases++;
	double tcpu1 = (double)clock() / (double)CLOCKS_PER_SEC ;

	// Shuffling the list of patterns to select a random subset of factorPatternTesting x n patterns
	int sizeorderPatterns = orderPatterns.size();
	for (int i = 0; i < (int)(params->factorPatternTesting*(double)params->nbClients); i++)
		std::swap(orderPatterns[i], orderPatterns[rand()%sizeorderPatterns]);

	for (int sizePattern = params->maxSeqMining; sizePattern >= params->minSeqMining ; sizePattern--)
	{
		int patternsTested = 0;
		int maxPatternTests = (int)(params->factorPatternTesting*(double)params->nbClients); // Only a subset of the patterns will be tested
		for (int patIndex = 0; patIndex < (int)orderPatterns.size() && patternsTested < maxPatternTests ; patIndex++) 
		{
			if (orderPatterns[patIndex] < mining->getNbSequences(sizePattern))
			{
				/* FIRST CHECKING IF PATTERN ALREADY BELONGS TO THE SOLUTION */
				const std::vector <int> & sequence = mining->frequentSequence(sizePattern, orderPatterns[patIndex]);	// Pattern which we want to inject
				bool isDifferent1 = false;
				bool isDifferent2 = false;
				for (int i = 1; i < sizePattern; i++)
				{
					// In order of the pattern
					if (clients[sequence[i]].prev->cour != sequence[i - 1])
						isDifferent1 = true;
					
					// Or in reversed order
					if (clients[sequence[i]].next->cour != sequence[i - 1])
						isDifferent2 = true;
				}			

				if (isDifferent1 && isDifferent2) // It is worth testing this move, as it does not currently belong to the solution in any direction
				{
					/* INITIALIZING DATA STRUCTURES AND COUNTING THE NEW TEST */
					params->totalPILSMovesTested++;
					patternsTested++;
					involvedRoutes.clear();
					for (int i = 0; i < sizePattern; i++)
					{
						clients[sequence[i]].isPatternElement = true; // Marking nodes which compose the pattern
						Route * myRouteR = clients[sequence[i]].route;
						if (!myRouteR->isPatternElement)
						{
							involvedRoutes.push_back(myRouteR->cour);  // Storing routes involved into a list
							myRouteR->isPatternElement = true;
						}
					}

					/* ENUMERATING ROUTE SEGMENTS "BROKEN/GENERATED" BY THE PATTERN AND CALCULATING BUDGET FOR RECONSTRUCTION */
					vectorSegmentsBegin.clear();
					vectorSegmentsMiddle.clear();
					vectorSegmentsEnd.clear();
					reconstructionActions.clear();
					reversalActions.clear();
					double costPre = 0.;
					double costPost = 0.;
					for (int routeID : involvedRoutes)
					{
						Route * myRoute = &routes[routeID];
						costPre += myRoute->penalizedCost;
						bool isFirstIt = true;
						Node * courNode = myRoute->depot;
						while (isFirstIt || courNode != myRoute->depot)
						{
							isFirstIt = false;
							while (courNode->isPatternElement)
								courNode = courNode->next;

							Segment mySegment;
							initSegment(mySegment, courNode);
							courNode = courNode->next;
							while (courNode != myRoute->depot && !courNode->isPatternElement)
							{
								appendSegment(mySegment, courNode);
								courNode = courNode->next;
							}

							// add the segment in the suitable vector
							if (mySegment.node1->isDepot && !mySegment.node1->isDepotEnd) vectorSegmentsBegin.push_back(mySegment);
							else if (mySegment.node2->isDepotEnd) vectorSegmentsEnd.push_back(mySegment);
							else vectorSegmentsMiddle.push_back(mySegment);
							costPost += mySegment.penalizedCost;
						}
					}

					// Add the segment corresponding to the pattern
					Segment mySegmentPattern;
					initSegment(mySegmentPattern, &clients[sequence[0]]);
					for (int i = 1; i < sizePattern; i++)
						appendSegment(mySegmentPattern, &clients[sequence[i]]);
					vectorSegmentsMiddle.push_back(mySegmentPattern);
					costPost += mySegmentPattern.penalizedCost;

					/* START THE RECURSIVE SEARCH FOR A BEST SET OF RECONNECTIONS */
					foundImprovingPILS = false;
					bestBudgetReconstruction = MY_EPSILON;
					findReconnections((int)vectorSegmentsBegin.size() - 1, (int)vectorSegmentsMiddle.size() - 1, costPre - costPost);

					/* UPDATING THE SOLUTION IF AN IMPROVING MOVE HAS BEEN FOUND */
					if (foundImprovingPILS)
					{
						// Updating some statistics about PILS moves
						params->totalPILSMovesApplied++;
						params->totalPILSMovesApplied_PerPatternSize[sizePattern]++;
						params->totalPILSMovesApplied_PerInvolvedRoutes[involvedRoutes.size()]++;

						// Integer used to calculate the order of the move
						int newEdges = 0;

						std::multiset<std::pair<int,int>> setOfEdges;
						for (int routeID : involvedRoutes)
						{
							Route * myRoute = &routes[routeID];
							
							//Loop below ignores depot. We only store edges, where an edge (i,j) is guaranteed i < j.
							//We reverse an arc (i,j) iff i > j if necessary. 
							Node * courNode = myRoute->depot;
							bool firstIt = true;
							while (firstIt || courNode->next != myRoute->depot)
							{
								int e_i = courNode->cour;
								int e_j = courNode->next->cour;
								if(e_i > e_j)
									std::swap(e_i,e_j);

								setOfEdges.insert(std::make_pair(e_i,e_j));
								firstIt = false;
								courNode = courNode->next;
							}
						}

						// Updating the linking for the pattern
						for (int i = 1; i < sizePattern; i++) 
						{
							clients[sequence[i]].prev = &clients[sequence[i - 1]];
							clients[sequence[i - 1]].next = &clients[sequence[i]];
						}
						for (int i = 0; i < (int)bestReversalActions.size(); i++) // Applying possible reversals of segments
						{
							// courNode->prev is set to courNode->next after reversal of the links
							for (Node * courNode = bestReversalActions[i].first; courNode != bestReversalActions[i].second; courNode = courNode->prev)
							{
								Node * temp = courNode->next;
								courNode->next = courNode->prev;
								courNode->prev = temp;
							}
							bestReversalActions[i].second->next = bestReversalActions[i].second->prev; // Don't forget the last element
						}
						for (int i = 0; i < (int)bestReconstructionActions.size(); i++) // Updating the linking for the connections
						{
							bestReconstructionActions[i].first->next = bestReconstructionActions[i].second;
							bestReconstructionActions[i].second->prev = bestReconstructionActions[i].first;
						}

						// calling update routes to tidy up all route variables (Also fixes route attribution to be identical to the depot)
						for (int routeID : involvedRoutes)
							updateRouteData(&routes[routeID], false);

						// incrementing the move counter
						nbMoves++;
						
						for (int routeID : involvedRoutes)
						{
							Route * myRoute = &routes[routeID];
							
							//Loop below ignores depot. We only store edges, where an edge (i,j) is guaranteed i < j.
							//We reverse an arc (i,j) iff i > j if necessary. 
							Node * courNode = myRoute->depot;
							bool firstIt = true;
							while (firstIt || courNode->next != myRoute->depot)
							{
								int e_i = courNode->cour;
								int e_j = courNode->next->cour;
								if(e_i > e_j)
									std::swap(e_i,e_j);

								// If this edge is not contained in the set of edges, then we count an edge difference
								// Otherwise, we count the edge as "already seen" (useful to count correctly new edges involving the depot)
								auto it = setOfEdges.find(std::make_pair(e_i, e_j));
								if (it == setOfEdges.end())
									newEdges++;
								else
									setOfEdges.erase(it);

								firstIt = false;
								courNode = courNode->next;
							}
						}
						params->totalPILSMovesApplied_PerMoveOrder[newEdges]++;
					}

					// Don't forget to unflag the nodes and routes which appear in the pattern
					for (int i = 0; i < sizePattern; i++)
						clients[sequence[i]].isPatternElement = false;
					for (int routeID : involvedRoutes)
						routes[routeID].isPatternElement = false;
				}
			}
		}
	}

	double tcpu2 = (double)clock() / (double)CLOCKS_PER_SEC;
	params->totalCPUTimePILS += tcpu2 - tcpu1;
}

void LocalSearch::findReconnections(int indexVectorBeginEnd, int indexVectorMiddle, double budgetCost)
{
	Segment mySegmentResult;

	/* BASE CASE -- only two segments left (begin + end) */ 
	if (indexVectorBeginEnd == 0 && indexVectorMiddle == -1)
	{
		combineSegment(mySegmentResult, vectorSegmentsBegin[0], vectorSegmentsEnd[0]);
		double costCombination = mySegmentResult.penalizedCost - vectorSegmentsBegin[0].penalizedCost - vectorSegmentsEnd[0].penalizedCost;
		if (budgetCost - costCombination > bestBudgetReconstruction)
		{
			//if (!reversalActions.empty()) std::cout << "Reversal move PILS: " << budgetCost - costCombination << std::endl;
			//else std::cout << "Standard move PILS: " << budgetCost - costCombination << std::endl;
			bestBudgetReconstruction = budgetCost - costCombination;
			foundImprovingPILS = true;

			reconstructionActions.push_back(std::pair<Node*, Node*>(vectorSegmentsBegin[0].node2, vectorSegmentsEnd[0].node1));
			bestReconstructionActions = reconstructionActions;
			bestReversalActions = reversalActions;
			reconstructionActions.pop_back();
		}
	}
	/* INDUCTION STEP */
	else
	{
		Segment mySegment1 = vectorSegmentsBegin[indexVectorBeginEnd]; // Select the segment of index indexVectorBeginEnd in vectorSegmentsBegin
		for (int i = 0; i <= indexVectorMiddle; i++) // Attempt to append any element of vectorMiddle afterwards
		{
			Segment mySegment2 = vectorSegmentsMiddle[i];

			// Try to append the middle segment -- without reversal
			combineSegment(mySegmentResult, mySegment1, mySegment2);
			double costCombination = mySegmentResult.penalizedCost - mySegment1.penalizedCost - mySegment2.penalizedCost;
			double updatedBudgetCost = budgetCost - costCombination;
			if (updatedBudgetCost > bestBudgetReconstruction) // Otherwise, eliminate move as it could not improve 
			{
				reconstructionActions.push_back(std::pair<Node*, Node*>(mySegment1.node2, mySegment2.node1));
				vectorSegmentsBegin[indexVectorBeginEnd] = mySegmentResult; // Organize the elements for the next recursion
				vectorSegmentsMiddle[i] = vectorSegmentsMiddle[indexVectorMiddle];
				findReconnections(indexVectorBeginEnd, indexVectorMiddle - 1, updatedBudgetCost); // Call the recursion (one middle segment has been removed)
				vectorSegmentsBegin[indexVectorBeginEnd] = mySegment1; // Rewind the changes after the recursion
				vectorSegmentsMiddle[i] = mySegment2;
				reconstructionActions.pop_back();
			}

			// Try to append the middle segment -- with reversal
			if (mySegment2.node1 != mySegment2.node2) // Do not try reverting a middle segment which has only one node
			{
				Segment myReversedSegment2;
				reverseSegment(myReversedSegment2, mySegment2);
				combineSegment(mySegmentResult, mySegment1, myReversedSegment2);
				costCombination = mySegmentResult.penalizedCost - mySegment1.penalizedCost - mySegment2.penalizedCost;
				updatedBudgetCost = budgetCost - costCombination;
				if (updatedBudgetCost > bestBudgetReconstruction) // Otherwise, eliminate move as it could not improve 
				{
					reversalActions.push_back(std::pair<Node*, Node*>(mySegment2.node1, mySegment2.node2));
					reconstructionActions.push_back(std::pair<Node*, Node*>(mySegment1.node2, mySegment2.node2));
					vectorSegmentsBegin[indexVectorBeginEnd] = mySegmentResult; // Organize the elements for the next recursion
					vectorSegmentsMiddle[i] = vectorSegmentsMiddle[indexVectorMiddle];
					findReconnections(indexVectorBeginEnd, indexVectorMiddle - 1, updatedBudgetCost); // Call the recursion (one middle segment has been removed)
					vectorSegmentsBegin[indexVectorBeginEnd] = mySegment1; // Rewind the changes after the recursion
					vectorSegmentsMiddle[i] = mySegment2;
					reconstructionActions.pop_back();
					reversalActions.pop_back();
				}
			}
		}
		if (indexVectorBeginEnd > 0 || indexVectorMiddle == -1) // Forbidden to combine the last begin and end if some middle elements are left
		{
			for (int i = 0; i <= indexVectorBeginEnd; i++) // Attempt to append any element of vectorSegmentsEnd afterwards (vectorSegmentsEnd always has the same number of elements as vectorSegmentBegin)
			{
				Segment mySegment2 = vectorSegmentsEnd[i];
				combineSegment(mySegmentResult, mySegment1, mySegment2);
				double costCombination = mySegmentResult.penalizedCost - mySegment1.penalizedCost - mySegment2.penalizedCost;
				double updatedBudgetCost = budgetCost - costCombination;
				if (updatedBudgetCost > bestBudgetReconstruction) // Otherwise, eliminate move as it could not improve 
				{
					reconstructionActions.push_back(std::pair<Node*, Node*>(mySegment1.node2, mySegment2.node1));
					vectorSegmentsEnd[i] = vectorSegmentsEnd[indexVectorBeginEnd]; // Organize the elements for the next recursion
					findReconnections(indexVectorBeginEnd-1, indexVectorMiddle, updatedBudgetCost); // Call the recursion (one begin and end segment have been removed)
					vectorSegmentsEnd[i] = mySegment2; // Rewind the changes after the recursion
					reconstructionActions.pop_back();
				}
			}
		}
	}
}

void LocalSearch::reverseSegment(Segment & mySegmentResult, const Segment & mySegment)
{
	mySegmentResult.node1 = mySegment.node2;
	mySegmentResult.node2 = mySegment.node1;
	mySegmentResult.distance = mySegment.distance + mySegment.reversalDistance;
	mySegmentResult.load = mySegment.load;
	mySegmentResult.time = mySegment.time + mySegment.reversalDistance;
	mySegmentResult.reversalDistance = -mySegment.reversalDistance;
	mySegmentResult.penalizedCost = mySegmentResult.distance + excedentLength(mySegmentResult.time) + excedentCharge(mySegmentResult.load);
}

void LocalSearch::combineSegment(Segment & mySegmentResult, const Segment & mySegment1, const Segment & mySegment2)
{
	mySegmentResult.node1 = mySegment1.node1;
	mySegmentResult.node2 = mySegment2.node2;
	mySegmentResult.distance = mySegment1.distance + params->timeCost[mySegment1.node2->cour][mySegment2.node1->cour] + mySegment2.distance;
	mySegmentResult.load = mySegment1.load + mySegment2.load;
	mySegmentResult.time = mySegment1.time + params->timeCost[mySegment1.node2->cour][mySegment2.node1->cour] + mySegment2.time;
	mySegmentResult.reversalDistance = mySegment1.reversalDistance + params->timeCost[mySegment2.node1->cour][mySegment1.node2->cour] - params->timeCost[mySegment1.node2->cour][mySegment2.node1->cour] + mySegment2.reversalDistance;
	mySegmentResult.penalizedCost = mySegmentResult.distance + excedentLength(mySegmentResult.time) + excedentCharge(mySegmentResult.load);
}

void LocalSearch::initSegment(Segment & mySegmentResult, Node * node)
{
	mySegmentResult.node1 = node;
	mySegmentResult.node2 = node;
	mySegmentResult.distance = 0.;
	mySegmentResult.load = params->cli[node->cour].demand;
	mySegmentResult.time = params->cli[node->cour].serviceDuration;
	mySegmentResult.reversalDistance = 0.;
	mySegmentResult.penalizedCost = 0.;
}

void LocalSearch::appendSegment(Segment & mySegmentResult, Node * node)
{
	mySegmentResult.distance += params->timeCost[mySegmentResult.node2->cour][node->cour];
	mySegmentResult.load += params->cli[node->cour].demand;
	mySegmentResult.time += params->timeCost[mySegmentResult.node2->cour][node->cour] + params->cli[node->cour].serviceDuration;
	mySegmentResult.reversalDistance += params->timeCost[node->cour][mySegmentResult.node2->cour] - params->timeCost[mySegmentResult.node2->cour][node->cour];
	mySegmentResult.node2 = node;
	mySegmentResult.penalizedCost = mySegmentResult.distance + excedentLength(mySegmentResult.time) + excedentCharge(mySegmentResult.load);
}

int LocalSearch::localSearchRI()
{
	double tcpu1 = (double)clock() / (double)CLOCKS_PER_SEC;
	searchCompleted = false;
	bool movePerformed;
	firstLoop = true;

	while (!searchCompleted)
	{
		searchCompleted = true;
		movePerformed = false;
		int posU = 0;
		while (posU < params->nbClients)
		{
			noeudU = &clients[orderNodes[posU]];
			setLocalVariablesRouteU();
			std::vector <int> & correlatedVertices = params->correlatedVertices[orderNodes[posU]];
			for (int posV = 0; posV < (int)correlatedVertices.size() && !movePerformed; posV++)
			{
				noeudV = &clients[correlatedVertices[posV]];
				if (firstLoop || std::max<int>(noeudU->route->whenLastModified, noeudV->route->whenLastModified) > noeudU->whenLastTested)
				{
					setLocalVariablesRouteV();

					// RELOCATE
					if (!movePerformed) movePerformed = move1();
					if (!movePerformed) movePerformed = move2();
					if (!movePerformed) movePerformed = move3();

					// SWAP (avoiding symmetrical calculations)
					if (noeudUCour <= noeudVCour)
					{
						if (!movePerformed) movePerformed = move4();
						if (!movePerformed) movePerformed = move6();
					}
					if (!movePerformed) movePerformed = move5();

					if (routeU == routeV)
					{
						// 2-OPT
						if (!movePerformed) movePerformed = move7();
					}
					else
					{
						// 2-OPT*
						if (!movePerformed) movePerformed = move8();
						if (!movePerformed) movePerformed = move9();
					}

					// Trying moves that insert noeudU directly after the depot
					if (!movePerformed && noeudV->prev->isDepot)
					{
						noeudV = noeudV->prev;
						setLocalVariablesRouteV();

						// RELOCATE
						if (!movePerformed) movePerformed = move1();
						if (!movePerformed) movePerformed = move2();
						if (!movePerformed) movePerformed = move3();
						
						// 2-OPT*
						if (!noeudV->next->isDepot && routeU != routeV)
						{
							if (!movePerformed) movePerformed = move8();
							if (!movePerformed) movePerformed = move9();
						}
					}

					// Trying moves that insert noeudU in an empty route (or cut a route in two in the case of 2-opt*)
					if (!movePerformed && !emptyRoutes.empty())
					{
						noeudV = routes[*emptyRoutes.begin()].depot;
						setLocalVariablesRouteV();

						// RELOCATE
						if (!movePerformed) movePerformed = move1();
						if (!movePerformed) movePerformed = move2();
						if (!movePerformed) movePerformed = move3();

						// 2-OPT*
						if (!noeudV->next->isDepot && routeU != routeV)
						{
							if (!movePerformed) movePerformed = move8();
							if (!movePerformed) movePerformed = move9();
						}
					}
				}
			}
			if (!movePerformed)
			{
				noeudU->whenLastTested = nbMoves;
				posU++;
			}
			else
			{
				nbMoves++;
				movePerformed = false;
			}
		}
		firstLoop = false;
	}

	double tcpu2 = (double)clock() / (double)CLOCKS_PER_SEC;
	params->totalCPUTimeLS += tcpu2 - tcpu1;
	return nbMoves;
}

void LocalSearch::setLocalVariablesRouteU()
{
	routeU = noeudU->route;
	noeudX = noeudU->next;
	noeudXSuivCour = noeudX->next->cour;
	noeudUCour = noeudU->cour;
	noeudUPredCour = noeudU->prev->cour;
	noeudXCour = noeudX->cour;
	loadU    = params->cli[noeudUCour].demand;
	serviceU = params->cli[noeudUCour].serviceDuration;
	loadX	 = params->cli[noeudXCour].demand;
	serviceX = params->cli[noeudXCour].serviceDuration;
	distUpU  = params->timeCost[noeudUPredCour][noeudUCour];
	distUpX  = params->timeCost[noeudUPredCour][noeudXCour];
	distUpXs = params->timeCost[noeudUPredCour][noeudXSuivCour];
	distUX   = params->timeCost[noeudUCour][noeudXCour];
	distXU   = params->timeCost[noeudXCour][noeudUCour];
	distXXs  = params->timeCost[noeudXCour][noeudXSuivCour];
}

void LocalSearch::setLocalVariablesRouteV()
{
	routeV = noeudV->route;
	noeudY = noeudV->next;
	noeudYSuivCour = noeudY->next->cour;
	noeudVCour = noeudV->cour;
	noeudVPredCour = noeudV->prev->cour;
	noeudYCour = noeudY->cour;
	loadV    = params->cli[noeudVCour].demand;
	serviceV = params->cli[noeudVCour].serviceDuration;
	loadY	 = params->cli[noeudYCour].demand;
	serviceY = params->cli[noeudYCour].serviceDuration;

	distUpV = params->timeCost[noeudUPredCour][noeudVCour];
	distUY  = params->timeCost[noeudUCour][noeudYCour];
	distUV  = params->timeCost[noeudUCour][noeudVCour];
	distXY  = params->timeCost[noeudXCour][noeudYCour];
	distXYs = params->timeCost[noeudXCour][noeudYSuivCour];
	distVpV = params->timeCost[noeudVPredCour][noeudVCour];
	distVpU = params->timeCost[noeudVPredCour][noeudUCour];
	distVU  = params->timeCost[noeudVCour][noeudUCour];
	distVX  = params->timeCost[noeudVCour][noeudXCour];
	distVY = params->timeCost[noeudVCour][noeudYCour];
	distVXs = params->timeCost[noeudVCour][noeudXSuivCour];
	distYXs = params->timeCost[noeudYCour][noeudXSuivCour];
	distYYs = params->timeCost[noeudYCour][noeudYSuivCour];
}

int LocalSearch::localSearchPI()
{
	searchCompleted = false;
	firstLoop = true;

	while (!searchCompleted)
	{
		searchCompleted = true;
		for (int posU = 0; posU < params->nbClients; posU++)
		{
			Node * client = &clients[orderNodes[posU]];
			int clientCour = client->cour;
			double coutInsertion = 1.e30;
			Node * placeInsertion = NULL;

			for (int r = 0; r < params->nbVehicles; r++)
			{	
				if (costInsertClient[r][clientCour] > 1.e29 || firstLoop)
				{
					Route * myRoute = &routes[r];
					costInsertClient[r][clientCour] = 1.e30 ;
					if (client->route != myRoute)
					{	
						double coutBase = excedentCharge(myRoute->load + params->cli[clientCour].demand)
							- excedentCharge(myRoute->load) - excedentLength(myRoute->duration);

						bool firstIt = true;
						Node * courNode = myRoute->depot;
						while (!courNode->isDepot || firstIt == true)
						{
							firstIt = false;
							double mycout = params->timeCost[courNode->cour][clientCour]
								+ params->timeCost[clientCour][courNode->next->cour]
								- params->timeCost[courNode->cour][courNode->next->cour];

							mycout += excedentLength(myRoute->duration + mycout + params->cli[clientCour].serviceDuration) 
								   + coutBase;

							if (mycout < costInsertClient[r][clientCour])
							{
								costInsertClient[r][clientCour] = mycout;
								placeInsertClient[r][clientCour] = courNode;
							}
							courNode = courNode->next;
						}
					}
					else // Already located in the route
					{
						double coutCourant = params->timeCost[client->prev->cour][clientCour] 
							- params->timeCost[client->prev->cour][client->next->cour]
							+ params->timeCost[clientCour][client->next->cour] ;

						double coutBase = excedentCharge(myRoute->load)
							- excedentCharge(myRoute->load - params->cli[clientCour].demand)
							- excedentLength(myRoute->duration - coutCourant - params->cli[clientCour].serviceDuration);

						costInsertClient[r][clientCour] = coutCourant
							+ excedentLength(myRoute->duration) + coutBase;
							
						placeInsertClient[r][clientCour] = client->prev;

						// cependant on va regarder si il n'y a pas une meilleure insertion possible
						// temporairement on va enlever le noeud
						client->prev->next = client->next;
						client->next->prev = client->prev;
						
						// et parcourir la route � nouveau
						Node * courNode = myRoute->depot;
						bool firstIt = true;
						while (!courNode->isDepot || firstIt == true)
						{
							firstIt = false;
							double mycout = params->timeCost[courNode->cour][clientCour]
								+ params->timeCost[clientCour][courNode->next->cour]
								- params->timeCost[courNode->cour][courNode->next->cour];

							// au final on peut placer d'une meilleure mani�re
							if (mycout < coutCourant - MY_EPSILON)
							{
								costInsertClient[r][clientCour] = mycout
									+ excedentLength(myRoute->duration - coutCourant + mycout)
									+ coutBase;
								placeInsertClient[r][clientCour] = courNode;
								coutCourant = mycout;
							}
							courNode = courNode->next;
						}

						// on replace le noeud
						client->prev->next = client;
						client->next->prev = client;
					}
				}

				if (costInsertClient[r][clientCour] < coutInsertion - MY_EPSILON)
				{
					coutInsertion = costInsertClient[r][clientCour];
					placeInsertion = placeInsertClient[r][clientCour];
				}
			}

			// If needed, we apply the move
			if (placeInsertion != client->prev)
			{
				Route * myRoute1 = client->route;
				Route * myRoute2 = placeInsertion->route;
				insertNode(client, placeInsertion);
				updateRouteData(myRoute1,false);
				if (myRoute1 != myRoute2) updateRouteData(myRoute2, false);
				searchCompleted = false;
				nbMoves++;
			}
		}
		firstLoop = false;
	}
	return nbMoves;
}

void LocalSearch::insertNode(Node * U, Node * V)
{
	U->prev->next = U->next;
	U->next->prev = U->prev;
	V->next->prev = U;
	U->prev = V;
	U->next = V->next;
	V->next = U;
	U->route = V->route;
}

void LocalSearch::swapNode(Node * U, Node * V)
{
	Node * myVPred = V->prev;
	Node * myVSuiv = V->next;
	Node * myUPred = U->prev;
	Node * myUSuiv = U->next;
	Route * myRouteU = U->route;
	Route * myRouteV = V->route;

	myUPred->next = V;
	myUSuiv->prev = V;
	myVPred->next = U;
	myVSuiv->prev = U;

	U->prev = myVPred;
	U->next = myVSuiv;
	V->prev = myUPred;
	V->next = myUSuiv;

	U->route = myRouteV;
	V->route = myRouteU;
}

bool LocalSearch::move1()
{
	double costSuppU = distUpX - distUpU - distUX;
	double costSuppV = distVU + distUY - distVY;

	if (routeU != routeV)
	{
		costSuppU += excedentLength(routeU->duration + costSuppU - serviceU)
			+ excedentCharge(routeU->load - loadU) 
			- routeU->penalty;

		costSuppV += excedentLength(routeV->duration + costSuppV + serviceU)
			+ excedentCharge(routeV->load + loadU) 
			- routeV->penalty;
	}

	if (costSuppU + costSuppV > -MY_EPSILON) return false;
	if (noeudUCour == noeudYCour) return false;

	insertNode(noeudU, noeudV);
	updateRouteData(routeU, true);
	if (routeU != routeV) updateRouteData(routeV, true);

	params->totalLSMovesApplied++;
	searchCompleted = false;
	return true;
}

bool LocalSearch::move2()
{
	double costSuppU = distUpXs - distUpU - distXXs;
	double costSuppV = distVU + distXY - distVY;

	if (routeU != routeV)
	{
		costSuppU += excedentLength(routeU->duration + costSuppU - distUX - serviceU - serviceX)
			+ excedentCharge(routeU->load - loadU - loadX) 
			- routeU->penalty;

		costSuppV += excedentLength(routeV->duration + costSuppV + distUX + serviceU + serviceX)
			+ excedentCharge(routeV->load + loadU + loadX) 
			- routeV->penalty;
	}

	if (costSuppU + costSuppV > -MY_EPSILON) return false;
	if (noeudU == noeudY || noeudV == noeudX || noeudX->isDepot) return false;

	insertNode(noeudU, noeudV);
	insertNode(noeudX, noeudU);
	updateRouteData(routeU, true);
	if (routeU != routeV) updateRouteData(routeV, true);

	params->totalLSMovesApplied++;
	searchCompleted = false;
	return true;
}

bool LocalSearch::move3()
{
	double costSuppU = distUpXs - distUpU - distUX - distXXs;
	double costSuppV = distVX + distXU + distUY - distVY;

	if (routeU != routeV)
	{
		costSuppU += excedentLength(routeU->duration + costSuppU - serviceU - serviceX)
			+ excedentCharge(routeU->load - loadU - loadX) 
			- routeU->penalty;

		costSuppV += excedentLength(routeV->duration + costSuppV + serviceU + serviceX)
			+ excedentCharge(routeV->load + loadU + loadX) 
			- routeV->penalty;
	}

	if (costSuppU + costSuppV > -MY_EPSILON) return false;
	if (noeudU == noeudY || noeudX == noeudV || noeudX->isDepot) return false;

	insertNode(noeudX, noeudV);
	insertNode(noeudU, noeudX);
	updateRouteData(routeU, true);
	if (routeU != routeV) updateRouteData(routeV, true);

	params->totalLSMovesApplied++;
	searchCompleted = false;
	return true;
}

bool LocalSearch::move4()
{
	double costSuppU = distUpV + distVX - distUpU - distUX;
	double costSuppV = distVpU + distUY - distVpV - distVY;

	if (routeU != routeV)
	{
		costSuppU += excedentLength(routeU->duration + costSuppU + serviceV - serviceU)
			+ excedentCharge(routeU->load + loadV - loadU) 
			- routeU->penalty;

		costSuppV += excedentLength(routeV->duration + costSuppV - serviceV + serviceU)
			+ excedentCharge(routeV->load + loadU - loadV) 
			- routeV->penalty;
	}

	if (costSuppU + costSuppV > -MY_EPSILON) return false;
	if (noeudUCour == noeudVPredCour || noeudUCour == noeudYCour) return false;

	swapNode(noeudU, noeudV);
	updateRouteData(routeU, true);
	if (routeU != routeV) updateRouteData(routeV, true);

	params->totalLSMovesApplied++;
	searchCompleted = false;
	return true;
}

bool LocalSearch::move5()
{
	double costSuppU = distUpV + distVXs - distUpU - distXXs;
	double costSuppV = distVpU + distXY - distVpV - distVY;

	if (routeU != routeV)
	{
		costSuppU += excedentLength(routeU->duration + costSuppU - distUX + serviceV - serviceU - serviceX)
			+ excedentCharge(routeU->load + loadV - loadU - loadX) 
			- routeU->penalty;

		costSuppV += excedentLength(routeV->duration + costSuppV + distUX - serviceV + serviceU + serviceX)
			+ excedentCharge(routeV->load + loadU + loadX - loadV) 
			- routeV->penalty;
	}

	if (costSuppU + costSuppV > -MY_EPSILON) return false;
	if (noeudU == noeudV->prev || noeudX == noeudV->prev || noeudU == noeudY || noeudX->isDepot) return false;

	swapNode(noeudU, noeudV);
	insertNode(noeudX, noeudU);
	updateRouteData(routeU, true);
	if (routeU != routeV) updateRouteData(routeV, true);

	params->totalLSMovesApplied++;
	searchCompleted = false;
	return true;
}

bool LocalSearch::move6()
{
	double costSuppU = distUpV + distYXs - distUpU - distXXs;
	double costSuppV = distVpU + distXYs - distVpV - distYYs;

	if (routeU != routeV)
	{
		costSuppU += excedentLength(routeU->duration + costSuppU - distUX + distVY + serviceV + serviceY - serviceU - serviceX)
			+ excedentCharge(routeU->load + loadV + loadY - loadU - loadX) 
			- routeU->penalty;

		costSuppV += excedentLength(routeV->duration + costSuppV + distUX - distVY - serviceV - serviceY + serviceU + serviceX)
			+ excedentCharge(routeV->load + loadU + loadX - loadV - loadY) 
			- routeV->penalty;
	}

	if (costSuppU + costSuppV > -MY_EPSILON) return false;
	if (noeudX->isDepot || noeudY->isDepot || noeudY == noeudU->prev || noeudU == noeudY || noeudX == noeudV || noeudV == noeudX->next) return false;

	swapNode(noeudU, noeudV);
	swapNode(noeudX, noeudY);
	updateRouteData(routeU, true);
	if (routeU != routeV) updateRouteData(routeV, true);

	params->totalLSMovesApplied++;
	searchCompleted = false;
	return true;
}

bool LocalSearch::move7()
{
	if (noeudU->position > noeudV->position) return false;
	
	double cost = distUV + distXY - distUX - distVY + noeudV->cumulatedReversalDistance - noeudX->cumulatedReversalDistance;
	
	if (cost > -MY_EPSILON) return false;
	if (noeudU->next == noeudV) return false;

	Node * nodeNum = noeudX->next;
	noeudX->prev = nodeNum;
	noeudX->next = noeudY;

	while (nodeNum != noeudV)
	{
		Node * temp = nodeNum->next;
		nodeNum->next = nodeNum->prev;
		nodeNum->prev = temp;
		nodeNum = temp;
	}

	noeudV->next = noeudV->prev;
	noeudV->prev = noeudU;
	noeudU->next = noeudV;
	noeudY->prev = noeudX;

	updateRouteData(routeU, true);

	params->totalLSMovesApplied++;
	searchCompleted = false;
	return true;
}

bool LocalSearch::move8()
{
	double cost = distUV + distXY - distUX - distVY
		+ excedentLength(noeudU->cumulatedTime + noeudV->cumulatedTime + noeudV->cumulatedReversalDistance + distUV)
		+ excedentLength(routeU->duration - noeudU->cumulatedTime - distUX + routeU->reversalDistance - noeudX->cumulatedReversalDistance + routeV->duration - noeudV->cumulatedTime - distVY + distXY)
		+ excedentCharge(noeudU->cumulatedLoad + noeudV->cumulatedLoad)
		+ excedentCharge(routeU->load + routeV->load - noeudU->cumulatedLoad - noeudV->cumulatedLoad)
		- routeU->penalty - routeV->penalty
		+ noeudV->cumulatedReversalDistance + routeU->reversalDistance - noeudX->cumulatedReversalDistance;

	if (cost > -MY_EPSILON) return false;

	Node * depotU = routeU->depot;
	Node * depotV = routeV->depot;
	Node * depotUFin = routeU->depot->prev;
	Node * depotVFin = routeV->depot->prev;
	Node * depotVSuiv = depotV->next;

	Node * temp;
	Node * xx = noeudX;
	Node * vv = noeudV;

	while (!xx->isDepot)
	{
		temp = xx->next;
		xx->next = xx->prev;
		xx->prev = temp;
		xx->route = routeV;
		xx = temp;
	}

	while (!vv->isDepot)
	{
		temp = vv->prev;
		vv->prev = vv->next;
		vv->next = temp;
		vv->route = routeU;
		vv = temp;
	}

	noeudU->next = noeudV;
	noeudV->prev = noeudU;
	noeudX->next = noeudY;
	noeudY->prev = noeudX;

	if (noeudX->isDepot)
	{
		depotUFin->next = depotU;
		depotUFin->prev = depotVSuiv;
		depotUFin->prev->next = depotUFin;
		depotV->next = noeudY;
		noeudY->prev = depotV;
	}
	else if (noeudV->isDepot)
	{
		depotV->next = depotUFin->prev;
		depotV->next->prev = depotV;
		depotV->prev = depotVFin;
		depotUFin->prev = noeudU;
		noeudU->next = depotUFin;
	}
	else
	{
		depotV->next = depotUFin->prev;
		depotV->next->prev = depotV;
		depotUFin->prev = depotVSuiv;
		depotUFin->prev->next = depotUFin;
	}

	updateRouteData(routeU, true);
	updateRouteData(routeV, true);

	params->totalLSMovesApplied++;
	searchCompleted = false;
	return true;
}

bool LocalSearch::move9()
{
	double cost = distUY + distVX - distUX - distVY
		+ excedentLength(noeudU->cumulatedTime + routeV->duration - noeudV->cumulatedTime - distVY + distUY)
		+ excedentLength(routeU->duration - noeudU->cumulatedTime - distUX + noeudV->cumulatedTime + distVX)
		+ excedentCharge(noeudU->cumulatedLoad + routeV->load - noeudV->cumulatedLoad)
		+ excedentCharge(noeudV->cumulatedLoad + routeU->load - noeudU->cumulatedLoad)
		- routeU->penalty - routeV->penalty;

	if (cost > -MY_EPSILON) return false;

	Node * depotU = routeU->depot;
	Node * depotV = routeV->depot;
	Node * depotUFin = depotU->prev;
	Node * depotVFin = depotV->prev;
	Node * depotUpred = depotUFin->prev;

	Node * count = noeudY;
	while (!count->isDepot)
	{
		count->route = routeU;
		count = count->next;
	}

	count = noeudX;
	while (!count->isDepot)
	{
		count->route = routeV;
		count = count->next;
	}

	noeudU->next = noeudY;
	noeudY->prev = noeudU;
	noeudV->next = noeudX;
	noeudX->prev = noeudV;

	if (noeudX->isDepot)
	{
		depotUFin->prev = depotVFin->prev;
		depotUFin->prev->next = depotUFin;
		noeudV->next = depotVFin;
		depotVFin->prev = noeudV;
	}
	else
	{
		depotUFin->prev = depotVFin->prev;
		depotUFin->prev->next = depotUFin;
		depotVFin->prev = depotUpred;
		depotVFin->prev->next = depotVFin;
	}

	updateRouteData(routeU, true);
	updateRouteData(routeV, true);

	params->totalLSMovesApplied++;
	searchCompleted = false;
	return true;
}

void LocalSearch::updateRouteData(Route * myRoute, bool isRI)
{
	int myplace = 0;

	Node * noeud = myRoute->depot;
	noeud->position = myplace;
	double mycharge = 0.;
	double mytime = 0.;
	double mycost = 0.;
	double myReversalDistance = 0.;
	double myXvalue = 0.;
	double myYvalue = 0.;
	noeud->cumulatedLoad = 0.;
	noeud->cumulatedTime = 0.;
	noeud->cumulatedReversalDistance = 0.;

	bool firstIt = true;
	while (!noeud->isDepot || firstIt)
	{
		firstIt = false;
		myplace++;

		noeud = noeud->next;
		noeud->position = myplace;
		noeud->route = myRoute; // For PILS
		mycharge += params->cli[noeud->cour].demand;
		mytime += params->timeCost[noeud->prev->cour][noeud->cour] + params->cli[noeud->cour].serviceDuration;
		mycost += params->timeCost[noeud->prev->cour][noeud->cour];
		myReversalDistance += params->timeCost[noeud->cour][noeud->prev->cour] - params->timeCost[noeud->prev->cour][noeud->cour] ;
		myXvalue += params->cli[noeud->cour].coordX;
		myYvalue += params->cli[noeud->cour].coordY;
		noeud->cumulatedLoad = mycharge;
		noeud->cumulatedTime = mytime;
		noeud->cumulatedReversalDistance = myReversalDistance;
	}

	noeud->next = myRoute->depot; // For PILS: When recombining the routes, a depot may be associated to a different depotEnd, this fixes the pointers.
	myRoute->depot->prev = noeud;

	myRoute->duration = mytime;
	myRoute->load = mycharge;
	myRoute->penalty = excedentLength(mytime) + excedentCharge(mycharge);
	myRoute->penalizedCost = mycost + myRoute->penalty;
	myRoute->nbNodes = myplace;
	myRoute->reversalDistance = myReversalDistance;
	myRoute->polarAngle = atan2((myYvalue / (double)myplace) - params->cli[0].coordY, (myXvalue / (double)myplace) - params->cli[0].coordX);

	if (myRoute->depot->next->isDepot) emptyRoutes.insert(myRoute->cour); // If its an empty route, store it
	else emptyRoutes.erase(myRoute->cour); // Otherwise make sure its not part of the empty route vector

	if (isRI) // Reinitialize memories (RI uses a different memory than PI)
	{
		// Remember "when" this route has been last modified (to filter unnecessary move evaluations) 
		myRoute->whenLastModified = nbMoves+1; // There is a +1 since the move counter is incremeted after the route update
	}
	else
	{
		for (int i = 0; i <= params->nbClients; i++)
			costInsertClient[myRoute->cour][i] = 1.e30;
	}
}

double LocalSearch::computeSolutionCost()
{
	double sumPenalizedCost = 0.;
	for (int r = 0; r < params->nbVehicles; r++)
		sumPenalizedCost += routes[r].penalizedCost;
	return sumPenalizedCost;
}

void LocalSearch::loadIndividual(Individual * indiv)
{
	emptyRoutes.clear();
	nbMoves = -1; 
	for (int r = 0; r < params->nbVehicles; r++)
	{
		Node * myDepot = &depots[r];
		Node * myDepotFin = &depotsEnd[r];
		Route * myRoute = &routes[r];
		myDepot->prev = myDepotFin;
		myDepotFin->next = myDepot;
		if (!indiv->chromR[r].empty())
		{
			Node * myClient = &clients[indiv->chromR[r][0]];
			myClient->route = myRoute;
			myClient->prev = myDepot;
			myDepot->next = myClient;
			for (int i = 1; i < (int)indiv->chromR[r].size(); i++)
			{
				Node * myClientPred = myClient;
				myClient = &clients[indiv->chromR[r][i]]; 
				myClient->prev = myClientPred;
				myClientPred->next = myClient;
				myClient->route = myRoute;
			}
			myClient->next = myDepotFin;
			myDepotFin->prev = myClient;
		}
		else
		{
			myDepot->next = myDepotFin;
			myDepotFin->prev = myDepot;
		}
		updateRouteData(&routes[r], true);
	}

	for (int i = 1; i <= params->nbClients; i++)
		clients[i].whenLastTested = -1;
	nbMoves = 0;
}

void LocalSearch::exportIndividual(Individual * indiv)
{
	std::vector < std::pair <double, int> > routePolarAngles ;
	for (int r = 0; r < params->nbVehicles; r++)
		routePolarAngles.push_back(std::pair <double, int>(routes[r].polarAngle, r));
	std::sort(routePolarAngles.begin(), routePolarAngles.end());

	int pos = 0;
	for (int r = 0; r < params->nbVehicles; r++)
	{
		indiv->chromR[r].clear();
		Node * node = depots[routePolarAngles[r].second].next;
		while (!node->isDepot)
		{
			indiv->chromT[pos] = node->cour;
			indiv->chromR[r].push_back(node->cour);
			node = node->next;
			pos++;
		}
	}

	indiv->evaluateCompleteCost();
}

LocalSearch::LocalSearch(Params * params, Mining * mining) : params (params), mining(mining), maxRouteTime(params->durationLimit), vehicleCapacity(params->vehicleCapacity)
{
	clients = std::vector < Node >(params->nbClients + 1);
	routes = std::vector < Route >(params->nbVehicles);
	depots = std::vector < Node >(params->nbVehicles);
	depotsEnd = std::vector < Node >(params->nbVehicles);
	costInsertClient = std::vector < std::vector <double> >(params->nbVehicles, std::vector <double>(params->nbClients + 1, 1.e30));
	placeInsertClient = std::vector < std::vector <Node *> >(params->nbVehicles, std::vector <Node *>(params->nbClients + 1, NULL));
	for (int i = 0; i <= params->nbClients; i++) 
	{ 
		clients[i].cour = i; 
		clients[i].isDepot = false; 
		clients[i].isDepotEnd = false;
		clients[i].isPatternElement = false;
	}
	for (int i = 0; i < params->nbVehicles; i++)
	{
		routes[i].cour = i;
		routes[i].depot = &depots[i];
		routes[i].isPatternElement = false;
		depots[i].cour = 0;
		depots[i].isDepot = true;
		depots[i].isDepotEnd = false;
		depots[i].isPatternElement = false;
		depots[i].route = &routes[i];
		depotsEnd[i].cour = 0;
		depotsEnd[i].isDepot = true;
		depotsEnd[i].isDepotEnd = true;
		depotsEnd[i].isPatternElement = false;
		depotsEnd[i].route = &routes[i];
	}
	for (int i= 1 ; i <= params->nbClients ; i++) 
		orderNodes.push_back(i);
	
	for (int i = 0; i < (int)(params->factorPatternMining*(double)params->nbClients); i++) 
		orderPatterns.push_back(i);
}
