#ifndef LOCALSEARCH_H
#define LOCALSEARCH_H

#include "Individual.h"
#include "Mining.h"

/**
 * Author: Thibaut Vidal
 * 
 * All local search and PILS functions are implemented in this class.
 * We refer to PILS data structures in Mining.h/Mining.cpp
 * 
 */

struct Node ;
struct Route
{
	int cour;					// Route index
	int whenLastModified;		// When the route was last modified
	int nbNodes;				// Number of nodes in the route
	Node * depot;				// Pointer to the associated depot
	double duration;			// Total time on the route
	double load;				// Total load on the route
	double penalizedCost;		// Total penalized cost of the route
	double reversalDistance;	// Difference of cost if the route is reversed
	double penalty;				// Current sum of load and duration penalties
	double polarAngle;			// Polar angle of the barycenter of the route
	bool isPatternElement;		// Tells whether this route is part of the pattern under current consideration (PILS)
};

struct Node
{
	bool isDepot;						// Tells whether this node represents a depot or not
	bool isDepotEnd;					// Tells whether this node represents an "end depot" (sentinel node) or not
	bool isPatternElement;				// Tells whether this node is part of the pattern under current consideration (PILS)
	int cour;							// Node index
	int whenLastTested;					// Last time the moves associated to this node have been tested without any improvement
	int position;						// Position in the route
	Node * next;						// Next node in the route order
	Node * prev;						// Previous node in the route order
	Route * route;						// Pointer towards the associated route
	double cumulatedLoad;				// Cumulated load on this route until the customer (including itself)
	double cumulatedTime;				// Cumulated time on this route until the customer (including itself)
	double cumulatedReversalDistance;	// Difference of cost if the segment of route (0...cour) is reversed (useful for 2-opt moves with asymmetric problems)
};

// Used in PILS to track which segments still need to be connected
struct Segment
{
	Node * node1;						// First extremity of the segment
	Node * node2;						// Second extremity of the segment
	double distance;					// Distance over the segment
	double load;						// For checking load constraints in PILS (load on the considered segment)
	double time;						// For checking duration constaints in PILS (duration on the considered segment)
	double reversalDistance;			// To allow asymmetric problems in PILS (change of distance if the considered segment was reversed)
	double penalizedCost;				// Penalized cost of this segment
};

class LocalSearch
{

private:
	
	Params * params ;							// Problem parameters
	Mining * mining	;							// Mining structure
	double maxRouteTime;						// Route duration constraint
	double vehicleCapacity;						// Capacity constraint
	bool searchCompleted;						// Tells whether all moves have been evaluated without success
	int nbMoves;								// Number of moves applied during the local search
	std::vector < int > orderNodes;				// Randomized order for checking the nodes in the LS
	bool moveFoundSameDay;						// Temporary
	std::set < int > emptyRoutes;				// indices of all empty routes
	bool firstLoop;								// Says whether we are in the first loop of the search

	/* THE SOLUTION IS REPRESENTED AS A LINKED LIST OF ELEMENTS */
	std::vector < Node > clients;				// Elements representing clients (clients[0] is a sentinel and should not be accessed)
	std::vector < Node > depots;				// Elements representing depots
	std::vector < Node > depotsEnd;				// Duplicate of the depots to mark the end of the routes
	std::vector < Route > routes;				// Elements representing routes

	/* PILS FUNCTIONS */
	bool foundImprovingPILS;
	std::vector <int> involvedRoutes;											// Routes involved in the PILS
	std::vector <Segment> vectorSegmentsBegin;								// Segments considered in the recursion: pieces that start with a depot
	std::vector <Segment> vectorSegmentsMiddle;								// Segments considered in the recursion: middle pieces
	std::vector <Segment> vectorSegmentsEnd;								// Segments considered in the recursion: pieces that end with an end-depot
	std::vector< std::pair<Node *, Node *> > reconstructionActions;			// Reconnections which are currently selected in the recursion (for solution updates)
	std::vector< std::pair<Node *, Node *> > reversalActions;				// Reversals which are currently selected in the recursion (for solution updates)
	std::vector< std::pair<Node *, Node *> > bestReconstructionActions;		// Best reconnections found in the recursion (for solution updates)
	std::vector< std::pair<Node *, Node *> > bestReversalActions;			// Best reversals found in the recursion (for solution updates)
	double bestBudgetReconstruction;										// Represents the best solution improvement found in the recursive search (0 would mean a zero-cost move and positive would be a strict improvement)
	std::vector<int> orderPatterns;											// Structures for randomized enumeration
	
	// Operations on segments -- makes it easy to implement PILS
	void reverseSegment(Segment & mySegmentResult, const Segment & mySegment);								 // Reversal operator on segments
	void combineSegment(Segment & mySegmentResult, const Segment & mySegment1, const Segment & mySegment2);  // Concatenation operator on segments
	void initSegment(Segment & mySegmentResult, Node * node);												 // Initialize the segment with this node
	void appendSegment(Segment & mySegmentResult, Node * node);												 // update the segment to add another node
	
	// Recursive function to calculate the best reconnections
	// Needs to be called with foundImprovingPILS set to FALSE
	// (0...indexVectorBeginEnd) and (0...indexVectorMiddleEnd) represent the range which is currently under consideration in the vectors of segments
	// BudgetReconnection is the extra cost allowed (reconnections and possible excess of load or duration constraints)
	// This recursive function will either terminate with 
	// foundImprovingPILS is FALSE and reconstructionActions is empty, or
	// foundImprovingPILS is TRUE and reconstructionActions contains the pairs of nodes which needs to be joined
	void findReconnections(int indexVectorBeginEnd, int indexVectorMiddle, double budgetCost);

	/* MEMORY STRUCTURES */
	std::vector < std::vector < double > > costInsertClient;		// (PI) For each route and node, storing the cheapest insertion cost 
	std::vector < std::vector < Node * > > placeInsertClient;		// (PI) For each route and node, storing the cheapest insertion position
	
	/* TEMPORARY VARIABLES USED IN THE LOCAL SEARCH LOOPS */
	Node * noeudU ;
	Node * noeudX ;
    Node * noeudV ;
	Node * noeudY ;
	Route * routeU ;
	Route * routeV ;
	int noeudUPredCour, noeudUCour, noeudXCour, noeudXSuivCour ;	// UPred -> U -> X -> XSuiv
	int noeudVPredCour, noeudVCour, noeudYCour, noeudYSuivCour ;	// VPred -> V -> Y -> YSuiv
	double loadU, loadX, loadV, loadY;
	double serviceU, serviceX, serviceV, serviceY;
	double distUpU, distUX, distXU, distXXs, distUpX, distUpXs;
	double distVpV, distVY, distYYs ;
	double distUV, distVU, distVpU, distUpV, distXY, distVX, distVXs, distUY, distXYs, distYXs ;
	double penalityLength, penalityCapa;

	// Initializes some local variables and distances associated to routeU to avoid always querying the same values in the distance matrix
	void setLocalVariablesRouteU();

	// Initializes some local variables and distances associated to routeV to avoid always querying the same values in the distance matrix
	void setLocalVariablesRouteV();

	// Functions in charge of excess load and duration penalty calculations
	#define excedentLength(x) _excedentLength(x)
	//#define excedentLength(x) 0. // <--- Outcomment this line instead of the previous one to gain some speed if your problem does not include duration constraints
	inline double _excedentLength(double myLength) {return std::max<double>(0., myLength - maxRouteTime)*penalityLength;}
	inline double excedentCharge(double myCharge) {return std::max<double>(0., myCharge - vehicleCapacity)*penalityCapa;}

	/* RELOCATE MOVES */
	bool move1 (); // If U is a client node, remove U and insert it after V
	bool move2 (); // If U and X are client nodes, remove them and insert (U,X) after V
	bool move3 (); // If U and X are client nodes, remove them and insert (X,U) after V

	/* SWAP MOVES */
	bool move4 (); // If U and V are client nodes, swap U and V
	bool move5 (); // If U, X and V are client nodes, swap (U,X) and V
	bool move6 (); // If (U,X) and (V,Y) are client nodes, swap (U,X) and (V,Y) 
	 
	/* 2-OPT and 2-OPT* MOVES */
	bool move7 (); // If route(U) == route(V), replace (U,X) and (V,Y) by (U,V) and (X,Y)
	bool move8 (); // If route(U) != route(V), replace (U,X) and (V,Y) by (U,V) and (X,Y)
	bool move9 (); // If route(U) != route(V), replace (U,X) and (V,Y) by (U,Y) and (V,X)

	/* ROUTINES TO UPDATE THE SOLUTIONS */
	static void insertNode(Node * U, Node * V);			// Solution update: Insert U after V
	static void swapNode(Node * U, Node * V) ;			// Solution update: Swap U and V							   
	void updateRouteData(Route * myRoute, bool isRI);	// Updates the preprocessed data of a route

	/* ROUTINES USEFUL FOR DEBUGGING */
	double computeSolutionCost();						// Only to be used for debugging, supposes all routes are updated. Reports the sum of the route costs

	/* GENERAL FUNCTIONS */   
	int localSearchRI(); // RI Local Search
	int localSearchPI(); // PI Local Search
	void localSearchPILS(); // PILS Local Search

	public:

	// Loading an initial solution into the local search
	void loadIndividual(Individual * indiv);

	// Exporting the LS solution into an individual
	void exportIndividual(Individual * indiv);

	// Run the local search
	void run(Individual * indiv);

	// Repairing an individual
	void repair(Individual * indiv);

	// Constructor
	LocalSearch(Params * params, Mining * mining);
};

#endif
