#ifndef MINING_H
#define MINING_H

#include "Individual.h"
#include <map>
#include <queue>

/**
 * Author: Thibaut Vidal & Italo Santana
 * 
 * PILS data structures and basic getters and setters are defined in this file.
 * Each MapAndHeap object stands for a Binary Heap that stores the k most frequent patterns.
 * Mining class has a vector of MapAndHeap, where each of its position regards a pattern size.
 * 
 */

struct MapElement { int frequency = 1; int indexMinHeap = -1; 
	#if ACTIVE_SIDE_EXPERIMENTS > 0
		double bestSolutionValue = 1.e30; //+infinity
	#endif
};
typedef std::map<std::vector<int>, MapElement>::iterator MyIterator;

class MapAndHeap
{

public:
	
	// Map to store all sequences found in the solutions (typically based on a red-black tree, complexity to insert = log(X) * sequenceSize in the worse case), where X represents the number of elements
	// An Unordered_Map could be a slightly better alternative to remove the log(X) factor, but subject to caution when defining a good hashing function 
	// An efficient Trie (with a Map at each level) would be even better but require more coding effort... this is only useful if insertions become a bottleneck
	std::map<std::vector<int>, MapElement> myMap;

	int nbElementsHeap ;				// current number of elements in the heap
	int heapMaxCapacity ;				// capacity of the heap
	std::vector < MyIterator > elementsHeap ;	// elements of the heap (iterators towards the elements of the map)

	void heapify(int idx)
	{
		int left = 2*idx + 1;
		int right = 2*idx + 2;
		int smallest = idx;
		if (left < heapMaxCapacity  && elementsHeap[left]->second.frequency  < elementsHeap[smallest]->second.frequency) smallest = left;
		if (right < heapMaxCapacity && elementsHeap[right]->second.frequency < elementsHeap[smallest]->second.frequency) smallest = right;

		if (smallest != idx)
		{
			// Always update the associated index in the map data structure
			elementsHeap[smallest]->second.indexMinHeap = idx;
			elementsHeap[idx]->second.indexMinHeap = smallest;

			// Swap elements in the min heap
			MyIterator temp = elementsHeap[smallest];
			elementsHeap[smallest] = elementsHeap[idx];
			elementsHeap[idx] = temp;
			heapify(smallest);
		}
	}

	void insert(const std::vector <int> & mySequence)
	{
		/* MANAGEMENT OF THE MAP */
		auto result = myMap.insert(std::pair <std::vector<int>, MapElement>(mySequence, MapElement()));
		MyIterator it = result.first;                      // map.insert returns a pair whose first member is an iterator towards the data which has been inserted
		bool insertionPerformed = result.second;           // map.insert returns a pair whose second member is a boolean valued to TRUE iff the insertion has been done
		if (!insertionPerformed) it->second.frequency++;   // If the element already existed in the map, we increase its frequency

		/* MANAGEMENT OF THE HEAP */
		if (it->second.indexMinHeap != -1)
		{
			// Element is already in the heap but frequency has changed, restore min heap property (only if the structure is already organized as a heap)
			if (nbElementsHeap == heapMaxCapacity) heapify(it->second.indexMinHeap);
		}
		else if (nbElementsHeap < heapMaxCapacity)
		{
			// Element is not in the heap and there is space, add it
			elementsHeap[nbElementsHeap] = it;
			it->second.indexMinHeap = nbElementsHeap;
			nbElementsHeap++;
			
			// Once the full size is attained, we enforce heap property in the entire vector (only need to do that once)
			if (nbElementsHeap == heapMaxCapacity) 
				for (int i = (heapMaxCapacity - 1) / 2; i >= 0; i--)
					heapify(i);
		}
		else if (it->second.frequency > elementsHeap[0]->second.frequency)
		{
			// Element is not in the heap but more frequent than the root, therefore it replaces the root element
			elementsHeap[0]->second.indexMinHeap = -1;
			elementsHeap[0] = it;
			it->second.indexMinHeap = 0;
			heapify(0);
		}
	}

	MapAndHeap(int heapMaxCapacity) : heapMaxCapacity(heapMaxCapacity)
	{
		elementsHeap = std::vector < MyIterator > (heapMaxCapacity);
		nbElementsHeap = 0;
	}
};

// This stores statistics on frequent sequences of visits found within the solutions
// Can return a limited number of such sequences, to be exploited in a local search.
class Mining
{

private:

	// Access to the problem parameters
	Params * params ;

	// Each element of this structure is designed to store all "most-frequent" sequences of size k for k=2...params->maxSeqMining
	std::vector < MapAndHeap > myMapAndHeap;

public:

	// Collects all patterns of all desired size values from this solution
	void collect(Individual * indiv)
	{
		params->totalExtractions++;
		double tcpu1 = (double)clock() / (double)CLOCKS_PER_SEC;
		for (int sequenceSize = params->minSeqMining; sequenceSize <= params->maxSeqMining; sequenceSize++)
		{
			for (int r = 0; r < params->nbVehicles; r++) // Iterating on the routes of the current solution
			{
				std::vector<int> & myRoute = indiv->chromR[r];
				for (int indexEnd = sequenceSize ; indexEnd <= myRoute.size(); indexEnd++)
				{
					std::vector <int> mySequence(myRoute.begin() + indexEnd - sequenceSize, myRoute.begin() + indexEnd);

					if(mySequence[0] > mySequence[sequenceSize-1]) //Avoids that two symmetric patterns are inserted
						std::reverse(mySequence.begin(),mySequence.end());

					myMapAndHeap[sequenceSize].insert(mySequence);
				}
			}
		}
		double tcpu2 = (double)clock() / (double)CLOCKS_PER_SEC;
		params->totalExtractionTimePILS += tcpu2 - tcpu1;
	}

	// Accessor to the i^th frequent sequence in the heap for this sequence size. 
	// sequenceSize must be greater than 1
	const std::vector <int> & frequentSequence(int sequenceSize, int i)
	{
		// As a safety check, verify that the map and heap elements are indeed always pointing to each other
		MyIterator myIt = myMapAndHeap[sequenceSize].elementsHeap[i];
		if (myIt->second.indexMinHeap != i) throw std::string("ERROR linking in Pattern Mining structure");
		return myIt->first;
	}

	// Accessor to the number of sequences currently selected for a given sequence size
	int getNbSequences(int sequenceSize)
	{
		return myMapAndHeap[sequenceSize].nbElementsHeap;
	}

	#if ACTIVE_SIDE_EXPERIMENTS > 0
		// Accessor to the i^th frequent sequence in the heap for this sequence size. 
		// sequenceSize must be greater than 1
		MyIterator getHeapElement(int sequenceSize, int i)
		{
			// As a safety check, verify that the map and heap elements are indeed always pointing to each other
			MyIterator myIt = myMapAndHeap[sequenceSize].elementsHeap[i];
			if (myIt->second.indexMinHeap != i) throw std::string("ERROR linking in Pattern Mining structure");
			return myIt;
		}
		void updatePatternCostValue(const std::vector <int> & mySequence, double newCostvalue)
		{
			int patternSize = mySequence.size();

			if(myMapAndHeap[patternSize].myMap.find(mySequence) != myMapAndHeap[patternSize].myMap.end())
			{
				if(newCostvalue < myMapAndHeap[patternSize].myMap[mySequence].bestSolutionValue)
					myMapAndHeap[patternSize].myMap[mySequence].bestSolutionValue = newCostvalue;
			}
		}
	#endif

	// Constructor
	Mining(Params * params) : params(params)
	{
		myMapAndHeap = std::vector < MapAndHeap >(params->maxSeqMining + 1, MapAndHeap((int)(params->factorPatternMining*(double)params->nbClients)));
	}

};

#if ACTIVE_SIDE_EXPERIMENTS > 0 

class SideExperiments
{
private:
	double costOptimalSolution;

	// Big tour format including starting and ending depots. Moreover, only one depot is considered between two routes.
	std::vector<int> optimalSolution;

	// Problem parameters
	Params *params;

	// Mining variable
	Mining *mining;

	static bool descSortPatternList(MyIterator i, MyIterator j)
	{
		return (i->second.frequency > j->second.frequency);
	}

public:

	void exportExperiment1()
	{
		//For each pattern size, sort all patterns by their frequency.
		//For each pattern size, the results are reported in "bins" format.
		printf("Exp1 Processing\n");
		std::string output = "bin_size(nbClients):" + std::to_string(params->nbClients) + "\n";
		for (int sizePattern = params->minSeqMining; sizePattern <= params->maxSeqMining; sizePattern++)
		{
			std::vector < MyIterator > allPatterns;
			//Get all patterns from the same size
			for (int patIndex = 0; patIndex < mining->getNbSequences(sizePattern); patIndex++)
			{
				MyIterator myIt = mining->getHeapElement(sizePattern, patIndex);
				allPatterns.push_back(myIt);
			}

			//Sort this pattern list by frequency
			std::sort(allPatterns.begin(), allPatterns.end(), descSortPatternList);
			output += std::to_string(allPatterns.size()) + "\n";
			int counter = 0;
			int counterAll = 0;
			int patIndex;
			int bin_index = 1;

			for (patIndex = 0; patIndex < allPatterns.size(); patIndex++)
			{
				//Separating each counter in a specific bin 
				if (patIndex % params->nbClients == 0 && patIndex > 0)
				{
					output += "sizePattern:" + std::to_string(sizePattern) + ";bin:" + std::to_string(bin_index) + ";percentage:" + std::to_string((counter*1.0) / counterAll) + ";nbPatterns:" + std::to_string((int)counterAll) + "\n";
					counter = 0;
					counterAll = 0;
					bin_index++;
				}

				const std::vector<int> & pattern = allPatterns[patIndex]->first;

				//Check if current pattern is within optimal solution 
				for (int indexEnd = sizePattern; indexEnd <= optimalSolution.size(); indexEnd++)
				{
					std::vector <int> optPattern(optimalSolution.begin() + indexEnd - sizePattern, optimalSolution.begin() + indexEnd);

					//If first element of the pattern is greater than the last one, we must reverse it.
					if (optPattern[0] > optPattern[sizePattern - 1])
						std::reverse(optPattern.begin(), optPattern.end());

					int i = 0;
					while (i < sizePattern)
					{
						if (optPattern[i] != pattern[i])
							break;
						i++;
					}
					//if both patterns match, we do not need to visit the remaining elements of the solution
					if (i == sizePattern)
					{
						counter++;
						break;
					}
				}
				counterAll++;
			}
			output += "sizePattern:" + std::to_string(sizePattern) + ";bin:" + std::to_string(bin_index) + ";percentage:" + std::to_string((counter*1.0) / counterAll) + ";nbPatterns:" + std::to_string((int)counterAll) + "\n";
			output += "\n";
		}
		std::ofstream myFile(params->pathToSolution + ".exp1", std::ios::out);
		myFile << output;
		myFile.close();
		printf("Exp1 - END\n");
	}

	void updatePatternCostExperiment2(Individual *offspring)
	{
		for (int sequenceSize = params->minSeqMining; sequenceSize <= params->maxSeqMining; sequenceSize++)
		{
			for (int r = 0; r < params->nbVehicles; r++) // Iterating on the routes of the given solution
			{
				std::vector<int> & myRoute = offspring->chromR[r];
				for (int indexEnd = sequenceSize; indexEnd <= myRoute.size(); indexEnd++)
				{
					std::vector <int> mySequence(myRoute.begin() + indexEnd - sequenceSize, myRoute.begin() + indexEnd);

					if (mySequence[0] > mySequence[sequenceSize - 1]) //Avoids that two symmetric patterns are inserted
						std::reverse(mySequence.begin(), mySequence.end());
					if (offspring->myCostSol.penalizedCost < costOptimalSolution)
					{
						printf("feasible? %d, %lf, %lf \n", offspring->isFeasible, offspring->myCostSol.penalizedCost, costOptimalSolution);
						printf("Algo errado aqui;");
						exit(0);
					}
					mining->updatePatternCostValue(mySequence, offspring->myCostSol.penalizedCost);
				}
			}
		}
	}

	void exportExperiment2()
	{
		printf("Exp2 Processing\n");
		std::string output = "OPT: " + std::to_string(costOptimalSolution) + "\n";
		output += "pattern,frequency,best_solution,gapOPT\n";
		for (int sizePattern = params->minSeqMining; sizePattern <= params->maxSeqMining; sizePattern++)
		{
			std::vector < MyIterator > allPatterns;
			//Get all patterns from the same size
			for (int patIndex = 0; patIndex < mining->getNbSequences(sizePattern); patIndex++)
			{
				MyIterator myIt = mining->getHeapElement(sizePattern, patIndex);
				if (myIt->second.bestSolutionValue == 1.e30)
				{
					continue;
				}
				allPatterns.push_back(myIt);
			}

			//Sort this pattern list by frequency
			std::sort(allPatterns.begin(), allPatterns.end(), descSortPatternList);
			int patIndex = 0;
			int bin_index = 1;
			int counter = 0;
			int counterAll = 0;
			double accumulatedGap = 0.0;
			for (patIndex = 0; patIndex < allPatterns.size(); patIndex++)
			{
				//Separating each counter in a specific bin 
				if (patIndex % params->nbClients == 0 && patIndex > 0)
				{
					output += "sizePattern:" + std::to_string(sizePattern) + ";bin:" + std::to_string(bin_index) + ";avg_gap:" + std::to_string(accumulatedGap / counterAll) + ";nbPatterns:" + std::to_string((int)counterAll) + "\n";
					counterAll = 0;
					bin_index++;
					accumulatedGap = 0.0;
				}

				const std::vector<int> & pattern = allPatterns[patIndex]->first;
				accumulatedGap += (allPatterns[patIndex]->second.bestSolutionValue - this->costOptimalSolution)*100.0 / this->costOptimalSolution;
				counterAll++;
			}
			output += "sizePattern:" + std::to_string(sizePattern) + ";bin:" + std::to_string(bin_index) + ";avg_gap:" + std::to_string(accumulatedGap / counterAll) + ";nbPatterns:" + std::to_string((int)counterAll) + "\n";
			output += "\n";
		}

		std::ofstream myFile(params->pathToSolution + ".exp2", std::ios::out);
		myFile << output;
		myFile.close();
		printf("Exp2 - END\n");
	}

	void readOptimalSolutionCharacteristics()
	{
		std::size_t pos = params->pathToInstance.find("X-");      // position of "X-" in the instance name
		std::string basename = params->pathToInstance.substr(pos);
		if (basename.compare("X-n195-k51.vrp") == 0)
		{
			optimalSolution = { 0, 159, 0, 55, 38, 18, 23, 47, 0, 50, 3, 0, 57, 9, 72, 26, 14, 0, 21, 89, 170, 92, 0, 58, 35, 44, 0, 36, 78, 107, 11, 167, 161, 98, 0, 117, 131, 189, 67, 0, 86, 106, 160, 0, 31, 54, 29, 104, 0, 42, 164, 115, 91, 108, 180, 0, 64, 52, 0, 12, 39, 20, 90, 0, 17, 155, 120, 24, 0, 66, 152, 85, 71, 188, 0, 48, 13, 118, 0, 33, 168, 143, 0, 88, 61, 53, 74, 0, 10, 135, 0, 122, 8, 114, 46, 0, 147, 163, 191, 0, 130, 99, 150, 183, 0, 174, 158, 142, 103, 134, 0, 110, 113, 176, 0, 100, 186, 187, 105, 136, 0, 4, 156, 119, 0, 144, 157, 166, 0, 178, 77, 49, 0, 2, 87, 37, 123, 149, 0, 95, 28, 177, 0, 68, 80, 0, 69, 171, 109, 111, 102, 0, 6, 1, 0, 25, 22, 0, 127, 94, 141, 140, 0, 128, 133, 175, 97, 153, 121, 0, 27, 51, 65, 30, 0, 60, 79, 0, 62, 162, 116, 0, 73, 41, 0, 34, 193, 96, 83, 138, 0, 132, 63, 0, 125, 148, 165, 0, 32, 139, 154, 179, 0, 101, 137, 0, 185, 16, 129, 126, 0, 59, 56, 145, 0, 190, 172, 82, 192, 0, 146, 124, 184, 112, 45, 0, 173, 151, 70, 169, 0, 181, 194, 81, 7, 0, 84, 76, 15, 40, 19, 182, 0, 5, 43, 75, 93, 0 };
			costOptimalSolution = 44225;
		}
		else if (basename.compare("X-n200-k36.vrp") == 0)
		{
			optimalSolution = { 0, 47, 36, 60, 157, 16, 0, 29, 44, 110, 108, 23, 0, 69, 81, 133, 170, 102, 0, 181, 138, 100, 168, 99, 0, 59, 171, 169, 119, 196, 0, 75, 6, 117, 61, 49, 172, 0, 79, 162, 11, 84, 178, 0, 31, 32, 12, 20, 10, 0, 45, 113, 48, 58, 63, 0, 28, 179, 185, 77, 5, 0, 18, 121, 154, 78, 125, 40, 0, 74, 70, 188, 163, 46, 57, 0, 150, 65, 88, 106, 85, 27, 0, 194, 68, 67, 62, 151, 0, 76, 4, 129, 109, 142, 26, 92, 0, 55, 17, 14, 145, 72, 190, 167, 186, 0, 52, 187, 39, 182, 193, 87, 140, 98, 0, 115, 130, 147, 97, 83, 111, 0, 152, 131, 7, 134, 123, 53, 0, 120, 73, 156, 94, 180, 0, 103, 38, 89, 19, 191, 0, 159, 132, 82, 42, 101, 0, 153, 173, 195, 54, 183, 135, 0, 3, 50, 35, 105, 155, 0, 93, 8, 166, 66, 198, 0, 175, 122, 114, 146, 30, 149, 0, 13, 176, 25, 43, 24, 41, 0, 189, 95, 104, 116, 177, 0, 33, 22, 51, 15, 71, 34, 0, 118, 136, 91, 161, 158, 0, 199, 184, 37, 56, 128, 0, 141, 143, 137, 96, 127, 0, 90, 64, 21, 164, 174, 0, 86, 1, 112, 144, 9, 0, 197, 148, 192, 107, 2, 126, 0, 160, 124, 165, 139, 80, 0 };
			costOptimalSolution = 58578;
		}
		else if (basename.compare("X-n204-k19.vrp") == 0)
		{
			optimalSolution = { 0, 145, 142, 194, 90, 0, 5, 176, 2, 104, 190, 114, 193, 159, 98, 118, 0, 8, 166, 121, 112, 9, 187, 111, 203, 144, 116, 148, 0, 188, 172, 133, 127, 154, 195, 132, 62, 70, 124, 151, 0, 109, 152, 18, 182, 200, 12, 136, 135, 202, 150, 106, 0, 199, 119, 48, 36, 101, 53, 34, 15, 100, 78, 32, 0, 177, 167, 131, 169, 164, 16, 198, 181, 155, 123, 170, 0, 173, 65, 143, 128, 72, 30, 174, 52, 27, 49, 75, 83, 0, 160, 113, 122, 157, 105, 162, 71, 179, 99, 120, 186, 79, 0, 63, 50, 45, 38, 189, 26, 183, 153, 77, 96, 0, 19, 80, 46, 20, 82, 35, 37, 60, 64, 88, 0, 58, 29, 95, 67, 7, 84, 39, 33, 43, 17, 0, 42, 21, 73, 69, 23, 24, 28, 6, 86, 92, 85, 0, 41, 31, 59, 4, 25, 91, 54, 13, 87, 3, 74, 0, 201, 137, 55, 156, 110, 66, 40, 94, 56, 11, 51, 61, 0, 197, 139, 89, 93, 165, 180, 158, 146, 103, 184, 163, 57, 0, 22, 125, 117, 81, 44, 10, 102, 14, 147, 185, 140, 141, 0, 191, 138, 115, 168, 175, 130, 161, 171, 47, 129, 68, 0, 76, 97, 196, 108, 178, 107, 149, 192, 126, 134, 1, 0 };
			costOptimalSolution = 19565;
		}
		else if (basename.compare("X-n209-k16.vrp") == 0)
		{
			optimalSolution = { 0, 169, 105, 78, 136, 53, 58, 187, 77, 111, 97, 75, 178, 40, 119, 0, 20, 149, 188, 122, 181, 142, 147, 106, 15, 35, 25, 81, 39, 112, 0, 65, 18, 132, 68, 177, 128, 50, 175, 45, 183, 131, 16, 0, 14, 33, 148, 206, 155, 95, 5, 57, 82, 207, 71, 63, 0, 110, 84, 98, 152, 44, 49, 26, 156, 70, 100, 205, 42, 0, 34, 38, 56, 173, 203, 103, 29, 54, 72, 47, 199, 161, 88, 190, 0, 200, 176, 130, 101, 108, 74, 96, 172, 37, 166, 7, 129, 31, 102, 0, 87, 163, 104, 114, 135, 123, 73, 170, 192, 67, 99, 27, 17, 165, 91, 0, 64, 134, 153, 11, 28, 115, 36, 202, 185, 179, 154, 150, 140, 89, 0, 160, 168, 52, 133, 59, 137, 79, 146, 55, 164, 76, 19, 21, 194, 0, 145, 13, 62, 117, 139, 184, 41, 195, 66, 1, 83, 2, 113, 0, 93, 30, 197, 85, 171, 158, 32, 109, 189, 24, 157, 182, 61, 120, 51, 0, 198, 69, 186, 180, 22, 138, 121, 167, 3, 151, 86, 10, 118, 0, 208, 127, 43, 60, 9, 4, 124, 162, 8, 90, 126, 174, 48, 0, 159, 144, 143, 204, 80, 201, 0, 196, 6, 125, 12, 193, 92, 107, 141, 46, 94, 23, 191, 116, 0 };
			costOptimalSolution = 30656;
		}
		else if (basename.compare("X-n214-k11.vrp") == 0)
		{
			optimalSolution = { 0, 33, 78, 76, 166, 180, 175, 97, 136, 182, 115, 59, 173, 50, 201, 46, 149, 10, 206, 191, 0, 189, 133, 87, 77, 139, 141, 130, 188, 192, 43, 88, 29, 74, 94, 22, 81, 56, 195, 159, 125, 73, 38, 39, 49, 47, 0, 13, 181, 168, 144, 48, 62, 91, 147, 2, 212, 156, 177, 204, 104, 157, 41, 5, 129, 11, 40, 167, 135, 124, 0, 80, 120, 93, 82, 27, 117, 53, 45, 132, 68, 63, 17, 187, 186, 200, 64, 0, 55, 36, 162, 160, 30, 196, 3, 199, 60, 142, 18, 110, 176, 108, 42, 79, 197, 131, 101, 0, 155, 21, 126, 198, 9, 134, 146, 90, 86, 54, 119, 171, 178, 194, 19, 15, 98, 34, 163, 99, 0, 96, 137, 172, 7, 83, 210, 92, 179, 123, 158, 105, 23, 70, 116, 6, 169, 0, 4, 72, 203, 35, 1, 111, 57, 128, 165, 67, 154, 112, 202, 211, 174, 140, 143, 0, 31, 107, 32, 121, 12, 127, 52, 25, 170, 8, 89, 69, 61, 85, 100, 28, 75, 0, 58, 103, 207, 44, 145, 150, 184, 95, 113, 122, 114, 16, 14, 66, 37, 164, 51, 183, 109, 205, 20, 161, 190, 153, 0, 193, 106, 65, 71, 138, 152, 209, 26, 213, 148, 102, 185, 118, 208, 24, 84, 151, 0 };
			costOptimalSolution = 10856;
		}
		else if (basename.compare("X-n219-k73.vrp") == 0)
		{
			optimalSolution = { 0, 53, 96, 128, 0, 153, 165, 172, 0, 112, 3, 93, 0, 217, 31, 152, 0, 159, 175, 37, 0, 88, 116, 83, 0, 57, 102, 25, 0, 12, 137, 74, 0, 205, 36, 76, 0, 38, 40, 150, 0, 209, 15, 22, 0, 70, 20, 41, 0, 212, 69, 136, 0, 99, 78, 18, 0, 171, 58, 156, 0, 56, 218, 13, 0, 2, 65, 72, 0, 178, 190, 71, 0, 144, 200, 148, 0, 48, 7, 91, 0, 33, 29, 23, 0, 43, 169, 161, 0, 34, 133, 130, 0, 168, 92, 59, 0, 134, 89, 211, 0, 210, 201, 207, 0, 104, 118, 154, 0, 11, 14, 80, 0, 155, 181, 125, 0, 185, 166, 0, 174, 73, 17, 0, 19, 213, 184, 0, 182, 199, 195, 0, 129, 158, 198, 0, 131, 138, 183, 0, 63, 42, 187, 0, 21, 44, 203, 0, 28, 100, 126, 0, 147, 85, 162, 0, 94, 186, 145, 0, 196, 120, 194, 0, 119, 8, 215, 0, 189, 179, 139, 0, 111, 45, 10, 0, 143, 68, 105, 0, 170, 123, 47, 0, 30, 173, 146, 0, 50, 103, 216, 0, 132, 62, 108, 0, 55, 54, 151, 0, 141, 98, 82, 0, 27, 122, 26, 0, 142, 163, 101, 0, 117, 49, 24, 0, 9, 5, 176, 0, 149, 77, 177, 0, 106, 197, 86, 0, 188, 32, 39, 0, 60, 140, 35, 0, 110, 107, 109, 0, 192, 167, 16, 0, 67, 206, 6, 0, 81, 204, 113, 0, 180, 4, 191, 0, 95, 52, 61, 0, 66, 160, 84, 0, 214, 79, 202, 0, 114, 46, 75, 0, 121, 135, 90, 0, 157, 124, 193, 0, 127, 64, 51, 0, 97, 1, 208, 0, 164, 115, 87, 0 };
			costOptimalSolution = 117595;
		}
		else if (basename.compare("X-n223-k34.vrp") == 0)
		{
			optimalSolution = { 0, 66, 120, 123, 94, 135, 162, 153, 78, 0, 24, 31, 130, 181, 69, 0, 132, 52, 5, 38, 121, 216, 0, 12, 96, 63, 48, 64, 0, 194, 151, 142, 219, 139, 178, 169, 0, 208, 217, 183, 190, 188, 70, 0, 199, 113, 128, 147, 89, 165, 212, 0, 21, 37, 80, 4, 152, 192, 26, 0, 174, 163, 122, 186, 77, 0, 221, 118, 205, 46, 182, 0, 144, 127, 155, 14, 47, 214, 116, 0, 156, 138, 198, 67, 60, 179, 0, 140, 206, 161, 172, 19, 0, 99, 13, 28, 146, 134, 53, 210, 0, 211, 97, 50, 110, 33, 83, 40, 0, 41, 84, 62, 76, 45, 0, 98, 91, 29, 72, 2, 16, 61, 10, 0, 51, 103, 23, 36, 30, 73, 101, 57, 86, 7, 104, 56, 0, 65, 59, 55, 85, 213, 54, 0, 79, 93, 18, 75, 81, 109, 25, 0, 6, 11, 34, 3, 90, 17, 0, 20, 35, 82, 74, 0, 136, 95, 100, 0, 68, 167, 49, 106, 58, 180, 187, 168, 189, 42, 126, 117, 0, 131, 129, 222, 218, 164, 22, 0, 200, 141, 170, 87, 27, 158, 204, 0, 119, 220, 114, 143, 137, 171, 203, 0, 108, 44, 43, 148, 0, 92, 154, 1, 105, 32, 111, 0, 112, 176, 149, 173, 202, 0, 197, 88, 125, 150, 107, 184, 133, 71, 196, 8, 9, 0, 177, 193, 166, 209, 102, 0, 185, 39, 124, 15, 207, 215, 157, 175, 0, 195, 160, 201, 191, 115, 145, 159, 0 };
			costOptimalSolution = 40437;
		}
		else if (basename.compare("X-n228-k23.vrp") == 0)
		{
			optimalSolution = { 0, 129, 227, 140, 27, 184, 176, 118, 106, 42, 0, 155, 28, 26, 192, 218, 0, 69, 124, 114, 51, 32, 104, 173, 133, 74, 158, 62, 146, 222, 80, 170, 156, 0, 90, 35, 58, 2, 103, 29, 142, 0, 193, 191, 207, 18, 189, 186, 78, 128, 86, 70, 0, 101, 225, 77, 30, 202, 68, 44, 195, 38, 0, 220, 61, 1, 40, 57, 100, 83, 159, 172, 206, 110, 76, 47, 203, 0, 117, 151, 212, 137, 36, 14, 4, 148, 0, 182, 72, 167, 50, 88, 208, 20, 13, 177, 67, 145, 98, 0, 52, 116, 82, 92, 144, 162, 33, 39, 112, 164, 171, 0, 217, 12, 37, 65, 75, 134, 15, 0, 181, 201, 190, 152, 64, 6, 108, 139, 165, 97, 122, 196, 0, 71, 194, 200, 213, 25, 59, 109, 141, 198, 91, 205, 0, 119, 56, 5, 163, 138, 99, 221, 136, 111, 121, 127, 79, 224, 0, 16, 19, 0, 216, 84, 126, 169, 48, 214, 3, 54, 149, 0, 160, 153, 161, 10, 180, 46, 188, 199, 63, 130, 135, 94, 209, 95, 166, 34, 178, 41, 0, 24, 9, 107, 89, 123, 0, 113, 174, 143, 210, 215, 96, 147, 7, 105, 226, 23, 0, 60, 157, 73, 115, 49, 43, 31, 21, 93, 120, 0, 183, 223, 132, 102, 168, 87, 11, 53, 179, 197, 150, 187, 0, 66, 81, 22, 17, 175, 0, 204, 219, 85, 45, 131, 154, 8, 55, 185, 125, 211, 0 };
			costOptimalSolution = 25742;
		}
		else if (basename.compare("X-n233-k16.vrp") == 0)
		{
			optimalSolution = { 0, 124, 168, 115, 138, 227, 136, 137, 66, 127, 19, 7, 62, 80, 40, 0, 48, 109, 86, 23, 3, 58, 82, 31, 0, 112, 166, 4, 68, 106, 2, 116, 65, 0, 110, 38, 64, 28, 8, 93, 77, 27, 0, 193, 224, 191, 145, 6, 114, 84, 87, 45, 71, 92, 108, 172, 0, 39, 180, 135, 176, 117, 51, 67, 99, 228, 223, 46, 126, 75, 78, 20, 15, 52, 74, 230, 0, 153, 222, 197, 111, 30, 85, 17, 5, 129, 183, 141, 130, 219, 175, 118, 218, 190, 37, 147, 25, 63, 177, 113, 90, 133, 47, 0, 76, 41, 79, 29, 101, 16, 97, 43, 0, 55, 60, 1, 42, 81, 89, 226, 35, 56, 0, 170, 57, 10, 220, 61, 14, 208, 91, 155, 0, 21, 206, 173, 139, 189, 200, 164, 181, 0, 11, 0, 215, 192, 131, 53, 59, 194, 18, 100, 120, 134, 0, 151, 160, 182, 211, 221, 122, 143, 213, 148, 165, 98, 54, 205, 50, 0, 13, 179, 34, 132, 142, 123, 169, 146, 186, 128, 198, 207, 171, 225, 156, 184, 188, 187, 185, 178, 195, 201, 214, 199, 49, 72, 0, 102, 159, 162, 174, 217, 119, 158, 210, 70, 144, 196, 88, 9, 36, 167, 44, 95, 94, 12, 140, 83, 231, 202, 103, 216, 203, 22, 104, 149, 0, 107, 212, 161, 157, 229, 121, 33, 209, 125, 154, 150, 32, 105, 73, 26, 24, 204, 232, 152, 96, 69, 163, 0 };
			costOptimalSolution = 19230;
		}
		else if (basename.compare("X-n237-k14.vrp") == 0)
		{
			optimalSolution = { 0, 175, 97, 71, 199, 209, 141, 17, 75, 226, 187, 176, 63, 4, 163, 194, 56, 77, 39, 0, 230, 23, 236, 54, 69, 197, 119, 140, 123, 103, 190, 144, 171, 6, 170, 91, 165, 185, 0, 200, 18, 19, 55, 42, 154, 99, 60, 193, 12, 127, 208, 139, 32, 134, 156, 125, 211, 0, 29, 120, 20, 159, 147, 76, 36, 216, 16, 98, 64, 142, 15, 92, 136, 110, 81, 169, 0, 177, 43, 8, 196, 51, 48, 146, 215, 118, 25, 85, 100, 65, 108, 47, 132, 9, 38, 0, 28, 109, 0, 10, 13, 102, 40, 133, 207, 220, 126, 90, 30, 24, 151, 72, 44, 225, 67, 219, 66, 0, 62, 70, 222, 49, 113, 101, 79, 233, 174, 94, 37, 117, 188, 41, 191, 172, 80, 180, 0, 130, 2, 145, 68, 149, 52, 88, 128, 202, 34, 96, 124, 204, 234, 27, 195, 164, 21, 0, 192, 183, 14, 122, 50, 150, 189, 59, 82, 179, 181, 157, 78, 137, 186, 235, 212, 46, 0, 229, 84, 26, 228, 210, 213, 61, 5, 57, 161, 3, 198, 104, 227, 107, 231, 89, 168, 0, 129, 135, 87, 114, 162, 201, 205, 166, 217, 223, 33, 155, 218, 138, 22, 152, 214, 35, 0, 173, 115, 86, 158, 31, 111, 206, 182, 45, 53, 221, 121, 112, 93, 58, 95, 73, 7, 0, 224, 184, 148, 1, 203, 167, 83, 105, 143, 232, 178, 153, 116, 160, 11, 131, 74, 106, 0 };
			costOptimalSolution = 27042;
		}
		else if (basename.compare("X-n242-k48.vrp") == 0)
		{
			optimalSolution = { 0, 128, 63, 143, 131, 7, 34, 59, 0, 238, 192, 161, 89, 0, 155, 108, 32, 130, 38, 0, 210, 112, 16, 240, 0, 11, 43, 181, 173, 0, 100, 133, 149, 136, 118, 0, 165, 176, 75, 10, 0, 65, 5, 223, 142, 73, 81, 0, 151, 77, 172, 189, 0, 132, 97, 180, 183, 0, 91, 39, 195, 197, 152, 0, 159, 56, 113, 235, 0, 49, 114, 0, 186, 52, 179, 185, 218, 0, 21, 140, 60, 83, 87, 0, 224, 30, 201, 8, 0, 105, 119, 232, 230, 205, 35, 0, 69, 92, 124, 184, 95, 70, 138, 0, 67, 123, 237, 110, 0, 25, 20, 50, 241, 231, 166, 0, 208, 156, 90, 19, 47, 0, 211, 3, 88, 80, 33, 31, 206, 0, 217, 18, 170, 214, 71, 62, 0, 27, 55, 187, 199, 127, 147, 0, 225, 222, 58, 125, 233, 0, 9, 167, 174, 93, 84, 0, 101, 157, 209, 207, 0, 126, 190, 42, 0, 122, 76, 17, 160, 135, 44, 158, 0, 141, 64, 226, 175, 46, 188, 0, 22, 37, 239, 200, 198, 0, 216, 171, 57, 0, 111, 163, 1, 202, 4, 0, 45, 72, 164, 219, 221, 0, 177, 94, 169, 234, 26, 0, 36, 79, 14, 203, 153, 0, 54, 85, 236, 15, 129, 0, 146, 134, 150, 12, 139, 68, 6, 0, 96, 196, 121, 0, 103, 86, 228, 82, 168, 212, 0, 215, 220, 213, 117, 98, 28, 0, 148, 104, 144, 162, 182, 191, 0, 61, 51, 194, 204, 115, 0, 137, 13, 107, 227, 0, 120, 41, 193, 109, 178, 40, 0, 24, 29, 99, 74, 154, 0, 78, 145, 66, 102, 48, 0, 116, 53, 23, 2, 229, 106, 0 };
			costOptimalSolution = 82751;
		}
		else if (basename.compare("X-n247-k50.vrp") == 0)
		{
			optimalSolution = { 0, 53, 131, 161, 0, 45, 34, 156, 0, 178, 149, 80, 185, 51, 106, 121, 84, 212, 105, 0, 41, 87, 13, 151, 0, 158, 1, 181, 237, 134, 112, 0, 164, 70, 175, 227, 173, 63, 200, 98, 0, 117, 165, 128, 33, 74, 120, 238, 225, 0, 52, 0, 206, 83, 195, 114, 109, 245, 194, 93, 8, 0, 239, 220, 144, 229, 230, 5, 210, 188, 137, 179, 182, 0, 17, 23, 0, 36, 18, 123, 0, 21, 85, 0, 86, 126, 73, 39, 246, 226, 90, 197, 0, 167, 71, 190, 62, 233, 104, 172, 76, 183, 0, 198, 214, 193, 196, 77, 234, 240, 127, 28, 89, 79, 0, 160, 6, 59, 0, 88, 201, 78, 69, 184, 38, 203, 146, 107, 221, 145, 0, 141, 154, 19, 152, 118, 0, 16, 0, 48, 24, 0, 170, 244, 91, 102, 217, 199, 35, 140, 0, 142, 163, 110, 9, 139, 75, 125, 129, 81, 216, 0, 44, 103, 186, 187, 0, 37, 2, 0, 20, 42, 143, 0, 92, 11, 67, 0, 169, 56, 60, 0, 236, 46, 0, 3, 243, 115, 0, 111, 122, 130, 157, 47, 150, 219, 0, 162, 116, 202, 50, 228, 153, 0, 14, 95, 0, 12, 0, 68, 27, 132, 0, 97, 215, 135, 25, 100, 0, 61, 0, 66, 0, 43, 26, 94, 0, 159, 29, 7, 0, 10, 0, 207, 218, 171, 189, 32, 211, 0, 65, 58, 0, 15, 31, 213, 223, 0, 55, 30, 0, 119, 4, 0, 57, 0, 205, 99, 136, 177, 54, 191, 232, 113, 174, 0, 166, 208, 82, 108, 22, 180, 176, 222, 204, 192, 235, 133, 96, 0, 209, 40, 224, 155, 241, 124, 168, 101, 138, 0, 231, 49, 64, 72, 147, 242, 148, 0 };
			costOptimalSolution = 37274;
		}
		else if (basename.compare("X-n251-k28.vrp") == 0)
		{
			optimalSolution = { 0, 79, 37, 97, 107, 90, 0, 20, 229, 4, 81, 11, 53, 111, 121, 112, 0, 35, 69, 14, 23, 3, 21, 179, 110, 113, 96, 0, 39, 244, 205, 130, 136, 9, 87, 73, 0, 190, 234, 246, 140, 159, 227, 147, 64, 86, 0, 148, 128, 131, 218, 202, 223, 219, 176, 200, 0, 31, 71, 137, 66, 215, 108, 158, 206, 116, 0, 175, 172, 210, 145, 165, 171, 8, 211, 182, 135, 0, 198, 138, 222, 28, 181, 141, 248, 183, 0, 214, 133, 191, 220, 62, 67, 189, 177, 195, 17, 0, 160, 173, 209, 247, 154, 76, 166, 150, 0, 40, 157, 184, 228, 152, 207, 187, 42, 102, 0, 82, 240, 237, 127, 192, 230, 161, 203, 235, 0, 239, 143, 146, 153, 117, 94, 217, 156, 201, 167, 0, 80, 119, 101, 103, 25, 55, 118, 15, 0, 221, 204, 114, 164, 34, 51, 123, 124, 188, 84, 0, 144, 245, 238, 45, 236, 168, 22, 178, 243, 0, 149, 72, 13, 32, 52, 59, 26, 169, 75, 0, 125, 74, 115, 232, 105, 109, 99, 56, 50, 0, 33, 18, 89, 43, 122, 93, 29, 12, 91, 54, 0, 61, 48, 104, 100, 49, 68, 70, 57, 27, 41, 0, 47, 65, 92, 250, 60, 120, 129, 24, 197, 10, 0, 6, 139, 226, 16, 199, 186, 1, 83, 5, 0, 134, 88, 30, 44, 63, 180, 216, 193, 194, 77, 0, 36, 163, 151, 249, 242, 46, 225, 241, 213, 212, 0, 155, 162, 126, 231, 170, 224, 208, 196, 0, 38, 132, 174, 233, 185, 19, 142, 2, 98, 0, 78, 85, 7, 58, 106, 95, 0 };
			costOptimalSolution = 38684;
		}
		else if (basename.compare("X-n256-k16.vrp") == 0)
		{
			optimalSolution = { 0, 254, 220, 160, 27, 212, 44, 203, 200, 87, 25, 86, 47, 122, 167, 106, 0, 192, 151, 233, 189, 43, 164, 128, 48, 181, 4, 198, 166, 225, 82, 33, 0, 249, 50, 77, 179, 88, 213, 60, 215, 221, 171, 51, 95, 185, 214, 246, 0, 197, 15, 67, 158, 145, 104, 17, 30, 127, 109, 177, 29, 71, 124, 64, 193, 0, 201, 217, 153, 3, 16, 107, 226, 28, 57, 46, 42, 182, 62, 45, 241, 118, 0, 140, 250, 154, 98, 58, 242, 245, 223, 94, 224, 31, 211, 188, 248, 159, 168, 0, 130, 115, 169, 228, 206, 231, 196, 6, 120, 253, 73, 239, 97, 55, 142, 133, 0, 8, 72, 102, 163, 113, 81, 208, 195, 187, 68, 70, 93, 65, 119, 155, 24, 84, 0, 61, 40, 83, 90, 244, 165, 49, 116, 194, 22, 219, 56, 175, 234, 173, 0, 240, 184, 150, 52, 161, 138, 103, 222, 1, 136, 76, 144, 129, 237, 19, 23, 0, 251, 10, 69, 209, 126, 79, 180, 162, 96, 178, 157, 110, 75, 121, 236, 204, 11, 0, 232, 35, 21, 38, 190, 125, 80, 18, 108, 89, 7, 99, 34, 205, 227, 13, 0, 218, 141, 174, 210, 149, 112, 32, 85, 111, 238, 41, 37, 114, 20, 147, 146, 0, 12, 105, 39, 216, 176, 230, 14, 202, 123, 5, 229, 63, 36, 59, 26, 132, 148, 0, 54, 74, 91, 92, 172, 143, 199, 252, 78, 152, 170, 137, 235, 207, 243, 53, 0, 247, 134, 9, 135, 117, 2, 191, 66, 255, 156, 183, 131, 100, 186, 139, 101, 0 };
			costOptimalSolution = 18839;
		}
		else if (basename.compare("X-n261-k13.vrp") == 0)
		{
			optimalSolution = { 0, 181, 96, 121, 136, 184, 24, 157, 200, 88, 30, 126, 198, 13, 58, 128, 122, 101, 230, 255, 0, 257, 155, 222, 252, 38, 234, 170, 125, 239, 164, 204, 235, 224, 23, 146, 165, 113, 129, 81, 133, 21, 147, 74, 211, 0, 91, 232, 49, 7, 142, 168, 241, 160, 109, 124, 69, 120, 162, 32, 159, 242, 0, 1, 106, 2, 132, 220, 99, 40, 145, 9, 98, 259, 44, 16, 231, 62, 15, 229, 0, 246, 179, 195, 55, 186, 5, 73, 115, 33, 182, 208, 52, 108, 175, 94, 247, 161, 76, 72, 212, 166, 0, 93, 59, 97, 199, 47, 249, 185, 103, 34, 31, 3, 258, 158, 240, 187, 26, 217, 226, 0, 143, 178, 39, 188, 82, 180, 37, 214, 51, 64, 116, 171, 102, 42, 254, 251, 260, 53, 197, 174, 0, 79, 225, 6, 17, 238, 63, 245, 95, 192, 237, 60, 77, 154, 78, 27, 25, 194, 163, 12, 105, 151, 36, 90, 66, 68, 250, 0, 141, 41, 100, 48, 207, 19, 80, 236, 117, 111, 57, 119, 43, 112, 4, 50, 28, 209, 104, 0, 131, 221, 18, 138, 190, 244, 205, 167, 70, 243, 248, 169, 183, 45, 150, 173, 86, 110, 213, 67, 148, 0, 85, 210, 130, 172, 89, 65, 140, 227, 256, 56, 193, 223, 127, 83, 61, 201, 203, 20, 123, 8, 14, 152, 153, 35, 22, 10, 149, 215, 0, 177, 135, 134, 253, 46, 137, 107, 118, 156, 75, 0, 139, 176, 218, 84, 71, 206, 228, 87, 29, 189, 144, 233, 219, 11, 92, 191, 216, 114, 54, 196, 202, 0 };
			costOptimalSolution = 26558;
		}
		else if (basename.compare("X-n266-k58.vrp") == 0)
		{
			optimalSolution = { 0, 218, 248, 67, 229, 178, 0, 153, 180, 123, 172, 0, 115, 124, 110, 104, 0, 156, 214, 140, 226, 0, 163, 240, 133, 57, 152, 0, 255, 127, 203, 196, 0, 205, 256, 176, 192, 113, 0, 76, 243, 31, 53, 0, 170, 254, 141, 47, 0, 262, 166, 20, 10, 0, 142, 260, 222, 145, 135, 0, 30, 64, 6, 3, 0, 235, 173, 161, 69, 125, 0, 164, 234, 114, 13, 219, 0, 85, 107, 8, 9, 37, 0, 147, 238, 66, 98, 112, 0, 81, 233, 109, 29, 263, 0, 130, 38, 7, 5, 0, 185, 148, 137, 144, 198, 0, 32, 28, 11, 71, 1, 0, 26, 119, 61, 93, 2, 0, 227, 25, 149, 68, 0, 228, 213, 216, 169, 136, 0, 183, 204, 174, 181, 0, 94, 16, 45, 41, 102, 0, 74, 244, 210, 220, 160, 0, 241, 159, 182, 162, 0, 21, 258, 231, 73, 0, 99, 200, 207, 179, 15, 0, 117, 80, 131, 48, 247, 0, 242, 190, 12, 54, 146, 0, 187, 157, 158, 33, 211, 0, 155, 221, 129, 75, 92, 0, 42, 151, 132, 212, 0, 105, 184, 34, 175, 0, 171, 193, 62, 143, 186, 0, 101, 56, 35, 39, 208, 0, 40, 70, 82, 49, 106, 0, 249, 51, 89, 189, 0, 120, 100, 17, 122, 0, 195, 202, 165, 0, 97, 265, 259, 150, 43, 0, 14, 116, 50, 55, 60, 0, 225, 87, 22, 23, 0, 177, 86, 95, 52, 251, 0, 126, 121, 96, 77, 0, 206, 197, 237, 88, 103, 0, 224, 91, 58, 18, 0, 257, 78, 83, 72, 0, 118, 111, 36, 46, 4, 0, 108, 167, 19, 24, 128, 0, 253, 245, 63, 188, 0, 90, 65, 44, 59, 194, 0, 84, 230, 217, 134, 239, 0, 191, 209, 154, 246, 0, 168, 264, 199, 252, 79, 0, 236, 261, 139, 27, 223, 0, 232, 250, 138, 201, 215, 0 };
			costOptimalSolution = 75478;
		}
		else if (basename.compare("X-n270-k35.vrp") == 0)
		{
			optimalSolution = { 0, 146, 242, 139, 236, 249, 232, 155, 0, 121, 93, 46, 42, 17, 103, 8, 112, 0, 212, 158, 246, 11, 185, 138, 201, 189, 0, 150, 22, 255, 58, 134, 74, 182, 14, 0, 213, 127, 122, 97, 6, 245, 63, 0, 235, 154, 257, 164, 243, 13, 240, 173, 0, 191, 266, 192, 68, 193, 260, 128, 87, 0, 40, 142, 227, 177, 153, 195, 259, 184, 0, 41, 109, 19, 59, 92, 25, 20, 78, 0, 1, 194, 147, 231, 218, 234, 104, 89, 0, 3, 56, 47, 5, 37, 176, 86, 132, 490, 38, 10, 12, 72, 2, 123, 18, 0, 116, 107, 66, 45, 111, 125, 50, 0, 224, 124, 44, 205, 98, 157, 181, 156, 0, 32, 64, 80, 100, 16, 106, 35, 0, 62, 83, 71, 200, 0, 36, 57, 203, 190, 172, 130, 76, 102, 0, 250, 115, 252, 183, 151, 220, 26, 0, 60, 105, 126, 70, 54, 34, 77, 0, 131, 197, 75, 204, 216, 202, 61, 0, 94, 81, 108, 101, 168, 225, 9, 0, 30, 52, 145, 55, 95, 90, 269, 248, 0, 118, 247, 258, 43, 230, 15, 117, 0, 188, 253, 152, 254, 21, 161, 159, 0, 29, 33, 31, 79, 96, 53, 84, 73, 0, 48, 262, 140, 144, 199, 268, 149, 0, 206, 114, 141, 136, 219, 241, 175, 0, 198, 264, 222, 135, 171, 209, 267, 167, 0, 120, 27, 4, 7, 91, 65, 251, 51, 0, 263, 196, 187, 166, 207, 238, 237, 85, 0, 215, 165, 163, 67, 99, 228, 113, 69, 0, 119, 265, 169, 24, 129, 23, 88, 39, 0, 137, 178, 148, 179, 143, 133, 256, 162, 0, 223, 210, 160, 174, 239, 233, 261, 0, 208, 180, 221, 217, 28, 214, 244, 0, 211, 170, 229, 226, 186, 110, 82, 0 };
			costOptimalSolution = 35291;
		}
		else if (basename.compare("X-n275-k28.vrp") == 0)
		{
			optimalSolution = { 0, 152, 88, 252, 100, 239, 13, 241, 77, 157, 193, 0, 163, 124, 138, 235, 0, 206, 230, 158, 118, 166, 229, 122, 150, 170, 90, 0, 112, 171, 31, 165, 41, 224, 232, 67, 87, 215, 0, 45, 200, 151, 35, 4, 7, 94, 69, 110, 274, 0, 240, 49, 43, 8, 1, 19, 199, 167, 214, 176, 0, 102, 5, 101, 130, 46, 63, 140, 245, 209, 249, 0, 125, 242, 210, 164, 132, 267, 218, 261, 251, 11, 0, 123, 265, 173, 111, 127, 64, 201, 99, 195, 192, 0, 32, 56, 29, 15, 212, 160, 62, 82, 117, 190, 0, 196, 71, 113, 216, 187, 76, 80, 14, 68, 258, 0, 37, 95, 217, 247, 228, 233, 134, 148, 139, 61, 0, 161, 97, 75, 22, 23, 85, 144, 106, 186, 183, 0, 141, 268, 260, 244, 142, 182, 25, 2, 246, 179, 0, 36, 155, 189, 34, 211, 256, 55, 248, 234, 126, 0, 26, 156, 53, 57, 96, 103, 114, 58, 250, 238, 0, 16, 147, 159, 177, 133, 221, 208, 33, 266, 254, 0, 188, 204, 116, 24, 175, 253, 60, 74, 225, 86, 0, 115, 172, 9, 273, 168, 27, 107, 146, 226, 73, 0, 181, 119, 42, 198, 269, 65, 145, 202, 89, 272, 0, 104, 194, 257, 222, 236, 180, 120, 91, 109, 184, 0, 136, 185, 262, 207, 129, 47, 39, 38, 213, 135, 0, 219, 108, 78, 66, 270, 81, 174, 223, 149, 83, 0, 243, 169, 128, 220, 44, 84, 20, 21, 79, 92, 0, 6, 28, 59, 12, 231, 271, 54, 191, 197, 162, 0, 40, 227, 10, 3, 255, 18, 121, 178, 50, 51, 0, 98, 263, 264, 153, 154, 237, 70, 259, 72, 131, 0, 30, 17, 203, 105, 205, 48, 137, 143, 52, 93, 0 };
			costOptimalSolution = 21245;
		}
		else if (basename.compare("X-n280-k17.vrp") == 0)
		{
			optimalSolution = { 0, 90, 25, 76, 102, 78, 115, 143, 244, 3, 181, 195, 86, 113, 252, 29, 254, 258, 0, 155, 151, 218, 206, 8, 149, 84, 12, 213, 215, 0, 232, 17, 226, 4, 176, 80, 0, 69, 20, 10, 243, 228, 83, 128, 0, 230, 53, 196, 92, 187, 168, 227, 169, 221, 219, 64, 235, 119, 224, 72, 50, 33, 279, 59, 58, 40, 73, 278, 5, 65, 201, 245, 133, 0, 203, 164, 270, 71, 49, 225, 106, 2, 104, 118, 234, 257, 45, 217, 100, 174, 52, 0, 273, 138, 146, 95, 246, 47, 82, 190, 24, 88, 142, 140, 191, 171, 7, 237, 177, 179, 48, 145, 79, 170, 0, 180, 96, 266, 111, 239, 57, 28, 188, 13, 87, 256, 277, 116, 55, 134, 30, 135, 250, 0, 216, 126, 205, 70, 233, 253, 208, 62, 184, 1, 251, 139, 222, 238, 67, 114, 121, 38, 241, 199, 0, 63, 248, 22, 9, 77, 166, 43, 132, 0, 263, 136, 54, 153, 249, 262, 148, 108, 223, 198, 193, 175, 197, 14, 214, 265, 202, 200, 0, 42, 93, 23, 271, 269, 129, 18, 150, 272, 27, 274, 37, 141, 261, 34, 186, 99, 125, 0, 165, 98, 152, 137, 35, 75, 127, 120, 183, 36, 167, 6, 247, 39, 117, 41, 56, 44, 74, 267, 0, 159, 60, 210, 242, 94, 160, 182, 66, 268, 157, 260, 101, 275, 131, 162, 109, 255, 112, 122, 32, 21, 89, 276, 51, 156, 0, 194, 19, 144, 15, 158, 31, 147, 0, 97, 189, 61, 163, 264, 68, 178, 236, 105, 220, 161, 11, 173, 209, 103, 240, 0, 154, 26, 212, 229, 123, 81, 46, 211, 91, 110, 172, 259, 85, 107, 192, 207, 124, 204, 16, 185, 130, 231, 0 };
			costOptimalSolution = 33503;
		}
		else if (basename.compare("X-n284-k15.vrp") == 0)
		{
			optimalSolution = { 0, 22, 0, 87, 97, 21, 35, 260, 30, 55, 17, 219, 278, 110, 161, 76, 129, 193, 19, 58, 197, 247, 0, 171, 91, 223, 75, 222, 179, 46, 183, 57, 4, 257, 137, 159, 121, 60, 158, 244, 228, 7, 0, 177, 27, 119, 14, 190, 175, 167, 238, 216, 64, 132, 265, 81, 56, 201, 249, 134, 112, 208, 59, 195, 237, 255, 73, 23, 218, 80, 0, 120, 147, 48, 42, 182, 180, 139, 163, 12, 8, 74, 200, 84, 89, 215, 185, 67, 45, 196, 115, 226, 272, 153, 0, 53, 256, 95, 154, 279, 133, 69, 214, 39, 51, 113, 32, 283, 6, 250, 114, 210, 0, 146, 105, 248, 108, 205, 72, 52, 233, 10, 5, 31, 99, 118, 213, 173, 149, 271, 18, 0, 85, 106, 50, 281, 135, 270, 38, 166, 181, 192, 267, 162, 82, 25, 275, 211, 43, 277, 104, 0, 160, 246, 126, 124, 156, 232, 187, 107, 176, 117, 103, 268, 11, 44, 71, 34, 127, 231, 202, 15, 143, 111, 131, 102, 224, 0, 236, 40, 141, 194, 88, 140, 130, 86, 26, 16, 2, 184, 264, 94, 151, 254, 198, 148, 36, 77, 142, 240, 0, 122, 78, 230, 206, 47, 203, 262, 204, 96, 170, 273, 225, 152, 199, 188, 155, 37, 0, 168, 252, 24, 191, 242, 9, 92, 261, 164, 269, 259, 280, 207, 83, 189, 54, 251, 274, 41, 0, 276, 172, 1, 186, 125, 100, 258, 145, 65, 212, 234, 136, 28, 29, 116, 0, 221, 157, 243, 227, 235, 61, 20, 79, 220, 3, 138, 109, 90, 245, 253, 150, 209, 217, 239, 68, 13, 266, 63, 0, 241, 49, 98, 174, 62, 70, 144, 66, 178, 169, 165, 128, 101, 263, 123, 33, 282, 229, 93, 0 };
			costOptimalSolution = 20226;
		}
		else if (basename.compare("X-n289-k60.vrp") == 0)
		{
			optimalSolution = { 0, 101, 140, 68, 24, 60, 119, 127, 84, 116, 132, 0, 196, 215, 43, 266, 0, 224, 242, 159, 29, 83, 26, 223, 100, 285, 109, 0, 260, 188, 80, 33, 61, 77, 0, 251, 65, 197, 0, 230, 96, 70, 44, 0, 139, 27, 152, 92, 19, 0, 110, 90, 284, 131, 0, 189, 170, 206, 0, 22, 11, 106, 244, 0, 204, 94, 209, 0, 249, 218, 192, 0, 2, 25, 278, 0, 254, 145, 169, 0, 171, 281, 272, 199, 271, 0, 233, 282, 217, 0, 62, 168, 53, 250, 0, 227, 226, 134, 151, 208, 205, 287, 87, 0, 146, 191, 72, 211, 166, 0, 275, 78, 135, 0, 58, 82, 180, 0, 270, 195, 107, 0, 267, 213, 136, 0, 147, 263, 214, 248, 0, 113, 103, 124, 137, 0, 75, 174, 274, 3, 0, 256, 280, 219, 198, 0, 175, 201, 167, 185, 0, 15, 255, 245, 239, 0, 277, 252, 228, 0, 273, 187, 149, 0, 123, 45, 156, 203, 0, 212, 54, 234, 0, 20, 125, 126, 0, 38, 36, 247, 47, 0, 50, 164, 283, 162, 190, 220, 74, 165, 0, 98, 18, 13, 97, 0, 40, 241, 16, 91, 0, 286, 240, 32, 193, 0, 246, 114, 30, 128, 0, 79, 261, 172, 95, 0, 76, 183, 288, 0, 88, 41, 9, 64, 0, 63, 176, 186, 102, 262, 0, 178, 86, 216, 225, 0, 158, 184, 229, 0, 200, 148, 157, 0, 59, 6, 268, 12, 0, 67, 46, 35, 121, 52, 0, 133, 5, 34, 14, 112, 7, 21, 85, 130, 57, 0, 269, 155, 232, 0, 111, 122, 55, 115, 73, 108, 117, 8, 81, 0, 236, 17, 235, 99, 221, 222, 161, 89, 160, 0, 257, 173, 231, 104, 23, 141, 144, 4, 129, 31, 49, 10, 118, 202, 48, 71, 0, 39, 265, 238, 243, 279, 0, 105, 177, 150, 258, 207, 163, 194, 264, 276, 237, 0, 253, 210, 51, 0, 182, 120, 56, 0, 181, 1, 179, 142, 0, 154, 28, 153, 37, 0, 138, 69, 42, 93, 143, 66, 259, 0 };
			costOptimalSolution = 95151;
		}
		else if (basename.compare("X-n294-k50.vrp") == 0)
		{
			optimalSolution = { 0, 207, 219, 166, 121, 129, 41, 128, 140, 290, 0, 274, 118, 167, 21, 155, 143, 278, 0, 218, 211, 126, 199, 100, 0, 26, 120, 252, 142, 228, 0, 235, 280, 241, 56, 292, 192, 0, 187, 28, 35, 203, 213, 215, 0, 141, 19, 112, 65, 0, 24, 224, 61, 267, 0, 198, 123, 234, 95, 75, 257, 285, 0, 88, 178, 270, 259, 231, 0, 69, 188, 0, 266, 206, 151, 10, 124, 186, 46, 130, 0, 115, 76, 184, 233, 0, 144, 289, 33, 11, 86, 3, 0, 238, 246, 268, 99, 0, 58, 135, 237, 169, 92, 0, 180, 189, 54, 223, 91, 190, 0, 114, 256, 243, 279, 149, 0, 31, 84, 40, 291, 185, 104, 0, 116, 221, 239, 288, 0, 23, 48, 81, 72, 102, 0, 152, 60, 57, 70, 71, 205, 227, 0, 131, 30, 80, 4, 14, 1, 293, 160, 0, 45, 269, 117, 282, 242, 0, 183, 161, 249, 127, 87, 254, 132, 260, 0, 103, 67, 193, 97, 158, 96, 163, 0, 195, 44, 263, 20, 49, 52, 145, 5, 0, 134, 25, 113, 240, 262, 179, 0, 216, 181, 244, 147, 0, 230, 136, 250, 247, 283, 0, 210, 170, 111, 27, 36, 0, 90, 62, 197, 275, 37, 171, 251, 146, 0, 156, 276, 214, 50, 0, 12, 148, 204, 162, 64, 82, 0, 157, 101, 89, 32, 273, 0, 43, 83, 107, 59, 164, 225, 105, 22, 253, 0, 261, 201, 2, 194, 106, 271, 174, 172, 0, 154, 281, 168, 77, 159, 16, 0, 85, 109, 153, 38, 73, 175, 66, 0, 119, 8, 229, 29, 182, 15, 255, 0, 74, 177, 110, 245, 0, 222, 248, 94, 7, 277, 13, 284, 0, 98, 176, 55, 0, 265, 18, 191, 217, 232, 196, 0, 137, 258, 209, 264, 68, 17, 0, 108, 272, 138, 236, 139, 0, 42, 173, 125, 93, 63, 0, 51, 34, 150, 133, 287, 0, 208, 47, 286, 212, 53, 39, 0, 122, 165, 202, 200, 9, 0, 78, 220, 79, 6, 226, 0 };
			costOptimalSolution = 47161;
		}
		else if (basename.compare("X-n298-k31.vrp") == 0)
		{
			optimalSolution = { 0, 30, 236, 267, 245, 237, 168, 101, 290, 184, 0, 113, 10, 185, 32, 98, 288, 45, 134, 57, 91, 0, 47, 80, 226, 278, 255, 39, 71, 148, 29, 0, 84, 173, 171, 167, 76, 158, 65, 254, 174, 72, 37, 85, 2, 83, 0, 22, 58, 82, 1, 200, 42, 270, 105, 193, 108, 152, 106, 242, 0, 169, 26, 4, 272, 31, 153, 156, 100, 88, 280, 0, 81, 49, 209, 155, 260, 194, 35, 43, 110, 11, 275, 0, 224, 104, 89, 203, 161, 63, 252, 21, 163, 204, 0, 229, 282, 180, 215, 243, 146, 213, 147, 0, 145, 246, 19, 262, 51, 48, 133, 178, 249, 250, 0, 79, 293, 70, 69, 56, 59, 151, 0, 199, 8, 164, 201, 239, 112, 261, 135, 170, 0, 36, 24, 40, 121, 139, 289, 219, 77, 38, 52, 211, 285, 0, 208, 16, 124, 92, 172, 295, 271, 137, 274, 61, 0, 235, 41, 107, 46, 207, 62, 138, 0, 233, 122, 94, 136, 67, 263, 196, 0, 118, 221, 231, 216, 264, 286, 50, 33, 17, 190, 3, 0, 149, 132, 225, 54, 195, 97, 175, 73, 15, 7, 0, 6, 247, 109, 176, 244, 283, 12, 276, 257, 0, 55, 240, 18, 111, 64, 279, 297, 141, 0, 74, 256, 150, 9, 181, 123, 14, 228, 28, 93, 0, 75, 265, 183, 114, 296, 129, 143, 205, 284, 202, 0, 291, 140, 212, 197, 266, 214, 60, 34, 227, 0, 186, 223, 99, 23, 68, 187, 87, 230, 177, 269, 258, 234, 27, 191, 0, 273, 5, 206, 198, 217, 125, 210, 0, 13, 251, 119, 25, 142, 277, 20, 154, 0, 117, 218, 78, 127, 165, 120, 128, 166, 0, 248, 160, 182, 157, 96, 95, 86, 0, 241, 281, 294, 116, 130, 159, 292, 259, 131, 253, 126, 0, 162, 220, 115, 287, 179, 189, 44, 102, 144, 66, 192, 0, 103, 53, 232, 222, 90, 238, 188, 268, 0 };
			costOptimalSolution = 34231;
		}
		else if (basename.compare("X-n303-k21.vrp") == 0)
		{
			optimalSolution = { 0, 270, 143, 295, 219, 23, 249, 248, 76, 61, 153, 243, 27, 110, 238, 63, 118, 256, 98, 0, 5, 193, 30, 133, 113, 62, 25, 188, 69, 131, 224, 258, 293, 0, 115, 162, 46, 233, 103, 34, 296, 196, 274, 83, 172, 9, 167, 77, 134, 244, 149, 56, 0, 12, 99, 146, 176, 117, 223, 197, 47, 75, 93, 236, 241, 114, 21, 0, 67, 200, 246, 7, 58, 230, 299, 289, 242, 231, 125, 139, 129, 71, 199, 105, 215, 0, 10, 285, 20, 121, 180, 251, 39, 175, 14, 229, 277, 155, 144, 45, 267, 84, 22, 156, 54, 0, 130, 65, 132, 191, 182, 2, 96, 26, 281, 142, 290, 135, 201, 189, 245, 0, 160, 202, 262, 255, 194, 207, 138, 145, 254, 239, 240, 31, 217, 0, 226, 29, 300, 198, 259, 174, 36, 108, 163, 187, 68, 291, 166, 52, 205, 0, 302, 4, 234, 266, 171, 252, 268, 150, 253, 186, 90, 195, 280, 8, 123, 51, 60, 0, 53, 192, 164, 235, 95, 264, 97, 124, 112, 165, 278, 122, 273, 210, 0, 286, 148, 74, 100, 35, 15, 85, 137, 59, 66, 94, 101, 237, 211, 0, 301, 33, 209, 42, 92, 18, 126, 141, 159, 89, 218, 82, 275, 140, 190, 227, 257, 104, 0, 157, 228, 55, 106, 216, 221, 250, 40, 91, 151, 271, 50, 0, 107, 147, 204, 0, 44, 261, 152, 247, 181, 265, 78, 37, 28, 128, 32, 0, 154, 86, 3, 136, 269, 213, 81, 64, 282, 79, 168, 183, 185, 0, 222, 170, 214, 41, 260, 292, 17, 297, 48, 80, 1, 287, 43, 279, 178, 0, 208, 276, 57, 298, 19, 220, 173, 284, 6, 272, 184, 232, 0, 16, 206, 294, 72, 177, 263, 11, 73, 13, 87, 38, 88, 179, 288, 116, 119, 0, 24, 203, 70, 127, 102, 161, 111, 49, 169, 212, 225, 283, 120, 109, 158, 0 };
			costOptimalSolution = 21744;
		}
		else if (basename.compare("X-n308-k13.vrp") == 0)
		{
			optimalSolution = { 0, 57, 234, 37, 139, 51, 146, 153, 116, 63, 6, 81, 114, 118, 140, 25, 50, 142, 96, 1, 29, 56, 0, 210, 291, 112, 132, 77, 20, 249, 41, 120, 295, 10, 60, 79, 121, 138, 130, 108, 200, 206, 178, 83, 277, 182, 0, 136, 164, 103, 49, 135, 38, 101, 221, 126, 15, 70, 4, 84, 144, 97, 106, 137, 0, 212, 147, 143, 296, 269, 282, 222, 197, 196, 186, 117, 3, 276, 13, 270, 0, 246, 177, 230, 253, 257, 247, 213, 66, 180, 65, 100, 216, 173, 165, 260, 167, 9, 105, 261, 229, 251, 223, 194, 155, 263, 256, 0, 265, 30, 124, 69, 198, 11, 232, 240, 149, 94, 17, 228, 211, 204, 162, 86, 176, 250, 0, 288, 53, 154, 95, 23, 125, 71, 264, 189, 238, 207, 35, 18, 64, 286, 62, 39, 148, 267, 248, 293, 233, 171, 266, 199, 285, 156, 175, 85, 166, 281, 302, 0, 110, 226, 134, 161, 24, 89, 93, 42, 5, 128, 16, 59, 129, 80, 26, 68, 243, 28, 44, 272, 299, 48, 168, 46, 82, 303, 0, 87, 2, 113, 150, 292, 98, 205, 274, 280, 157, 183, 297, 163, 169, 104, 127, 218, 254, 72, 145, 40, 102, 52, 131, 31, 190, 0, 187, 245, 34, 170, 215, 73, 8, 58, 237, 307, 203, 109, 258, 160, 78, 279, 54, 115, 0, 209, 76, 152, 36, 55, 32, 123, 300, 179, 298, 74, 284, 255, 174, 275, 202, 99, 92, 283, 111, 61, 45, 214, 301, 7, 201, 88, 43, 271, 27, 119, 0, 195, 241, 22, 268, 21, 90, 141, 185, 259, 242, 19, 107, 188, 122, 224, 191, 306, 227, 47, 12, 239, 151, 33, 219, 192, 225, 172, 305, 252, 181, 236, 0, 67, 304, 220, 244, 91, 290, 184, 273, 14, 75, 278, 159, 289, 235, 231, 294, 262, 208, 158, 217, 193, 287, 133, 0 };
			costOptimalSolution = 25859;
		}
		else if (basename.compare("X-n313-k71.vrp") == 0)
		{
			optimalSolution = { 0, 31, 121, 70, 133, 153, 34, 99, 0, 48, 102, 44, 142, 36, 126, 152, 32, 0, 9, 137, 6, 4, 0, 3, 19, 60, 28, 82, 0, 2, 202, 30, 128, 89, 0, 79, 106, 139, 0, 115, 269, 207, 0, 51, 64, 57, 130, 84, 116, 90, 0, 69, 176, 277, 0, 150, 78, 122, 93, 0, 237, 117, 174, 145, 0, 25, 132, 230, 276, 0, 221, 270, 279, 0, 87, 155, 204, 302, 0, 95, 281, 147, 0, 208, 24, 257, 29, 0, 299, 280, 165, 112, 0, 131, 228, 164, 175, 0, 189, 250, 171, 162, 0, 217, 74, 252, 0, 127, 7, 56, 33, 0, 190, 260, 312, 234, 0, 226, 265, 278, 0, 289, 159, 214, 0, 18, 108, 61, 201, 0, 218, 224, 125, 287, 0, 262, 268, 298, 0, 72, 295, 249, 0, 141, 43, 211, 53, 0, 161, 54, 285, 0, 114, 266, 248, 0, 290, 239, 22, 0, 75, 303, 184, 0, 86, 243, 267, 107, 301, 0, 258, 311, 236, 0, 10, 215, 45, 163, 0, 272, 216, 17, 245, 0, 251, 253, 177, 282, 0, 5, 222, 229, 219, 111, 193, 309, 310, 206, 0, 274, 264, 256, 185, 297, 166, 205, 271, 67, 0, 110, 88, 124, 0, 158, 179, 273, 247, 38, 238, 173, 0, 195, 21, 196, 244, 149, 180, 191, 0, 42, 291, 231, 140, 198, 284, 11, 259, 293, 0, 296, 52, 136, 143, 118, 23, 0, 235, 26, 123, 66, 160, 35, 20, 148, 0, 13, 104, 101, 80, 46, 120, 151, 58, 97, 168, 300, 94, 135, 98, 0, 182, 62, 68, 59, 0, 83, 91, 63, 37, 144, 40, 146, 77, 65, 0, 288, 129, 15, 0, 1, 134, 12, 0, 292, 286, 178, 0, 27, 50, 85, 49, 105, 76, 0, 170, 188, 261, 0, 71, 109, 103, 0, 187, 138, 283, 0, 209, 192, 305, 294, 0, 92, 227, 197, 0, 306, 213, 255, 0, 81, 183, 308, 0, 246, 203, 39, 0, 307, 263, 233, 223, 0, 157, 199, 254, 41, 0, 220, 210, 73, 0, 212, 194, 242, 0, 304, 225, 96, 0, 154, 14, 0, 169, 181, 47, 113, 0, 119, 240, 156, 0, 16, 186, 200, 0, 167, 232, 275, 100, 0, 172, 8, 241, 55, 0 };
			costOptimalSolution = 94044;
		}
		else if (basename.compare("X-n317-k53.vrp") == 0)
		{
			optimalSolution = { 0, 41, 3, 276, 24, 118, 303, 0, 199, 30, 237, 215, 60, 240, 0, 141, 315, 307, 45, 206, 165, 0, 32, 183, 278, 226, 296, 180, 0, 195, 214, 58, 137, 177, 270, 0, 9, 306, 23, 144, 282, 75, 0, 47, 147, 247, 82, 213, 171, 0, 64, 184, 69, 35, 114, 106, 0, 209, 142, 104, 227, 88, 188, 0, 146, 136, 186, 291, 72, 148, 0, 232, 68, 261, 222, 111, 248, 0, 175, 308, 119, 279, 128, 103, 0, 275, 286, 4, 132, 281, 187, 0, 238, 153, 224, 89, 205, 117, 0, 299, 70, 218, 62, 160, 285, 0, 57, 297, 158, 249, 242, 231, 0, 38, 292, 301, 11, 244, 145, 0, 17, 8, 229, 83, 161, 211, 0, 120, 228, 99, 50, 152, 55, 0, 300, 74, 110, 135, 241, 18, 0, 90, 84, 280, 95, 271, 20, 0, 44, 298, 112, 134, 81, 133, 0, 48, 266, 59, 56, 239, 234, 0, 96, 208, 225, 65, 251, 284, 0, 63, 253, 200, 268, 259, 194, 0, 105, 61, 262, 164, 102, 93, 0, 185, 216, 101, 166, 230, 250, 0, 40, 257, 260, 316, 130, 125, 0, 73, 39, 245, 258, 305, 252, 0, 98, 173, 219, 131, 1, 150, 0, 129, 274, 6, 255, 288, 277, 0, 170, 25, 283, 254, 22, 269, 0, 19, 7, 43, 233, 217, 313, 0, 52, 67, 293, 27, 37, 204, 0, 201, 53, 109, 5, 221, 290, 0, 246, 243, 49, 178, 193, 223, 0, 16, 126, 34, 295, 122, 54, 0, 267, 116, 121, 100, 174, 115, 0, 272, 33, 156, 210, 212, 10, 0, 287, 124, 263, 46, 294, 36, 0, 236, 79, 179, 176, 29, 159, 0, 140, 163, 91, 28, 172, 191, 0, 197, 80, 309, 256, 51, 76, 0, 143, 302, 182, 149, 123, 235, 0, 21, 162, 311, 71, 198, 14, 0, 108, 289, 92, 151, 264, 94, 0, 127, 2, 265, 181, 312, 202, 0, 310, 189, 77, 113, 139, 203, 0, 97, 66, 86, 304, 190, 107, 0, 13, 207, 138, 196, 26, 167, 0, 154, 220, 12, 157, 273, 192, 0, 78, 85, 42, 155, 15, 314, 0, 169, 168, 31, 87, 0 };
			costOptimalSolution = 78355;
		}
		else if (basename.compare("X-n322-k28.vrp") == 0)
		{
			optimalSolution = { 0, 110, 108, 145, 12, 192, 171, 112, 302, 129, 104, 103, 0, 28, 36, 24, 212, 206, 9, 167, 84, 150, 274, 21, 277, 155, 0, 244, 266, 100, 176, 154, 283, 278, 13, 169, 86, 40, 123, 72, 0, 15, 156, 96, 320, 267, 34, 189, 121, 88, 191, 258, 17, 0, 194, 148, 144, 63, 3, 260, 226, 219, 4, 187, 82, 0, 90, 216, 137, 172, 273, 161, 77, 295, 5, 288, 259, 0, 27, 208, 115, 65, 30, 10, 248, 48, 290, 303, 306, 269, 0, 54, 316, 99, 207, 213, 214, 35, 43, 236, 182, 33, 0, 184, 230, 106, 177, 231, 193, 68, 113, 196, 281, 14, 81, 0, 57, 294, 319, 146, 164, 111, 271, 71, 307, 262, 58, 0, 64, 127, 203, 318, 234, 89, 8, 139, 135, 79, 252, 0, 128, 49, 199, 31, 185, 101, 142, 242, 317, 138, 173, 51, 126, 0, 275, 125, 270, 143, 218, 149, 97, 160, 50, 170, 18, 109, 0, 166, 190, 183, 26, 263, 254, 158, 250, 298, 59, 1, 0, 124, 55, 280, 75, 20, 232, 133, 311, 53, 276, 202, 312, 0, 305, 130, 287, 141, 46, 272, 23, 257, 117, 38, 61, 0, 76, 229, 107, 37, 62, 165, 7, 132, 224, 296, 205, 204, 0, 292, 69, 73, 180, 309, 131, 94, 293, 179, 152, 116, 0, 6, 220, 235, 98, 200, 52, 91, 289, 221, 78, 188, 0, 237, 29, 159, 60, 297, 92, 291, 264, 74, 134, 83, 0, 163, 256, 32, 239, 286, 215, 315, 136, 157, 181, 175, 0, 45, 304, 16, 223, 87, 118, 42, 209, 162, 249, 70, 0, 282, 247, 301, 210, 41, 93, 168, 217, 261, 153, 80, 95, 0, 245, 66, 174, 228, 313, 201, 284, 47, 39, 186, 0, 2, 310, 240, 67, 308, 198, 238, 119, 321, 11, 222, 0, 22, 285, 211, 122, 25, 255, 253, 147, 265, 233, 19, 0, 102, 140, 225, 44, 246, 300, 227, 105, 151, 268, 197, 279, 0, 85, 251, 299, 314, 114, 243, 241, 195, 56, 178, 120, 0 };
			costOptimalSolution = 29834;
		}
		else if (basename.compare("X-n327-k20.vrp") == 0)
		{
			optimalSolution = { 0, 159, 49, 267, 284, 269, 199, 177, 136, 160, 80, 13, 112, 67, 235, 127, 298, 255, 0, 282, 148, 96, 222, 243, 230, 105, 56, 287, 292, 185, 0, 114, 64, 29, 9, 73, 133, 115, 90, 77, 38, 5, 97, 51, 82, 151, 93, 130, 32, 0, 53, 26, 98, 42, 34, 241, 57, 181, 163, 134, 145, 78, 63, 278, 294, 0, 6, 275, 37, 193, 276, 22, 140, 45, 132, 74, 19, 129, 60, 204, 131, 320, 0, 308, 290, 146, 280, 24, 43, 242, 104, 150, 84, 113, 52, 25, 142, 123, 75, 54, 68, 72, 0, 238, 110, 70, 178, 322, 326, 299, 285, 36, 61, 258, 164, 306, 207, 286, 7, 88, 190, 0, 191, 106, 187, 231, 116, 118, 233, 10, 277, 92, 40, 254, 66, 291, 173, 225, 138, 0, 15, 135, 156, 31, 162, 46, 65, 109, 107, 144, 11, 119, 21, 101, 139, 121, 309, 0, 126, 33, 313, 216, 167, 214, 273, 265, 245, 220, 279, 198, 217, 251, 120, 197, 0, 247, 196, 201, 1, 316, 94, 18, 314, 69, 161, 152, 79, 175, 215, 117, 41, 288, 325, 0, 271, 183, 16, 155, 14, 47, 91, 59, 158, 28, 44, 323, 264, 86, 203, 95, 249, 87, 0, 23, 188, 295, 192, 179, 307, 12, 234, 240, 85, 229, 315, 168, 209, 206, 256, 8, 0, 149, 300, 99, 35, 202, 27, 283, 305, 257, 71, 122, 260, 182, 252, 261, 321, 319, 0, 250, 157, 213, 270, 304, 262, 186, 137, 141, 302, 246, 248, 30, 48, 289, 125, 228, 0, 223, 266, 102, 218, 89, 81, 3, 4, 76, 189, 62, 143, 147, 2, 310, 39, 226, 293, 0, 50, 100, 166, 274, 83, 324, 281, 153, 210, 221, 296, 154, 20, 311, 272, 0, 239, 55, 244, 170, 172, 312, 180, 317, 103, 301, 128, 174, 111, 211, 205, 303, 224, 0, 58, 253, 263, 236, 165, 237, 195, 268, 219, 0, 227, 232, 176, 169, 17, 108, 194, 208, 200, 259, 212, 184, 171, 318, 124, 297, 0 };
			costOptimalSolution = 27532;
		}
		else if (basename.compare("X-n331-k15.vrp") == 0)
		{
			optimalSolution = { 0, 300, 274, 129, 306, 66, 183, 157, 101, 156, 270, 310, 193, 219, 314, 131, 54, 214, 115, 222, 248, 44, 189, 83, 0, 159, 182, 309, 88, 21, 132, 269, 84, 162, 234, 226, 253, 31, 315, 114, 67, 4, 77, 80, 18, 118, 48, 288, 0, 8, 173, 28, 102, 32, 280, 90, 134, 284, 267, 249, 307, 46, 202, 278, 53, 263, 172, 262, 266, 72, 40, 117, 0, 210, 39, 96, 51, 55, 186, 170, 327, 320, 272, 236, 98, 163, 276, 122, 153, 79, 213, 250, 303, 328, 257, 228, 0, 252, 273, 179, 225, 70, 58, 74, 171, 148, 68, 296, 251, 175, 259, 103, 200, 110, 316, 125, 100, 319, 82, 247, 0, 305, 160, 137, 174, 287, 127, 47, 107, 282, 5, 277, 185, 167, 194, 324, 138, 128, 240, 217, 92, 7, 33, 297, 0, 99, 9, 45, 322, 317, 301, 237, 235, 1, 19, 261, 13, 14, 56, 231, 116, 180, 238, 188, 215, 199, 178, 26, 0, 42, 95, 10, 16, 142, 97, 147, 151, 65, 94, 244, 63, 86, 59, 89, 168, 144, 169, 120, 204, 146, 111, 152, 0, 30, 158, 329, 143, 203, 150, 61, 239, 136, 161, 311, 73, 264, 139, 312, 218, 254, 227, 155, 38, 184, 60, 291, 0, 76, 190, 37, 20, 78, 27, 293, 258, 209, 36, 325, 192, 3, 35, 330, 289, 112, 318, 286, 260, 181, 123, 243, 0, 126, 41, 71, 224, 206, 12, 242, 255, 0, 34, 279, 196, 2, 245, 52, 265, 211, 292, 233, 313, 119, 91, 201, 24, 109, 145, 106, 15, 166, 229, 164, 177, 0, 104, 141, 205, 256, 154, 6, 271, 135, 85, 57, 304, 230, 113, 25, 11, 165, 64, 140, 207, 187, 298, 23, 295, 0, 268, 216, 191, 208, 275, 43, 87, 75, 221, 283, 323, 198, 212, 281, 195, 220, 176, 50, 62, 17, 130, 223, 197, 0, 69, 285, 105, 299, 246, 49, 294, 290, 133, 81, 302, 321, 232, 93, 29, 241, 124, 22, 108, 326, 121, 308, 149, 0 };
			costOptimalSolution = 31102;
		}
		else if (basename.compare("X-n336-k84.vrp") == 0)
		{
			optimalSolution = { 0, 190, 85, 215, 44, 115, 216, 46, 306, 0, 322, 101, 262, 333, 37, 30, 141, 292, 50, 173, 0, 313, 77, 8, 211, 3, 276, 123, 198, 66, 83, 0, 269, 316, 225, 34, 93, 297, 221, 282, 229, 33, 192, 0, 21, 118, 139, 331, 304, 73, 47, 266, 36, 0, 32, 213, 54, 247, 186, 195, 65, 172, 294, 319, 162, 0, 227, 208, 31, 302, 169, 228, 233, 200, 25, 0, 69, 152, 209, 58, 180, 29, 0, 261, 120, 193, 7, 38, 320, 204, 290, 206, 311, 0, 257, 298, 178, 163, 238, 151, 273, 293, 0, 43, 285, 335, 132, 264, 0, 75, 277, 232, 274, 194, 0, 63, 149, 1, 205, 138, 0, 90, 127, 197, 11, 26, 268, 165, 0, 177, 201, 59, 246, 135, 271, 0, 314, 55, 330, 0, 23, 272, 16, 0, 5, 241, 114, 332, 0, 122, 102, 0, 214, 275, 61, 267, 88, 0, 119, 12, 113, 74, 95, 0, 160, 84, 235, 0, 196, 309, 86, 40, 0, 42, 243, 295, 97, 0, 230, 250, 161, 0, 159, 81, 70, 0, 171, 53, 175, 0, 312, 259, 300, 0, 60, 155, 79, 0, 329, 182, 134, 0, 185, 22, 125, 0, 258, 255, 244, 24, 0, 291, 28, 328, 0, 80, 19, 67, 0, 212, 236, 9, 0, 147, 48, 27, 0, 305, 183, 0, 308, 158, 278, 287, 0, 154, 153, 174, 252, 0, 181, 251, 323, 0, 315, 130, 0, 318, 256, 126, 0, 100, 156, 136, 189, 0, 168, 116, 0, 4, 245, 89, 0, 129, 202, 51, 105, 6, 0, 191, 307, 62, 0, 91, 57, 324, 0, 94, 254, 99, 0, 121, 52, 106, 0, 76, 124, 2, 0, 111, 143, 20, 288, 0, 14, 108, 231, 184, 0, 39, 260, 249, 0, 237, 170, 0, 286, 210, 0, 148, 82, 187, 0, 176, 296, 334, 0, 164, 239, 45, 146, 0, 325, 17, 0, 327, 131, 321, 0, 166, 10, 0, 72, 317, 0, 326, 226, 223, 0, 56, 222, 299, 0, 98, 150, 15, 0, 157, 303, 142, 0, 96, 87, 0, 179, 145, 167, 0, 280, 35, 18, 0, 68, 289, 218, 0, 49, 104, 281, 203, 219, 253, 0, 284, 64, 0, 217, 270, 41, 0, 207, 103, 224, 0, 188, 199, 0, 248, 137, 117, 0, 92, 133, 107, 0, 110, 71, 283, 0, 220, 265, 0, 140, 242, 0, 128, 13, 0, 310, 144, 234, 0, 301, 240, 0, 78, 279, 109, 0, 112, 263, 0 };
			costOptimalSolution = 139135;
		}
		else if (basename.compare("X-n344-k43.vrp") == 0)
		{
			optimalSolution = { 0, 309, 41, 246, 15, 72, 129, 224, 304, 185, 0, 275, 190, 229, 142, 209, 25, 61, 250, 213, 0, 74, 50, 48, 235, 53, 148, 254, 112, 0, 52, 103, 56, 86, 28, 105, 102, 125, 0, 194, 251, 186, 326, 328, 323, 274, 266, 16, 0, 330, 293, 173, 259, 221, 222, 271, 290, 0, 243, 174, 329, 216, 298, 169, 146, 0, 255, 245, 308, 70, 200, 247, 228, 181, 0, 24, 93, 333, 338, 147, 143, 118, 0, 132, 244, 278, 81, 14, 285, 150, 0, 158, 77, 195, 231, 144, 149, 65, 39, 157, 0, 106, 320, 116, 279, 183, 140, 78, 9, 0, 223, 230, 288, 305, 342, 198, 176, 292, 0, 1, 152, 312, 135, 260, 124, 261, 75, 0, 241, 238, 55, 110, 76, 337, 272, 0, 339, 94, 18, 101, 108, 164, 114, 98, 0, 23, 95, 253, 2, 5, 4, 291, 0, 11, 27, 313, 45, 6, 58, 167, 20, 26, 0, 188, 115, 171, 296, 104, 12, 67, 205, 0, 145, 7, 37, 156, 49, 51, 151, 21, 0, 208, 325, 202, 218, 331, 211, 172, 340, 341, 0, 154, 100, 62, 17, 119, 163, 137, 59, 0, 159, 3, 60, 34, 303, 97, 170, 268, 0, 113, 267, 92, 197, 121, 332, 187, 0, 299, 88, 128, 204, 237, 319, 322, 122, 0, 248, 182, 277, 192, 46, 10, 252, 71, 0, 280, 184, 240, 85, 109, 236, 307, 165, 0, 31, 239, 203, 317, 289, 68, 257, 136, 0, 196, 233, 141, 80, 315, 318, 134, 0, 91, 83, 42, 269, 161, 90, 294, 177, 0, 217, 179, 219, 22, 300, 210, 276, 270, 262, 0, 96, 111, 343, 66, 79, 335, 201, 212, 0, 284, 215, 258, 334, 273, 178, 295, 206, 0, 242, 199, 191, 283, 127, 311, 227, 281, 0, 327, 120, 29, 207, 64, 36, 139, 153, 0, 314, 193, 282, 38, 220, 189, 263, 0, 107, 57, 19, 226, 225, 306, 160, 0, 54, 99, 69, 162, 13, 89, 47, 73, 133, 0, 84, 264, 180, 131, 126, 117, 87, 249, 0, 82, 321, 63, 310, 316, 286, 166, 155, 0, 35, 43, 123, 32, 40, 8, 44, 130, 0, 302, 256, 287, 232, 265, 33, 175, 138, 214, 0, 301, 234, 297, 168, 324, 336, 30, 0 };
			costOptimalSolution = 42056;
		}
		else if (basename.compare("X-n351-k40.vrp") == 0)
		{
			optimalSolution = { 0, 238, 149, 163, 217, 103, 144, 15, 49, 220, 129, 184, 84, 10, 174, 0, 147, 124, 245, 271, 141, 242, 58, 39, 204, 40, 197, 111, 0, 232, 177, 100, 253, 56, 108, 122, 348, 0, 254, 104, 148, 13, 337, 55, 237, 0, 322, 34, 187, 97, 291, 14, 0, 114, 185, 71, 310, 319, 313, 186, 316, 268, 17, 342, 0, 240, 262, 93, 182, 159, 52, 224, 308, 121, 0, 259, 43, 309, 311, 173, 331, 318, 0, 88, 22, 203, 228, 222, 26, 57, 330, 346, 0, 307, 288, 130, 107, 248, 260, 193, 303, 42, 297, 0, 276, 286, 161, 299, 300, 126, 317, 0, 77, 293, 2, 91, 343, 23, 0, 86, 256, 63, 41, 263, 229, 80, 298, 152, 180, 236, 201, 0, 200, 325, 328, 75, 101, 264, 164, 61, 261, 0, 135, 66, 33, 195, 306, 150, 51, 0, 125, 167, 183, 98, 215, 335, 0, 133, 340, 292, 334, 106, 21, 290, 96, 29, 0, 59, 257, 160, 199, 190, 54, 158, 64, 12, 145, 349, 0, 281, 178, 87, 142, 314, 19, 333, 99, 323, 0, 265, 305, 192, 45, 1, 109, 206, 231, 279, 92, 175, 0, 241, 25, 53, 31, 218, 233, 16, 3, 0, 18, 205, 296, 243, 258, 171, 5, 326, 208, 287, 28, 0, 95, 345, 115, 37, 7, 9, 72, 69, 139, 212, 0, 82, 247, 172, 227, 327, 221, 269, 118, 295, 339, 0, 267, 162, 38, 211, 194, 219, 30, 320, 234, 116, 169, 0, 134, 44, 94, 252, 246, 105, 140, 35, 0, 157, 244, 117, 280, 284, 272, 119, 136, 341, 0, 166, 27, 128, 304, 226, 36, 270, 70, 223, 210, 0, 48, 324, 154, 110, 102, 151, 0, 255, 278, 181, 191, 131, 209, 251, 89, 6, 0, 249, 50, 123, 67, 68, 282, 11, 143, 74, 0, 301, 321, 24, 207, 312, 90, 338, 170, 65, 0, 214, 8, 73, 274, 179, 168, 76, 213, 120, 0, 294, 165, 60, 266, 176, 47, 83, 81, 0, 137, 315, 188, 344, 275, 225, 146, 85, 0, 347, 138, 336, 132, 78, 79, 156, 20, 0, 239, 273, 235, 285, 216, 127, 302, 32, 0, 4, 155, 196, 332, 202, 46, 0, 283, 113, 329, 189, 230, 350, 0, 153, 62, 198, 289, 277, 250, 0, 112, 0 };
			costOptimalSolution = 25928;
		}
		else if (basename.compare("X-n359-k29.vrp") == 0)
		{
			optimalSolution = { 0, 59, 115, 28, 120, 78, 53, 142, 82, 21, 312, 52, 50, 0, 30, 232, 171, 44, 27, 338, 132, 125, 75, 84, 131, 112, 152, 0, 278, 217, 239, 223, 184, 358, 322, 319, 233, 190, 134, 9, 253, 0, 244, 323, 102, 234, 72, 341, 293, 126, 64, 46, 150, 0, 246, 173, 41, 331, 129, 104, 216, 295, 262, 16, 224, 0, 130, 317, 192, 335, 114, 273, 327, 333, 264, 207, 306, 210, 54, 0, 49, 35, 204, 199, 188, 342, 4, 211, 283, 225, 187, 221, 0, 105, 83, 8, 14, 68, 43, 140, 67, 40, 163, 158, 153, 0, 51, 314, 281, 65, 123, 226, 113, 296, 168, 12, 45, 161, 96, 22, 0, 266, 337, 329, 279, 336, 180, 301, 228, 256, 138, 24, 208, 0, 245, 101, 20, 116, 128, 124, 176, 94, 291, 213, 62, 196, 100, 155, 0, 36, 56, 348, 321, 69, 241, 70, 265, 290, 270, 0, 257, 276, 219, 98, 166, 90, 39, 151, 37, 254, 271, 71, 200, 285, 0, 149, 119, 127, 328, 10, 42, 351, 345, 3, 0, 215, 143, 311, 326, 284, 76, 135, 55, 80, 97, 272, 227, 0, 181, 308, 349, 287, 92, 346, 57, 220, 197, 191, 107, 222, 95, 193, 0, 66, 305, 350, 238, 263, 206, 269, 145, 356, 202, 121, 182, 288, 260, 0, 214, 268, 313, 91, 195, 2, 255, 324, 144, 157, 25, 289, 0, 167, 309, 122, 58, 218, 77, 38, 292, 33, 334, 61, 103, 320, 343, 330, 332, 0, 247, 11, 274, 81, 310, 299, 294, 237, 186, 315, 31, 240, 231, 0, 357, 147, 229, 251, 118, 164, 307, 146, 117, 88, 29, 159, 0, 13, 26, 34, 63, 325, 7, 1, 74, 160, 170, 85, 110, 141, 111, 0, 318, 48, 6, 108, 252, 156, 209, 344, 189, 235, 230, 198, 275, 5, 0, 19, 148, 23, 139, 248, 169, 282, 352, 300, 267, 0, 355, 243, 259, 179, 236, 298, 297, 185, 258, 212, 242, 0, 86, 286, 183, 60, 302, 249, 353, 175, 73, 303, 172, 87, 316, 0, 17, 89, 354, 250, 201, 280, 136, 340, 15, 261, 32, 277, 106, 0, 178, 47, 109, 93, 18, 194, 339, 99, 137, 79, 165, 0, 162, 174, 347, 203, 304, 205, 154, 133, 177, 0 };
			costOptimalSolution = 51505;
		}
		else if (basename.compare("X-n367-k17.vrp") == 0)
		{
			optimalSolution = { 0, 255, 217, 234, 2, 298, 348, 7, 364, 220, 91, 157, 72, 77, 71, 31, 128, 314, 0, 121, 83, 248, 331, 52, 145, 26, 209, 6, 352, 155, 109, 302, 13, 141, 180, 44, 324, 100, 0, 163, 218, 159, 9, 190, 226, 158, 18, 296, 353, 362, 233, 0, 80, 207, 326, 42, 300, 144, 103, 342, 17, 84, 311, 183, 64, 101, 271, 189, 351, 79, 160, 170, 0, 41, 322, 165, 345, 119, 166, 126, 286, 259, 222, 8, 124, 90, 194, 303, 199, 86, 295, 355, 287, 0, 232, 162, 67, 88, 333, 285, 211, 282, 205, 120, 105, 244, 15, 338, 250, 196, 32, 37, 249, 192, 321, 95, 320, 0, 254, 129, 107, 150, 294, 208, 46, 87, 336, 216, 307, 293, 50, 261, 113, 240, 111, 3, 288, 354, 156, 312, 347, 186, 0, 262, 30, 63, 297, 228, 289, 96, 349, 110, 340, 99, 178, 24, 22, 363, 212, 188, 47, 0, 76, 225, 323, 266, 164, 316, 315, 117, 202, 235, 291, 245, 221, 305, 339, 12, 329, 269, 142, 146, 161, 299, 203, 182, 256, 0, 210, 82, 241, 45, 277, 350, 172, 5, 4, 75, 258, 40, 237, 66, 356, 0, 198, 130, 153, 151, 200, 366, 268, 357, 20, 73, 148, 114, 125, 327, 361, 175, 38, 206, 343, 193, 133, 138, 0, 28, 346, 313, 281, 131, 58, 23, 273, 318, 29, 36, 108, 54, 143, 122, 335, 112, 136, 191, 309, 257, 85, 60, 185, 169, 0, 173, 132, 246, 272, 181, 104, 168, 236, 330, 98, 106, 213, 62, 224, 127, 93, 48, 59, 276, 215, 149, 360, 43, 21, 251, 35, 227, 179, 147, 57, 25, 265, 177, 310, 56, 334, 0, 167, 283, 140, 201, 184, 1, 118, 264, 123, 317, 39, 14, 70, 247, 0, 19, 68, 197, 174, 187, 11, 10, 292, 139, 230, 231, 0, 204, 92, 137, 358, 243, 278, 74, 341, 365, 89, 102, 176, 16, 53, 325, 51, 359, 238, 116, 304, 152, 279, 49, 27, 267, 0, 78, 154, 195, 171, 239, 135, 115, 319, 301, 308, 94, 69, 284, 260, 253, 328, 344, 263, 332, 270, 275, 337, 290, 34, 65, 134, 306, 55, 229, 274, 97, 280, 33, 252, 81, 219, 223, 242, 214, 61, 0 };
			costOptimalSolution = 22814;
		}
		else if (basename.compare("X-n376-k94.vrp") == 0)
		{
			optimalSolution = { 0, 58, 57, 201, 160, 0, 374, 305, 190, 32, 0, 42, 359, 209, 278, 0, 215, 247, 45, 172, 0, 38, 295, 269, 20, 0, 337, 34, 123, 85, 0, 180, 191, 323, 154, 0, 27, 286, 69, 183, 0, 59, 258, 261, 9, 0, 87, 288, 354, 46, 0, 169, 10, 153, 2, 0, 266, 202, 324, 224, 0, 289, 101, 297, 364, 0, 304, 207, 263, 127, 0, 210, 103, 15, 260, 0, 113, 189, 108, 265, 0, 370, 363, 327, 125, 0, 276, 229, 138, 4, 0, 155, 328, 373, 66, 0, 89, 222, 347, 0, 182, 176, 291, 149, 0, 310, 218, 353, 311, 0, 351, 192, 48, 181, 0, 212, 150, 82, 285, 0, 307, 205, 128, 68, 0, 309, 168, 132, 54, 0, 26, 188, 120, 226, 0, 194, 5, 320, 303, 0, 167, 217, 147, 122, 0, 250, 178, 198, 331, 0, 110, 298, 119, 115, 0, 84, 1, 306, 356, 0, 173, 253, 236, 140, 0, 314, 213, 131, 31, 0, 338, 151, 16, 333, 0, 23, 17, 246, 80, 0, 11, 237, 196, 104, 0, 112, 51, 345, 64, 0, 25, 281, 308, 326, 0, 49, 81, 315, 365, 0, 98, 204, 355, 233, 0, 114, 251, 156, 313, 0, 193, 21, 232, 88, 0, 336, 134, 244, 33, 0, 40, 279, 372, 14, 0, 335, 92, 75, 144, 0, 70, 216, 219, 225, 0, 78, 29, 175, 148, 0, 19, 13, 277, 187, 0, 322, 67, 56, 249, 0, 206, 74, 106, 91, 0, 228, 245, 65, 171, 0, 76, 300, 55, 299, 0, 96, 195, 273, 95, 0, 99, 6, 53, 348, 0, 118, 111, 107, 235, 0, 139, 185, 268, 62, 0, 240, 371, 352, 290, 0, 243, 126, 94, 124, 0, 170, 159, 102, 72, 0, 164, 146, 79, 179, 0, 105, 361, 239, 36, 0, 238, 163, 360, 321, 0, 137, 234, 208, 316, 0, 211, 93, 317, 275, 0, 332, 312, 296, 271, 0, 318, 60, 357, 121, 0, 37, 100, 262, 7, 0, 30, 199, 294, 227, 0, 343, 116, 375, 301, 0, 366, 329, 203, 47, 0, 157, 22, 63, 349, 0, 97, 133, 287, 270, 0, 267, 141, 71, 362, 0, 358, 152, 90, 231, 0, 166, 368, 161, 264, 0, 341, 130, 86, 241, 0, 39, 242, 230, 43, 0, 142, 145, 283, 136, 0, 41, 3, 18, 129, 0, 272, 162, 165, 174, 0, 44, 302, 158, 319, 0, 280, 340, 342, 35, 0, 325, 350, 367, 24, 0, 256, 197, 223, 330, 0, 248, 83, 292, 77, 0, 252, 117, 257, 334, 0, 254, 346, 344, 73, 0, 220, 177, 274, 143, 0, 369, 52, 200, 50, 0, 284, 135, 293, 8, 0, 259, 186, 214, 28, 0, 109, 221, 184, 255, 0, 339, 61, 282, 12, 0 };
			costOptimalSolution = 147713;
		}
		else if (basename.compare("X-n384-k52.vrp") == 0)
		{
			optimalSolution = { 0, 141, 292, 178, 332, 276, 365, 265, 264, 0, 317, 315, 233, 256, 36, 78, 243, 122, 0, 204, 208, 76, 27, 374, 120, 156, 79, 0, 97, 105, 222, 330, 23, 381, 131, 0, 171, 7, 370, 157, 180, 366, 305, 0, 153, 326, 102, 220, 52, 69, 321, 4, 0, 46, 261, 251, 193, 37, 314, 128, 0, 114, 324, 40, 302, 216, 237, 288, 60, 0, 174, 329, 87, 73, 81, 217, 147, 241, 0, 31, 185, 275, 11, 358, 132, 15, 215, 0, 291, 113, 296, 310, 260, 83, 140, 155, 0, 5, 351, 206, 28, 124, 149, 318, 0, 240, 88, 320, 48, 115, 258, 108, 0, 333, 127, 84, 247, 331, 3, 267, 0, 255, 253, 266, 33, 129, 20, 186, 304, 0, 360, 182, 72, 158, 202, 328, 262, 107, 0, 181, 219, 379, 340, 196, 212, 86, 0, 63, 14, 109, 2, 74, 146, 142, 369, 0, 353, 56, 211, 159, 347, 268, 221, 0, 45, 210, 85, 89, 228, 165, 355, 0, 308, 376, 39, 119, 54, 190, 297, 133, 0, 110, 152, 323, 111, 334, 380, 65, 0, 238, 170, 112, 342, 90, 44, 98, 21, 0, 271, 350, 263, 80, 150, 377, 319, 0, 134, 335, 19, 126, 49, 298, 270, 0, 57, 104, 300, 356, 100, 227, 172, 339, 0, 307, 239, 91, 199, 205, 116, 169, 0, 64, 188, 293, 337, 278, 10, 16, 0, 345, 341, 200, 53, 6, 282, 0, 354, 67, 138, 94, 312, 189, 0, 274, 361, 280, 254, 62, 257, 283, 236, 0, 121, 338, 287, 32, 93, 166, 311, 0, 299, 144, 285, 70, 195, 294, 35, 0, 290, 135, 250, 357, 1, 103, 176, 0, 25, 101, 71, 378, 106, 232, 316, 0, 362, 66, 348, 367, 99, 368, 203, 346, 0, 30, 50, 214, 162, 137, 136, 244, 0, 92, 303, 306, 12, 349, 225, 55, 0, 167, 344, 51, 26, 130, 184, 269, 0, 218, 34, 161, 359, 173, 117, 24, 336, 0, 273, 38, 42, 29, 245, 125, 151, 0, 9, 252, 284, 295, 75, 123, 277, 0, 230, 325, 281, 226, 118, 272, 235, 47, 0, 364, 246, 313, 163, 231, 224, 198, 309, 0, 372, 248, 154, 229, 77, 363, 13, 0, 289, 68, 41, 175, 375, 192, 145, 0, 382, 18, 187, 322, 96, 197, 8, 0, 95, 234, 286, 301, 213, 249, 61, 0, 201, 164, 179, 168, 191, 279, 373, 327, 0, 160, 194, 183, 343, 59, 148, 207, 0, 352, 223, 43, 177, 82, 139, 259, 0, 242, 58, 209, 17, 143, 383, 22, 0, 371, 0 };
			costOptimalSolution = 65943;
		}
		else if (basename.compare("X-n393-k38.vrp") == 0)
		{
			optimalSolution = { 0, 73, 49, 126, 71, 96, 164, 139, 146, 90, 330, 120, 125, 0, 220, 154, 369, 20, 9, 61, 309, 355, 264, 0, 105, 65, 137, 1, 58, 129, 84, 174, 162, 179, 127, 0, 158, 4, 184, 29, 78, 104, 67, 68, 145, 2, 177, 0, 244, 8, 169, 141, 19, 46, 152, 323, 349, 0, 128, 136, 69, 130, 3, 180, 74, 23, 110, 62, 40, 0, 34, 60, 39, 32, 16, 28, 43, 147, 193, 358, 170, 0, 85, 196, 157, 53, 108, 5, 42, 98, 33, 22, 181, 0, 64, 161, 163, 112, 31, 99, 41, 173, 144, 106, 0, 192, 187, 133, 10, 30, 119, 44, 11, 183, 190, 0, 138, 194, 191, 77, 118, 166, 59, 91, 218, 238, 0, 111, 195, 189, 159, 168, 75, 107, 182, 15, 35, 0, 82, 57, 117, 36, 156, 37, 114, 97, 6, 26, 0, 352, 100, 52, 17, 115, 7, 83, 150, 50, 89, 0, 143, 124, 38, 95, 176, 79, 94, 14, 25, 178, 172, 0, 81, 295, 72, 345, 237, 63, 249, 360, 344, 103, 0, 140, 290, 383, 339, 142, 24, 167, 116, 246, 342, 92, 0, 322, 209, 223, 308, 175, 48, 280, 240, 102, 0, 54, 368, 109, 288, 211, 390, 255, 362, 18, 326, 317, 0, 198, 337, 364, 357, 363, 371, 121, 225, 243, 236, 0, 123, 343, 372, 221, 206, 272, 389, 86, 212, 0, 252, 347, 283, 66, 353, 235, 274, 251, 227, 0, 287, 289, 13, 80, 365, 377, 356, 135, 228, 286, 0, 354, 261, 373, 333, 265, 263, 233, 348, 374, 201, 260, 0, 165, 270, 266, 324, 276, 153, 335, 216, 375, 385, 0, 70, 229, 380, 241, 370, 262, 204, 391, 232, 222, 329, 0, 321, 325, 282, 340, 277, 367, 328, 259, 200, 307, 0, 171, 27, 214, 56, 199, 284, 281, 294, 250, 301, 185, 0, 258, 388, 210, 314, 302, 245, 205, 318, 392, 296, 273, 0, 299, 134, 239, 376, 313, 215, 379, 275, 278, 292, 0, 336, 148, 316, 382, 304, 45, 188, 350, 319, 268, 257, 226, 122, 0, 21, 253, 310, 279, 224, 242, 303, 315, 160, 366, 0, 155, 55, 76, 346, 306, 88, 297, 149, 300, 0, 101, 361, 341, 51, 217, 327, 151, 113, 132, 386, 0, 271, 12, 381, 285, 197, 298, 131, 202, 254, 256, 311, 0, 334, 305, 248, 203, 207, 234, 378, 332, 231, 213, 0, 384, 359, 351, 387, 87, 230, 47, 247, 293, 269, 93, 0, 320, 331, 267, 219, 312, 291, 186, 208, 338, 0 };
			costOptimalSolution = 38260;
		}
		else if (basename.compare("X-n401-k29.vrp") == 0)
		{
			optimalSolution = { 0, 88, 232, 377, 383, 319, 257, 294, 276, 69, 313, 187, 4, 248, 160, 131, 36, 180, 148, 75, 305, 86, 223, 61, 80, 340, 255, 0, 33, 156, 102, 267, 229, 398, 323, 97, 175, 199, 379, 320, 62, 65, 165, 116, 137, 274, 138, 225, 108, 280, 183, 49, 284, 322, 297, 236, 146, 0, 259, 253, 247, 289, 366, 353, 16, 63, 109, 296, 28, 200, 317, 387, 314, 258, 19, 206, 47, 286, 300, 355, 370, 153, 385, 158, 55, 390, 104, 207, 254, 277, 30, 0, 239, 130, 338, 93, 362, 343, 226, 372, 250, 179, 333, 204, 208, 122, 342, 141, 396, 378, 196, 266, 310, 298, 181, 273, 149, 177, 0, 178, 334, 316, 304, 215, 301, 327, 145, 360, 237, 295, 358, 349, 56, 188, 1, 190, 103, 43, 217, 143, 0, 59, 306, 18, 31, 210, 335, 373, 27, 73, 21, 81, 198, 0, 147, 202, 211, 79, 3, 150, 172, 235, 176, 78, 20, 384, 308, 243, 163, 110, 101, 64, 140, 0, 121, 74, 356, 157, 307, 234, 76, 136, 397, 369, 213, 166, 233, 45, 39, 0, 361, 194, 221, 34, 128, 154, 203, 262, 324, 347, 270, 315, 216, 392, 0, 238, 25, 106, 84, 339, 57, 151, 85, 288, 241, 11, 329, 279, 0, 117, 119, 242, 303, 354, 95, 246, 124, 82, 283, 0, 352, 278, 26, 224, 201, 52, 142, 245, 374, 105, 0, 400, 127, 268, 261, 219, 348, 92, 281, 231, 291, 174, 139, 0, 260, 133, 152, 8, 275, 114, 375, 94, 14, 41, 214, 0, 32, 192, 123, 99, 386, 271, 186, 389, 164, 0, 382, 331, 91, 197, 311, 309, 71, 227, 129, 230, 264, 0, 209, 292, 13, 50, 144, 22, 60, 83, 169, 68, 0, 325, 173, 125, 328, 287, 182, 44, 38, 185, 244, 0, 332, 222, 15, 89, 380, 113, 77, 70, 220, 12, 0, 168, 9, 285, 40, 17, 391, 46, 346, 251, 0, 299, 167, 351, 191, 282, 98, 90, 107, 321, 341, 318, 0, 371, 365, 42, 249, 326, 193, 290, 359, 376, 0, 120, 155, 58, 184, 265, 159, 135, 312, 344, 118, 0, 7, 240, 51, 330, 2, 24, 10, 337, 96, 35, 0, 381, 367, 23, 195, 256, 228, 126, 263, 388, 37, 0, 115, 345, 302, 293, 66, 132, 67, 368, 0, 205, 357, 212, 393, 29, 171, 161, 48, 53, 395, 0, 252, 111, 100, 218, 272, 336, 6, 350, 5, 87, 394, 0, 162, 269, 363, 72, 134, 364, 170, 54, 399, 112, 189, 0 };
			costOptimalSolution = 66187;
		}
		else if (basename.compare("X-n411-k19.vrp") == 0)
		{
			optimalSolution = { 0, 410, 326, 234, 126, 93, 279, 30, 24, 63, 369, 318, 349, 372, 246, 259, 87, 134, 395, 355, 337, 0, 115, 302, 212, 377, 26, 330, 292, 160, 6, 105, 300, 133, 364, 194, 0, 245, 407, 294, 338, 312, 229, 83, 253, 351, 1, 7, 211, 175, 0, 257, 4, 239, 138, 139, 124, 60, 183, 33, 0, 29, 112, 173, 281, 19, 249, 17, 129, 352, 400, 244, 171, 224, 252, 0, 233, 31, 286, 346, 288, 150, 57, 331, 54, 187, 333, 178, 141, 196, 18, 164, 222, 264, 123, 43, 155, 65, 240, 81, 303, 319, 136, 304, 121, 0, 186, 386, 108, 64, 163, 188, 25, 350, 74, 15, 101, 307, 220, 282, 247, 128, 0, 176, 67, 385, 46, 48, 250, 363, 359, 357, 179, 335, 94, 27, 100, 91, 317, 388, 117, 228, 10, 49, 42, 341, 389, 293, 316, 248, 390, 103, 0, 339, 79, 209, 289, 75, 260, 201, 56, 185, 214, 148, 193, 8, 206, 166, 232, 159, 374, 403, 298, 84, 190, 353, 287, 267, 191, 39, 0, 320, 92, 52, 334, 96, 225, 223, 3, 149, 379, 284, 301, 313, 384, 328, 21, 147, 358, 165, 0, 332, 370, 23, 291, 158, 174, 378, 35, 16, 157, 268, 383, 321, 373, 114, 285, 343, 89, 362, 0, 315, 314, 120, 258, 276, 345, 41, 127, 156, 118, 405, 12, 367, 106, 62, 354, 295, 202, 104, 0, 80, 86, 221, 261, 199, 14, 61, 71, 66, 20, 391, 53, 85, 205, 102, 0, 398, 408, 342, 177, 322, 70, 38, 189, 297, 192, 402, 152, 200, 283, 181, 251, 51, 306, 203, 55, 256, 393, 197, 50, 269, 394, 45, 143, 381, 145, 356, 404, 76, 409, 219, 217, 235, 273, 184, 365, 153, 237, 0, 238, 280, 274, 309, 113, 131, 213, 119, 116, 361, 151, 36, 278, 230, 130, 254, 77, 99, 324, 406, 236, 2, 34, 255, 392, 380, 137, 366, 387, 311, 69, 0, 399, 44, 305, 216, 227, 167, 207, 9, 272, 327, 146, 323, 13, 154, 58, 140, 242, 215, 275, 0, 122, 162, 73, 90, 376, 277, 59, 95, 344, 336, 40, 125, 172, 296, 68, 47, 28, 329, 142, 401, 368, 266, 22, 37, 231, 310, 180, 198, 195, 371, 109, 0, 360, 347, 271, 290, 97, 397, 107, 111, 170, 226, 5, 182, 263, 396, 204, 144, 270, 218, 208, 82, 0, 243, 135, 11, 325, 98, 161, 78, 375, 308, 265, 110, 340, 210, 132, 72, 262, 241, 299, 348, 382, 88, 32, 168, 169, 0 };
			costOptimalSolution = 19718;
		}
		else
		{
			throw std::string("ERROR No optimal solution defined\n");
		}
	}

	SideExperiments(Params *params, Mining *mining) : params(params), mining(mining)
	{
		readOptimalSolutionCharacteristics();
	}

};
#endif

#endif
