#include "Params.h"

void Params::setMethodParams()
{
	el = 6;
	mu = 25;
	lambda = 40;
	nbCountDistMeasure = 3;
	rho = 0.30;
	proxCst = 20;
	minValides = 0.25;
	maxValides = 0.35;
	minSeqMining = 2;
	progressReports = {1,2,5,10,15,20,30,50,75,100};

	// Baseline parameters
	maxSeqMining = 5;
	factorPatternMining = 5;
	factorPatternTesting = 1;
	samplingRatioMining = 10;
	settingPILS = 1;

	// Setting customer parameters (for the calibration tests)
	if      (seed <= 10)  settingPILS = 0;
	else if (seed <= 20)  settingPILS = 1;
	else if (seed <= 30)  settingPILS = 2;
	else if (seed <= 40)  settingPILS = 3;
	else if (seed <= 50)  factorPatternMining = 2;
	else if (seed <= 60)  factorPatternMining = 3;
	else if (seed <= 70)  factorPatternMining = 10;
	else if (seed <= 80)  factorPatternMining = 20;
	else if (seed <= 90)  factorPatternTesting = 0.2;
	else if (seed <= 100) factorPatternTesting = 0.5;
	else if (seed <= 110) factorPatternTesting = 1.5;
	else if (seed <= 120) factorPatternTesting = 2.0;
	else if (seed <= 130) maxSeqMining = 3;
	else if (seed <= 140) maxSeqMining = 4;
	else if (seed <= 150) maxSeqMining = 6;
	else if (seed <= 160) maxSeqMining = 7;
	else if (seed <= 170) samplingRatioMining = 5;
	else if (seed <= 180) samplingRatioMining = 20;
}

void Params::readInstance()
{
	std::string contenu;
	int useless ;
	maxDemand = 0.;

	if (type == 0 || type == 1 || type == 2) // Default input for CMT and Golden instances, the file type will be read from the instance
	{
		isRoundingInteger = false;
		inputFile >> type >> nbVehicles >> nbClients >> useless >> durationLimit >> vehicleCapacity;
		if (durationLimit == 0) {durationLimit = 1.e30; isDurationConstraint = false;}
		else isDurationConstraint = true;
	}
	else if (type == 3) // VRP format of Uchoa et al.
	{
		isRoundingInteger = true ; // The X instances of Uchoa et al. require rounding the distances
		isDurationConstraint = false ; // There are no duration constraints for these instances
		durationLimit = 1.e30;
		getline(inputFile, contenu);
		getline(inputFile, contenu);
		getline(inputFile, contenu);
		inputFile >> contenu ;
		inputFile >> contenu ;
		inputFile >> nbClients ;
		nbClients -- ; // The file indicates the total number of nodes (including the depot)
		getline(inputFile, contenu);
		getline(inputFile, contenu);
		inputFile >> contenu ;
		inputFile >> contenu ;
		inputFile >> vehicleCapacity;
		getline(inputFile, contenu);
		getline(inputFile, contenu);
	}

	// Read client information
	for (int i = 0 ; i <= nbClients ; i++)
		cli.push_back(readClient (i));

	// In Uchoa's data format, the customer demands are located at the end of the file
	if (type == 3)
	{
		getline(inputFile, contenu);
		getline(inputFile, contenu);
		for (int i = 0 ; i <= nbClients ; i++)
		{
			inputFile >> useless ;
			inputFile >> cli[i].demand ;
			if (cli[i].demand > maxDemand) maxDemand = cli[i].demand;
		}
	}

	// Evaluate the total demand and minimum number of vehicles necessary for it
	totalDemand = 0.;
	for (int i = 1; i <= nbClients; i++)
		totalDemand += cli[i].demand;
	lbVehicles = (int)std::ceil(totalDemand / vehicleCapacity);

	// Default initialization if the number of vehicles is not given by the user or instance file
	if (nbVehicles == -1)
	{
		nbVehicles = (int)std::ceil((double)lbVehicles*1.2) + 2;  // Default value: we give 20% more vehicles and 2 extra than the minimum given by the LB
		std::cout << "----- FLEET SIZE NOT SPECIFIED. DEFAULT INITIALIZATION TO: " << nbVehicles << std::endl;
	}
}

void Params::setupDataStructures () 
{
	// Distance matrix calculation 
	maxDist = 0.;
	timeCost = std::vector < std::vector< double > >(nbClients + 1, std::vector <double>(nbClients + 1));
	for (int i = 0; i <= nbClients; i++)
	{
		for (int j = 0; j <= nbClients; j++)
		{
			double d = std::sqrt((cli[i].coordX - cli[j].coordX)*(cli[i].coordX - cli[j].coordX) + (cli[i].coordY - cli[j].coordY)*(cli[i].coordY - cli[j].coordY));
			if (isRoundingInteger) { d += 0.5; d = (double)(int)d; } // integer rounding
			if (d > maxDist) maxDist = d;
			timeCost[i][j] = d;
		}
	}

	// Nearest neighbors calculation
	correlatedVertices = std::vector < std::vector < int > >(nbClients + 1);
	std::vector < std::set < int > > setCorrelatedVertices = std::vector < std::set <int> >(nbClients + 1);
	std::vector < std::pair <double, int> > ordreProximite;
	for (int i=1 ; i <= nbClients ; i++)
	{
		ordreProximite.clear();
		for (int j=1 ; j <= nbClients ; j++)
			if ( i != j ) ordreProximite.push_back(std::pair <double, int> (timeCost[i][j],j));
		std::sort(ordreProximite.begin(), ordreProximite.end());

		for (int j = 0; j < std::min<int>(proxCst, nbClients-1); j++)
		{
			// If i is correlated with j, then j should be correlated with i
			setCorrelatedVertices[i].insert(ordreProximite[j].second); 
			setCorrelatedVertices[ordreProximite[j].second].insert(i);
		}
	}

	// Filling the vector of correlated vertices
	for (int i = 1; i <= nbClients; i++)
		for (int x: setCorrelatedVertices[i])
			correlatedVertices[i].push_back(x);

	// Safeguards to avoid possible numerical instability in case of instances containing arbitrarily small or large numerical values
	if (maxDist < 0.1   || maxDist > 10000)   throw("ERROR: The distances are of very small or large scale. This could impact numerical stability. Please rescale the dataset and run again.");
	if (maxDemand < 0.1 || maxDemand > 10000) throw("ERROR: The load quantities are of very small or large scale. This could impact numerical stability. Please rescale the dataset and run again.");

	// A reasonable scale for the initial values of the penalties
	penalityLength = 1;
	penalityCapa = std::max<double>(0.01, std::min<double>(1000.,maxDist / maxDemand));

	// Initializing vectors used for statistics (we opted for a fixed size, for calibration purposes and output standardization)
	totalPILSMovesApplied_PerPatternSize = std::vector<int>(11, 0);
	totalPILSMovesApplied_PerInvolvedRoutes = std::vector<int>(11, 0);
	totalPILSMovesApplied_PerMoveOrder = std::vector<int>(21, 0);
}

Client Params::readClient (int i)
{
	Client client ;
	int nbPatterns ;
	int temp ;

	if (type == 1) // file format for CMT instances
	{
		inputFile >> client.custNum >> client.coordX >> client.coordY
			>> client.serviceDuration >> client.demand >> temp >> nbPatterns  ;
		if (client.demand > maxDemand) maxDemand = client.demand;
		for (int j = 0; j < nbPatterns; j++) inputFile >> temp; // Discarding pattern information (CVRP instances only).
	}
	else if (type == 2) // file format for Golden instances
	{
		client.custNum = i ;
		inputFile >> client.coordX >> client.coordY >> client.demand ;
		if (client.demand > maxDemand) maxDemand = client.demand;
		client.serviceDuration = 0 ;
	}
	else if (type == 3) // file format of Uchoa et al. instances
	{
		inputFile >> client.custNum >> client.coordX >> client.coordY;
		client.custNum -- ;
		client.demand = 0 ;
		client.serviceDuration = 0 ;
	}
	return client ;
}

Params::Params(std::string pathToInstance, std::string pathToSolution, int type, int nbVeh, int seedRNG, int settingPILS) : seed(seedRNG), pathToInstance(pathToInstance), pathToSolution(pathToSolution), type(type), nbVehicles(nbVeh), settingPILS(settingPILS)
{
	int mySeed = seed;
	if (mySeed == 0) mySeed = (int)time(NULL);

	// For calibration test, we will always use seeds 0 to 9
	std::srand((mySeed-1)%10);		
	std::cout << "----- INITIALIZING WITH SEED: " << (mySeed-1)%10 << " ----- " << std::endl;

	inputFile.open(pathToInstance.c_str());
	if (inputFile.is_open())
		readInstance();
	else
		throw std::invalid_argument("Impossible to open instance file: " + pathToInstance);

	startTime = clock();
	setMethodParams();
	setupDataStructures();
}