#ifndef PARAMS_H
#define PARAMS_H

#include <string>
#include <vector>
#include <list>
#include <set>
#include <fstream>
#include <iostream>
#include <math.h>
#include <time.h>
#include <algorithm>
#define MY_EPSILON 0.00001 // Precision parameter, used to avoid numerical instabilities

//HOW TO RUN SIDE EXPERIMENTS?
//Just replace 0 with 1 or 2.
//Please, access SideExperiments.h to check which experiment you wish to run. 
#define ACTIVE_SIDE_EXPERIMENTS 0

/**
 * Author: Thibaut Vidal
 * 
 * It gathers all HGSADC’s params, making easier to access this kind of content during runtime.
 * In addition, this class also implements all preprocessed data structures for move evaluation in O(1) amortized.
 * 
 */

struct Client
{
	int custNum;			// Index of the customer	
	double coordX;			// Coordinate X
	double coordY;			// Coordinate Y
	double serviceDuration; // Service duration
	double demand;			// Demand
};

class Params
{
public:

	/* GENERAL PARAMETERS */
	int seed;						// Random seed
	std::string pathToInstance;		// Path to the instance
	std::string pathToSolution;		// Path to the solution
	clock_t startTime;				// Time when the algorithm started
	std::ifstream inputFile;		// Input stream to read the input file

	/* PARAMETERS OF THE GENETIC ALGORITHM */
	int proxCst;						// Granular search parameter
	int nbCountDistMeasure;				// Number of closest individuals considered in the diversity contribution metric
	int el;								// Number of elite individuals in the population
	int mu;								// Minimum population size
	int lambda;							// Number of solutions created before reaching the maximum population size
	double penalityCapa;				// Penalty coefficient for one unit of capacity violation
	double penalityLength;				// Penalty coefficient for one unit of distance violation
	double minValides;					// Target (lower) threhold for the number of feasible individuals
	double maxValides;					// Target (upper) threhold for the number of feasible individuals
	double rho;							// Fraction of the population preserved in the diversification phase
	
	/* FOR REGULAR PROGRESS REPORT */
	std::vector <int> progressReports;	// Checkpoints at which the algorithm should export information
	int indexProgressReport ;			// Index of the current status of method in terms of progress report

	/* PARAMETERS OF THE PILS */
	int minSeqMining;			    // Minimum size of the sequences used in the mining mechanism	
	int maxSeqMining;			    // Maximum size of the sequences used in the mining mechanism
	int samplingRatioMining;		// Fraction of (locally minimum) solutions mined will be 1/samplingRatioMining, to avoid overhead due to pattern extraction
	double factorPatternMining;	// The number of "most frequent" sequences which will be tracked
	double factorPatternTesting;	// The number of patterns which will be tested in each LS (eandomy selected in the set of factorPatternMining)
	int settingPILS;				// PILS setting: OFF=0, BEFORE=1, AFTER=2, BEFORE&AFTER=3

	/* STATISTICS COLLECTED ON THE RUN */
	double totalExtractionTimePILS = 0.;
	double totalCPUTimePILS = 0.;
	double totalCPUTimeLS = 0.;
	int totalExtractions = 0;
	int totalPILSPhases = 0;
	int totalPILSMovesTested = 0;
	int totalPILSMovesApplied = 0;
	int totalLSMovesApplied = 0;
	std::vector<int> totalPILSMovesApplied_PerPatternSize;
	std::vector<int> totalPILSMovesApplied_PerInvolvedRoutes;
	std::vector<int> totalPILSMovesApplied_PerMoveOrder;

	/* DATA OF THE PROBLEM INSTANCE */

	// Instance format during parsing
	// 0 => Non-specified, assumes that the first integer in the instance file defines the type
	// 1 => CMT instances
	// 2 => Golden instances
	// 3 => X instances (from Uchoa et al). Same instance format used for Belgian districts by Florian Arnold.
	int type ;				

	bool isRoundingInteger ;								// Distance calculation convention
	bool isDurationConstraint ;								// Indicates if the problem includes duration constraints
	int nbClients ;											// Number of customers
	int nbVehicles ;										// Number of vehicles
	double totalDemand ;									// Total demand required by the clients
	int lbVehicles ;										// LB on the minimum number of vehicles needed to satisfy the demand (linear relaxation)
	double durationLimit;									// Route duration limit
	double vehicleCapacity;									// Capacity limit
	double maxDemand;										// Maximum demand of a customer
	double maxDist;											// Maximum distance between two customers
	std::vector < Client > cli ;							// Vector containing information on each customer
	std::vector < std::vector < double > > timeCost ;		// Distance matrix
	std::vector < std::vector < int > > correlatedVertices;	// Neighborhood restrictions: For each customer, list of nearby customers

	/* PARSING ROUTINES */

	// Setting the parameters of the method
	void setMethodParams();

	// Preprocess the other datastructures and computes the distance matrix
	void setupDataStructures();

	// Reads the instance file
	void readInstance();

	// Reads the information about customer i in the file
	Client readClient (int i);

	// Constructor
	Params(std::string pathToInstance, std::string pathToSolution, int type, int nbVeh, int seedRNG, int settingPILS);
};
#endif

