#include "Population.h"

void Population::generatePopulation()
{
	for (int i = 0; i < 4 * params->mu; i++)
	{
		Individual * randomIndiv = new Individual(params, true);
		split->generalSplit(randomIndiv, params->nbVehicles);
		localSearch->run(randomIndiv);
		addIndividual(randomIndiv, true);
		if (!randomIndiv->isFeasible && std::rand() % 2 == 0) // 50% CHANCES REPAIR
		{
			localSearch->repair(randomIndiv);
			if (randomIndiv->isFeasible)
				addIndividual(randomIndiv, false);
		}

		// MINING SEQUENCE INFORMATION ON A PERCENTAGE OF THE LOCAL MINIMA
		if (std::rand() % params->samplingRatioMining == 0)
			mining->collect(randomIndiv);

		delete randomIndiv;
	}
}

void Population::evalBiasedFitnessPopulation(SubPopulation & pop)
{
	// Ranking individuals based on their diversity contribution
	std::vector <std::pair <double, int> > classement;
	for (int i = 0 ; i < pop.size(); i++ )
		classement.push_back(std::pair <double, int>(-pop[i]->averageDistanceClosest(params->nbCountDistMeasure),i)) ;
	std::sort(classement.begin(), classement.end()); // Sorting by decreasing order of the distance to closest individual

	// Computing the biased fitness
	for (int i = 0 ; i < pop.size(); i++ )
	{
		double divRank = (double)i / (double)(pop.size() - 1);
		double fitRank = (double)classement[i].second / (double)(pop.size() - 1);
		pop[classement[i].second]->biasedFitness = fitRank + (1.0-(double)params->el/(double)pop.size()) * divRank ;
	}
}

int Population::addIndividual(Individual * indiv, bool updateFeasible)
{
	if (updateFeasible)
	{
		listFeasibilityLoad.push_back(indiv->myCostSol.capacityViol < MY_EPSILON);
		listFeasibilityDuration.push_back(indiv->myCostSol.lengthViol < MY_EPSILON);
		listFeasibilityLoad.pop_front();
		listFeasibilityDuration.pop_front();
	}
	
	if (indiv->isFeasible)
	{
		int result = placeIndividual(feasibleSubpopulation, indiv);
		if (feasibleSubpopulation.size() > params->mu + params->lambda)
			while (feasibleSubpopulation.size() > params->mu)
				removeIndividual(feasibleSubpopulation, getWorstBiasedFitness(feasibleSubpopulation));
		return result;
	}
	else
	{
		int result = placeIndividual(infeasibleSubpopulation, indiv);
		if (infeasibleSubpopulation.size() > params->mu + params->lambda)
			while (infeasibleSubpopulation.size() > params->mu)
				removeIndividual(infeasibleSubpopulation, getWorstBiasedFitness(infeasibleSubpopulation));
		return result;
	}
	
}

void Population::updateProximity (SubPopulation & pop, Individual * indiv)
{
	for (int k = 0; k < pop.size(); k++)
	{
		if (pop[k] != indiv)
		{
			pop[k]->addProximity(indiv);
			indiv->addProximity(pop[k]);
		}
	}
}

bool Population::fitExists ( SubPopulation & pop, Individual * indiv )
{
	double fitness = indiv->myCostSol.penalizedCost;
	for (int i=0 ; i < (int)pop.size() ; i++ )
	{
		if ( indiv != pop[i] && pop[i]->myCostSol.penalizedCost >= (fitness - 0.001) && pop[i]->myCostSol.penalizedCost <= (fitness + 0.001))
			return true ;
	}
	return false ;
}

void Population::diversify ()
{
	while (feasibleSubpopulation.size() > (int)(params->rho*(double)params->mu))
	{
		delete feasibleSubpopulation[feasibleSubpopulation.size()-1] ;
		feasibleSubpopulation.pop_back();
	}

	while (infeasibleSubpopulation.size() > (int)(params->rho*(double)params->mu))
	{
		delete infeasibleSubpopulation[infeasibleSubpopulation.size()-1] ;
		infeasibleSubpopulation.pop_back();
	}

	generatePopulation();
}

void Population::managePenalties()
{
	bool changeDone = false;
	
	// Adapt penalty values
	double fractionCharge = fractionFeasibleLoad();
	if (fractionCharge < params->minValides && params->penalityCapa < 1000)
	{
		params->penalityCapa = params->penalityCapa * 1.2;
		changeDone = true;
	}
	else if (fractionCharge > params->maxValides && params->penalityCapa > 0.01)
	{
		params->penalityCapa = params->penalityCapa * 0.85;
		changeDone = true;
	}

	double fractionTemps = fractionFeasibleDuration();
	if (fractionTemps < params->minValides && params->penalityLength < 1000)
	{
		params->penalityLength = params->penalityLength * 1.2;
		changeDone = true;
	}
	else if (fractionTemps > params->maxValides && params->penalityLength > 0.01)
	{
		params->penalityLength = params->penalityLength * 0.85;
		changeDone = true;
	}

	// Update the cost of the individuals if the penalty coefficients have changed
	if (changeDone)
	{
		Individual * indiv;
		// Update the evaluations
		for (int i = 0; i < infeasibleSubpopulation.size(); i++)
			infeasibleSubpopulation[i]->myCostSol.penalizedCost = infeasibleSubpopulation[i]->myCostSol.distance
			+ params->penalityCapa * infeasibleSubpopulation[i]->myCostSol.capacityViol
			+ params->penalityLength * infeasibleSubpopulation[i]->myCostSol.lengthViol;

		for (int i = 0; i < infeasibleSubpopulation.size(); i++)
		{
			for (int j = 0; j < infeasibleSubpopulation.size() - i - 1; j++)
			{
				if (infeasibleSubpopulation[j]->myCostSol.penalizedCost > infeasibleSubpopulation[j + 1]->myCostSol.penalizedCost + MY_EPSILON)
				{
					indiv = infeasibleSubpopulation[j];
					infeasibleSubpopulation[j] = infeasibleSubpopulation[j + 1];
					infeasibleSubpopulation[j + 1] = indiv;
				}
			}
		}
	}
}

int Population::placeIndividual(SubPopulation & pop, Individual * indiv)
{
	Individual * monIndiv = new Individual(params,false);
	Individual::copy(monIndiv, indiv);
	pop.push_back(monIndiv);
	int i = (int)pop.size() - 2;
	while (i >= 0)
	{
		if (pop[i]->myCostSol.penalizedCost > monIndiv->myCostSol.penalizedCost - MY_EPSILON) // New individuals are priviledged in case of tie
		{
			pop[i + 1] = pop[i];
			i--;
		}
		else
		{
			pop[i + 1] = monIndiv;
			updateProximity(pop, pop[i + 1]);
			return i + 1;
		}
	}
	
	pop[0] = monIndiv;
	updateProximity(pop, pop[0]);
	if (pop == feasibleSubpopulation && (pop.size() == 1 || monIndiv->myCostSol.penalizedCost < pop[1]->myCostSol.penalizedCost - MY_EPSILON))
		timeFoundBest = clock();
	return 0; 
}

void Population::removeIndividual(SubPopulation & pop, int p)
{
	Individual * leavingIndividual = pop[p];

	for ( int i=p+1 ; i < (int)pop.size() ; i++ )
		pop[i-1] = pop[i] ;
	pop.pop_back();

	for (int i=0 ; i < pop.size(); i++ ) 
		pop[i]->removeProximity(leavingIndividual);

	delete leavingIndividual ;
}

Individual * Population::getBinaryTournament ()
{
	Individual * individual1 ;
	Individual * individual2 ;

	evalBiasedFitnessPopulation(feasibleSubpopulation);
	evalBiasedFitnessPopulation(infeasibleSubpopulation);
	
	int place1 = std::rand() % (feasibleSubpopulation.size() + infeasibleSubpopulation.size()) ;
	if ( place1 >= feasibleSubpopulation.size())
		individual1 = infeasibleSubpopulation[place1 - feasibleSubpopulation.size()] ;
	else 
		individual1 = feasibleSubpopulation[place1] ;

	int place2 = std::rand() % (feasibleSubpopulation.size() + infeasibleSubpopulation.size()) ;
	if ( place2 >= feasibleSubpopulation.size())
		individual2 = infeasibleSubpopulation[place2 - feasibleSubpopulation.size()] ;
	else 
		individual2 = feasibleSubpopulation[place2] ;

	if (individual1->biasedFitness < individual2->biasedFitness)
		return individual1 ;
	else
		return individual2 ;		
}

Individual * Population::getBestFeasible ()
{
	if (!feasibleSubpopulation.empty()) return feasibleSubpopulation[0] ;
	else return NULL ;
}

Individual * Population::getBestInfeasible ()
{
	if (!infeasibleSubpopulation.empty()) return infeasibleSubpopulation[0] ;
	else return NULL ;
}

void Population::exportBestSolution (std::string fileName)
{
	std::ofstream myfile;
	myfile.precision(10);
	std::cout.precision(10);

	std::cout << "----- CHECKPOINT: " << params->progressReports[params->indexProgressReport] << "(%) -----" << std::endl;

	Individual * bestFeasible = getBestFeasible ();
	if (bestFeasible != NULL)
	{
		// Appending to the file if it's not the first checkpoint
		if (params->indexProgressReport == 0) myfile.open(fileName.data());
		else myfile.open(fileName.data(), std::ios_base::app);
		if (myfile.is_open())
		{
			myfile << params->pathToInstance << ";";								// Instance
			myfile << params->seed << ";";											// Seed
			myfile << params->nbClients << ";";										// n
			myfile << params->nbVehicles << ";";									// m
			myfile << params->settingPILS << ";";									// PILS status
			myfile << params->factorPatternMining << ";";							// Phi_Freq
			myfile << params->factorPatternTesting << ";";							// Phi_Size
			myfile << params->maxSeqMining << ";";									// L_max
			myfile << 1.0/(double)params->samplingRatioMining << ";";				// P_ex
			myfile << params->progressReports[params->indexProgressReport] << ";";	// Percentage of progress of the method when the data is printed

			myfile << bestFeasible->myCostSol.penalizedCost << ";";					// Solution value
			myfile << bestFeasible->myCostSol.nbRoutes << ";";						// Number of routes in the solution
			myfile << (double)clock()/(double)CLOCKS_PER_SEC << ";";				// Total CPU time (seconds)
			myfile << params->totalCPUTimeLS << ";";								// Total CPU time of LS (seconds)
			myfile << params->totalExtractionTimePILS << ";";						// Total Extraction time of PILS (seconds)
			myfile << params->totalCPUTimePILS << ";";								// Total CPU time of PILS (seconds)
			myfile << params->totalExtractions << ";";								// Total number of extractions
			myfile << params->totalPILSPhases << ";";								// Total number of PILS
			myfile << params->totalPILSMovesTested << ";";							// Total number of Injections Tested
			myfile << params->totalPILSMovesApplied << ";";							// Total number of Injections Applied
			
			// Total number of Injections Applied (per pattern size) 
			for (int x=2 ; x <= 10 ; x++) 
				myfile << params->totalPILSMovesApplied_PerPatternSize[x] << ";";
			
			// Total number of Injections Applied (per involved routes) 
			for (int x = 1; x <= 10; x++) 
				myfile << params->totalPILSMovesApplied_PerInvolvedRoutes[x] << ";";
			
			// Total number of Injections Applied (per move order)
			for (int x = 2; x <= 20; x++) 
				myfile << params->totalPILSMovesApplied_PerMoveOrder[x] << ";";

			myfile << (double)timeFoundBest / (double)CLOCKS_PER_SEC << ";";		// Exporting the CPU time to best solution (seconds)
			myfile << params->totalLSMovesApplied << ";";							// Total number of LS moves Applied
			myfile << std::endl;

			// COMMENTED PROCEDURES (UNCOMMENT FOR USUAL OUTPUT)
			/*std::cout << "----- WRITING BEST SOLUTION WITH VALUE " << bestFeasible->myCostSol.penalizedCost << " IN : " << fileName.c_str() << std::endl;
			myfile << "COST: " << bestFeasible->myCostSol.penalizedCost << std::endl;							  // Exporting solution value
			myfile << "NB_ROUTES: " << bestFeasible->myCostSol.nbRoutes << std::endl;                             // Exporting the number of routes
			myfile << "TIME(ms): " << (int)(1000.*(double)clock() / (double)CLOCKS_PER_SEC) << std::endl;         // Exporting the total CPU time in milliseconds
			myfile << "FOUND(ms): " << (int)(1000.*(double)timeFoundBest / (double)CLOCKS_PER_SEC) << std::endl;  // Exporting the time to best solution in milliseconds
			myfile << "TIME-LS(s): " << params->totalCPUTimeLS << std::endl;      
			myfile << "TIME-PILS(s): " << params->totalCPUTimePILS << std::endl;

			// Printing the routes
			int routeIndex = 1;
			for (int k = 0; k < params->nbVehicles; k++)
			{
				if (!bestFeasible->chromR[k].empty())
				{
					myfile << "ROUTE " << routeIndex << " WITH " << bestFeasible->chromR[k].size() << " CUSTOMER VISITS: 0";
					for (int i = 0; i < bestFeasible->chromR[k].size(); i++)
						myfile << " " << bestFeasible->chromR[k][i];
					myfile << " 0" << std::endl;
					routeIndex++;
				}
			}*/

			myfile.close();
		}
		else
			std::cout << "----- IMPOSSIBLE TO OPEN SOLUTION FILE: " << params->pathToSolution << " ----- " << std::endl;
	}
	else
		std::cout << "----- NO FEASIBLE SOLUTION FOUND -----" << std::endl;
}

// retourne la fraction d'individus valides en terme de charge
double Population::fractionFeasibleLoad ()
{ 
	int count = 0 ;
	for (auto it = listFeasibilityLoad.begin(); it != listFeasibilityLoad.end(); ++it )
		if (*it == true) count ++ ;
	return double(count)/100. ;
}

// retourne la fraction d'individus valides en terme de temps
double Population::fractionFeasibleDuration ()
{ 
	int count = 0 ;
	for (auto it = listFeasibilityDuration.begin(); it != listFeasibilityDuration.end(); ++it )
		if (*it == true) count ++ ;
	return double(count)/100.;
}

double Population::getDiversity(SubPopulation & pop)
{
	double total = 0 ;
	int size = (int)pop.size()/2;
	for (int i = 0; i < size ; i++)
		total += pop[i]->averageDistanceClosest(size);

	if (size > 0) return total / (double)size;
	else return -1;
}	

double Population::getAverageCostFeasible ()
{
	double average = 0 ;
	int size = (int)feasibleSubpopulation.size()/2;
	for (int i=0 ; i < size ; i ++)
		average += feasibleSubpopulation[i]->myCostSol.penalizedCost;

	if (size > 0) return average / (double)size;
	else return 0.;
}

double Population::getAverageCostInfeasible ()
{
	double average = 0 ;
	int size = (int)infeasibleSubpopulation.size() / 2;
	for (int i=0 ; i < size ; i ++)
		average += infeasibleSubpopulation[i]->myCostSol.penalizedCost;
	
	if (size > 0) return average / (double)size;
	else return 0.;
}

int Population::getWorstBiasedFitness(SubPopulation & pop)
{
	evalBiasedFitnessPopulation(pop);
	double worstBiasedFitness = -1.e30;
	int worstIndividual = -1;
	for (int i=1 ; i < pop.size(); i++)
	{
		double myBiasedFitness = pop[i]->biasedFitness;
		if (pop[i]->averageDistanceClosest(1) < MY_EPSILON) myBiasedFitness += 5 ;
		if (fitExists(pop,pop[i])) myBiasedFitness += 5 ;
		if (myBiasedFitness > worstBiasedFitness)
		{
			worstBiasedFitness = myBiasedFitness;
			worstIndividual = i;
		}
	}
	return worstIndividual;
}

void Population::printState(int nbIter)
{
	std::printf("It %6d | Sol ", nbIter);

	if (getBestFeasible() != NULL) {
		std::printf("%.3f ", getBestFeasible()->myCostSol.penalizedCost);
	} else{
		std::cout << "NO-VALID ";
	}

	if (getBestInfeasible() != NULL) {
		std::printf("%.3f", getBestInfeasible()->myCostSol.penalizedCost);
	} else {
		std::cout << "NO-INVALID";
	}

	std::printf(" | Avg %.3f %.3f", getAverageCostFeasible(), getAverageCostInfeasible());
	std::printf(" | Div %.3f %.3f", getDiversity(feasibleSubpopulation), getDiversity(infeasibleSubpopulation));
	std::printf(" | Val %.3f %.3f", fractionFeasibleLoad(), fractionFeasibleDuration());
	std::printf(" | Pen %.3f %.3f", params->penalityCapa, params->penalityLength);
	std::printf(" | Pop %zu %zu\n", feasibleSubpopulation.size(), infeasibleSubpopulation.size());
}

Population::Population(Params * params, Split * split, LocalSearch * localSearch, Mining * mining) : params(params), split(split), localSearch(localSearch), mining(mining)
{
	listFeasibilityLoad = std::list<bool>(100, true);
	listFeasibilityDuration = std::list<bool>(100, true);
	generatePopulation();
}

Population::~Population()
{
	for (int i = 0; i < (int)feasibleSubpopulation.size(); i++) delete feasibleSubpopulation[i];
	for (int i = 0; i < (int)infeasibleSubpopulation.size(); i++) delete infeasibleSubpopulation[i];
}