#ifndef POPULATION_H
#define POPULATION_H

#include "Individual.h"
#include "LocalSearch.h"
#include "Split.h"
#include "Mining.h"

/**
 * Author: Thibaut Vidal
 * 
 * It manages all functionalities that involves the population (set of individuals) of HGSADC. 
 * Functions for generating, diversifying and penalizing individuals are present in this class.
 * 
 */

typedef std::vector <Individual *> SubPopulation ;

class Population
{
   private:

   Params * params ;							// Problem parameters
   Split * split;								// Split algorithm
   LocalSearch * localSearch;					// Local search structure
   Mining * mining;								// Mining structure
   SubPopulation feasibleSubpopulation;				// Feasible subpopulation, kept ordered by increasing penalized cost
   SubPopulation infeasibleSubpopulation;				// Infeasible subpopulation, kept ordered by increasing penalized cost
   clock_t timeFoundBest ;						// Time when the best solution was found
   std::list <bool> listFeasibilityLoad ;		// Load feasibility of the last 100 individuals generated by LS
   std::list <bool> listFeasibilityDuration ;	// Duration feasibility of the last 100 individuals generated by LS

   // Checks if the fitness of indiv already exists in the subpopulation
   bool fitExists ( SubPopulation & pop, Individual * indiv ) ;

   // Inserts an individual in the population
   int placeIndividual (SubPopulation & pop, Individual * indiv);

   // Evaluates the biased fitness of all individuals in the population
   void evalBiasedFitnessPopulation(SubPopulation & pop);

   // Remove an individual
   void removeIndividual(SubPopulation & pop, int p);

   // Update distance between individuals
   void updateProximity(SubPopulation & pop, Individual * indiv);

   public:

   // Add an individual in the population (survivor selection is automatically triggered whenever the population reaches its maximum size)
   int addIndividual (Individual * indiv, bool updateFeasible);

   // Creates an initial population of individuals
   void generatePopulation();

   // Diversification
   void diversify ();

   // Adaptation of the penalty parameters
   void managePenalties();

   // Select an individal by binary tournament
   Individual * getBinaryTournament();

   // Select the best feasible individual
   Individual * getBestFeasible();

   // Select the best infeasible individual
   Individual * getBestInfeasible();

   // Select the worst individual in terms of biased fitness
   int getWorstBiasedFitness(SubPopulation & SubPopulation);

   // Returns the population diversity
   double getDiversity(SubPopulation & pop);

   // Returns the average solution value in the feasible subpopulation
   double getAverageCostFeasible ();

   // Returns the average solution value in the infeasible subpopulation
   double getAverageCostInfeasible ();

   // Compute the fraction of solutions which are feasible regarding load constraints
   double fractionFeasibleLoad();

   // Compute the fraction of solutions which are feasible regarding duration constraints
   double fractionFeasibleDuration();

   // Prints population state
   void printState(int NbIter);

   // Exports the best found solution in a file
   void exportBestSolution(std::string fileName);

   // Constructor
   Population(Params * params, Split * split, LocalSearch * localSearch, Mining * mining);

   // Destructor
   ~Population();
};

#endif
