#ifndef COMMAND_LINE_H
#define COMMAND_LINE_H

#include <iostream>
#include <string>

/**
 * Author: Thibaut Vidal
 * 
 * This class is responsible for validating all input information given by command line.
 * This information consists of instance path, instance type, output path, seed, time limit, number of vehicles, and PILS status. 
 * 
 */

class commandline
{
private:

	bool command_ok;				// Tells whether the line of command is valid
	int cpu_time;					// Allocated CPU time
	int seed;						// Random seed
	int type;						// Instance type for parsing
	int nbVeh;						// Number of vehicles
	int settingPILS;				// PILS setting: OFF=0, BEFORE=1, AFTER=2, BEFORE&AFTER=3
	std::string instance_name;		// Instance path
	std::string output_name;		// Output path

	// Setting the instance name
	void set_instance_name(std::string to_parse) { instance_name = to_parse; }

	// Setting the output name
	void set_output_name(std::string to_parse) { output_name = to_parse; }

	// Setting a default output name
	void set_default_output_name(std::string to_parse)
	{
		char caractere1 = '/';
		char caractere2 = '\\';

		int position = (int)to_parse.find_last_of(caractere1);
		int position2 = (int)to_parse.find_last_of(caractere2);
		if (position2 > position) position = position2;

		if (position != -1)
			output_name = to_parse.substr(0, position + 1) + "sol-" + to_parse.substr(position + 1, to_parse.length() - 1);
		else
			output_name = "sol-" + to_parse;
	}

public:

	// Constructor
	commandline(int argc, char* argv[])
	{
		command_ok = true;
		if (argc % 2 != 0 || argc > 14 || argc < 2)
		{
			std::cout << "----- NUMBER OF COMMANDLINE PARAMETERS IS NOT OK: " << argc << std::endl;
			command_ok = false;
		}
		else if (std::string(argv[1]) == "-?" || std::string(argv[1]) == "-help")
		{
			command_ok = false;
		}
		else
		{
			// Setting some default values
			set_instance_name(std::string(argv[1]));
			set_default_output_name(std::string(argv[1]));
			cpu_time = 0;
			seed = 0;
			type =  0;
			nbVeh = -1;
			settingPILS = 0;

			for (int i = 2; i < argc; i += 2)
			{
				if (std::string(argv[i]) == "-t")
					cpu_time = atoi(argv[i + 1]);
				else if (std::string(argv[i]) == "-sol")
					set_output_name(std::string(argv[i + 1]));
				else if (std::string(argv[i]) == "-seed")
					seed = atoi(argv[i + 1]);
				else if (std::string(argv[i]) == "-type")
					type = atoi(argv[i + 1]);
				else if (std::string(argv[i]) == "-veh")
					nbVeh = atoi(argv[i + 1]);
				else if (std::string(argv[i]) == "-pils")
					settingPILS = atoi(argv[i + 1]);
				else
				{
					std::cout << "----- NON RECOGNIZED ARGUMENT: " << std::string(argv[i]) << std::endl;
					command_ok = false;
				}
			}
		}
		if (!command_ok)
			display_help();
	}

	// Printing information message about correct commandline
	void display_help()
	{
		std::cout << std::endl;
		std::cout << "----------------------------------------- HGSADC algorithm -----------------------------------------------------" << std::endl;
		std::cout << "Usage: ./genvrp instancePath [-sol solPath] [-type instanceType] [-t myCPUtime] [-seed mySeed] [-veh nbVehicles]" << std::endl;
		std::cout << "[-sol solPath] sets the output solution path. Defaults to the instance file name prepended with sol-            " << std::endl;
		std::cout << "[-t myCPUtime] sets the CPU time limit in seconds. Defaults to t=360 seconds                                    " << std::endl;
		std::cout << "[-seed mySeed] sets a fixed seed. If mySeed=0, extracts the seed from the current time. Defaults to 0           " << std::endl;
		std::cout << "[-veh nbVehicles] sets the fleet size. Necessary when this information is not contained in the instance file    " << std::endl;
		std::cout << "[-type instanceType] sets the instance type                                                                     " << std::endl;
		std::cout << "[-pils status] allows PILS if status=1. Defaults to 0.                                                          " << std::endl;
		std::cout << "----------------------------------------------------------------------------------------------------------------" << std::endl;
		std::cout << std::endl;
	};

	// Getting path to the instance file
	std::string get_path_to_instance() { return instance_name; }

	// Getting path to the solution file
	std::string get_path_to_solution() { return output_name; }

	// Getting allocated CPU time
	int get_cpu_time() { return cpu_time; }

	// Getting instance type
	int get_type() { return type; }

	// Getting number of vehicles allowed
	int get_nbVeh() { return nbVeh; }

	// Getting the random seed
	int get_seed() { return seed; }

	// Getting pils value
	int get_settingPILS() {return settingPILS; }

	// Tests whether the commandline parameters are OK
	bool is_valid() { return command_ok; }
};
#endif
